//=============================================================================
// German MG42 3rd person attachment class.
//
// Author: Neil Johnson, Gearbox Software.
//=============================================================================
class AttachmentDEMG42_ extends InstantFireWeaponAttachment;



var Vector MuzzleOffset;
var int Flash;
var String	FirstPersonShellSoundName;
var Cue	FirstPersonShellSound;

//---------------------------------------------------------
//	ThirdPersonEffects
//---------------------------------------------------------
simulated event ThirdPersonEffects()
{
	local Vector MuzzleVec;

	if (FlashCount==0)
		return;

	// GBX:naj - MG42's have neither firing nor reload animations.  
	// Thus there are no calls here to Instigator.PlayFiring() or PlayReload()
	// as you might see with normal weapons.

	if ((MyAmmoTypeClass != None) && (FiringMode == WEAP_StateFiring))
	{
		MuzzleVec = GetMuzzleLocationWorld();

		if ( FiringMode == WEAP_StateFiring )
		{
			FlashMuzzleFlash( MuzzleVec );
			
			DoImpactEffects();

			if ((Instigator != None) && (Instigator.DrawType != DT_StaticMesh))
				EjectShell( GetChamberLocationWorld(), MyAmmoTypeClass.Default.AmmoShellClass );

			if( bUseTracers )
			{
				SendTracer( MuzzleVec, MyAmmoTypeClass.Default.BulletTracerClass );
			}
			if (gbxAmmo(Instigator.Weapon.Ammo[0]) != None)
				gbxAmmo(Instigator.Weapon.Ammo[0]).ApplyKick();
		}
	}
}

//---------------------------------------------------------
//	GetMuzzleLocationWorld
//---------------------------------------------------------
simulated function Vector GetMuzzleLocationWorld()
{
	local Vector X,Y,Z;
	GetAxes(Instigator.Weapon.Rotation, X,Y,Z);
	return Instigator.Weapon.Location + (MuzzleOffset.X*X) + (MuzzleOffset.Y*Y) + (MuzzleOffset.Z*Z);
}

//---------------------------------------------------------
//	InitFor
//---------------------------------------------------------
function InitFor(Inventory I)
{
	Instigator = I.Instigator;
}

//---------------------------------------------------------
//	FlashMuzzleFlash
//---------------------------------------------------------
simulated function FlashMuzzleFlash( Vector Location )
{
	if ( Instigator != None && Instigator.Weapon != None &&
		Instigator.Weapon.IsA( 'WeapDEMG42_' ) && Level.TimeSeconds - LastFlashTime > MinFlashInterval )
	{
		WeapDEMG42_( Instigator.Weapon ).FlashMuzzleFlash();
		LastFlashTime = Level.TimeSeconds;
	}
}

//---------------------------------------------------------
//	EjectShell
//---------------------------------------------------------
simulated function bool EjectShell( Vector Location, class<gbxWeaponShells> ShellClass )
{
	local bool	bEjected;
	
	if (	( Flash % 2 ) == 0 &&
			Instigator != None && Instigator.Weapon != None &&
			Instigator.Weapon.IsA( 'WeapDEMG42_' ) )
	{
		if ( WeapDEMG42_( Instigator.Weapon ).CanSpawnShellCasing() )
			bEjected = super.EjectShell( Location, ShellClass );
		
		if ( WeapDEMG42_( Instigator.Weapon ).ShouldPlayFirstPersonShellSound() )
		{
			if( FirstPersonShellSound == None && FirstPersonShellSoundName != "" )
			{
				FirstPersonShellSound = LoadSound( FirstPersonShellSoundName );
			}

			Instigator.PlayOwnedSoundCue( FirstPersonShellSound, SLOT_None,,,,, );
		}
	}

	Flash++;
	
	return bEjected;
}

//=========================================================
//	Default Properties
//=========================================================
defaultproperties
{
    LightType=LT_Pulse
    LightEffect=LE_NonIncidence
    LightPeriod=3
    LightBrightness=50
    LightHue=30
    LightSaturation=150
    LightRadius=4.0

	bUseTracers=true;
	TracerFrequency=3;

	RelativeLocation=(X=0.000000,Y=1.0000,Z=-0.3000)

	//MuzzleOffset=(X=63,Y=0,Z=15)
	MuzzleOffset=(X=55,Y=0,Z=0)

	FirstPersonShellSoundName="GEAR.SHELL"
	PawnAnimInfoClass=class'PawnAnimInfoFinalMG42_'

	MinFlashInterval=0.1
}
