//=============================================================================
// German MG42 fire mode class.
//
// Author: Neil Johnson, Gearbox Software.
//=============================================================================
class FireDEMG42_ extends InstantFire;



//---------------------------------------------------------
//	Variables
//---------------------------------------------------------
var Rotator		AimRot;		// Direction the weapon was aimed at.

//---------------------------------------------------------
//	PostNetBeginPlay
//---------------------------------------------------------
function PostNetBeginPlay()
{
	Super.PostNetBeginPlay();

	// GBX:naj - spawning the muzzle flash is required here to avoid it flashing the first time you mount the weapon.
	if ( ((FlashEmitter == None) || FlashEmitter.bDeleteMe) && FlashEmitterClass != None )
    {
		FlashEmitter = Spawn(FlashEmitterClass);
		gbxMuzzleFlash( FlashEmitter ).InitializeMuzzleFlash();
    }
}

//---------------------------------------------------------
//	PlayFiring
//---------------------------------------------------------
function PlayFiring()
{
	if ( Instigator.Weapon.IsA( 'WeapDEMG42_' ) )
	{
		WeapDEMG42_( Instigator.Weapon ).PlayFiringAnim();
	}

	ClientPlayForceFeedback(FireForce);
}

//---------------------------------------------------------
//	PlayReloading
//---------------------------------------------------------
simulated function PlayReloading()
{
	local bool bKeepFiring;

	bKeepFiring = bIsFiring;

	Weapon.ClientStopFire(0);
	Weapon.ReloadClip();

	if ( bKeepFiring )
		gbxWeapon( Weapon ).StartFiringWeapon();
}

simulated function bool ReadyToFire()
{
	return AllowFire();
}

//---------------------------------------------------------
//	AllowFire
//---------------------------------------------------------
simulated function bool AllowFire()
{
	if (Weapon.IsA('gbxWeapon') && Instigator.Controller.bIsHumanPlayer)
	{
		if (gbxWeapon(Weapon).bAimingAtFriendly)
			return false;
	}
	return Super.AllowFire();
	/*
	// Do the relatively-expensive muzzle check only on every third bullet.
	// (Handled in parent.)
	if ( ShotCount % 3 != 0 )
	{
		return true;
	}
    else if (Super.AllowFire() && Instigator.ApproveShot() )
	{
        return true;
	}
    else
    {
        //ClickTime = Level.TimeSeconds + 0.25;
        return false;
    }
	*/
}

//---------------------------------------------------------
//	GetMuzzleFlash
//---------------------------------------------------------
simulated function Emitter GetMuzzleFlash()
{
	local Vector X,Y,Z;
	local AttachmentDEMG42_ Attachment;

	GetAxes( Weapon.Rotation, X,Y,Z);

	if ( ((FlashEmitter == None) || FlashEmitter.bDeleteMe) && FlashEmitterClass != None )
    {
		Attachment = WeapDEMG42_( Weapon ).GetAttachment();
		if ( Attachment == None )
			return None;

        FlashEmitter = Spawn(FlashEmitterClass);
		FlashEmitter.SetLocation( Weapon.Location + Attachment.MuzzleOffset.X*X + Attachment.MuzzleOffset.Y*Y + Attachment.MuzzleOffset.Z*Z );
		FlashEmitter.SetRotation( Weapon.Rotation );
		FlashEmitter.SetBase( Weapon );		

		gbxMuzzleFlash( FlashEmitter ).InitializeMuzzleFlash();
    }

	FlashEmitter.Instigator = Self.Instigator;
	return FlashEmitter;
}

//---------------------------------------------------------
//	StartFiringWeapon
//---------------------------------------------------------
function StartFiringWeapon()
{
	if ( FlashEmitter != None && FlashEmitter.IsA( 'gbxMuzzleFlash' ) )
		gbxMuzzleFlash( FlashEmitter ).FireButtonPressed();
}

//---------------------------------------------------------
//	StopFiringWeapon
//---------------------------------------------------------
function StopFiringWeapon()
{
	if ( FlashEmitter != None && FlashEmitter.IsA( 'gbxMuzzleFlash' ) )
		gbxMuzzleFlash( FlashEmitter ).FireButtonReleased();
}

//---------------------------------------------------------
//	GetBestBurstLength
//---------------------------------------------------------
function GetBestBurstLength( ESuppressionLevel SuppressionLevel, out float MinLen, out float MaxLen )
{
	switch ( SuppressionLevel )
	{
	case SL_UNSUPPRESSED:
		MinLen = BurstLength_Low.Min;
		MaxLen = BurstLength_Low.Max;
		break;
	case SL_SUPPRESSED:
		MinLen = BurstLength_High.Min;
		MaxLen = BurstLength_High.Max;
		break;
	}
}

//=========================================================
//	Default Properties
//=========================================================
defaultproperties
{
    AmmoClass=class'AmmoDEMG42_'
    AmmoPerFire=1
    DamageType=class'DamageDEMG42_'
	Damage_Human_SP=(min=90,max=120)
	Damage_Human_MP=(min=90,max=120)
	Damage_AI_SP=(min=90,max=120)
	Damage_AI_MP=(min=90,max=120)
    bPawnRapidFireAnim=true
	Load=1

	FireAnim=Fire
    FireEndAnim=None
    FireLoopAnim=None
	ReloadAnim=None

	FlashEmitterClass=class'gbxEffects.MuzzleFlashMG42_'

    FireForce="MachineGunFire"   // jdf

	PreFireTime=0.0
	FireRateSP=0.06
	FireRateMP=0.06
    AIFireRateSP=0.06
	AIFireRateMP=0.06
    bModeExclusive=true

	SuppressionMultiplierSP=1.5

	TraceRange=15000

	bWaitForRelease=false

	ShakeOffsetMag=(X=1.0,Y=1.0,Z=1.0)
    ShakeOffsetRate=(X=1000.0,Y=1000.0,Z=1000.0)
    ShakeOffsetTime=2
    ShakeRotMag=(X=50.0,Y=50.0,Z=50.0)
    ShakeRotRate=(X=10000.0,Y=10000.0,Z=10000.0)
    ShakeRotTime=2

	BurstLength_Low=(Min=0.05,Max=1.5)
	BurstLength_High=(Min=0.2,Max=0.5)

	// This really sucks, but constants are supported here, so I'm just setting this to the value of the constant.
	FireType=1

	AimErrorMin_Human=450
	AimErrorMax_Human_Crouched=500	
	AimErrorMax_Human_Standing=900
	AimErrorMin_Human_Zoomed=150
	AimErrorMax_Human_Zoomed=450

	AimErrorMin_AI_SP=250
	AimErrorMax_AI_SP_Crouched=250
	AimMultiplier_AI_SP_Sighted=1.0

	ShotCountMod=3
}
