class FireDEMortar extends WeaponFire;

//---------------------------------------------------------
//	Variables
//---------------------------------------------------------
var gbxMortarShell			Shell;
var float					ShellDelay;
var Cue		MortarLaunchSound;
var Cue		MortarFlightSound;
var String	MortarLaunchSoundName;
var String	MortarFlightSoundName;

function PostBeginPlay() 
{
	Super.PostBeginPlay();

	if ( MortarFlightSoundName != "" )
	{
		MortarFlightSound = LoadSound(MortarFlightSoundName);
	}

	if ( MortarLaunchSoundName != "" )
	{
		MortarLaunchSound = LoadSound(MortarLaunchSoundName);
	}
}

//---------------------------------------------------------
//	PlayFiring
//---------------------------------------------------------
simulated function PlayFiring()
{
	Super.PlayFiring();

	WeapDEMortar( Weapon ).PlayFiring();
}

//---------------------------------------------------------
//	DoFireEffect
//---------------------------------------------------------
event DoFireEffect() {}

//---------------------------------------------------------
//	GetTargetLocMarching
//---------------------------------------------------------
function Vector GetTargetLocMarching( int TotalShots, int NumShotsAlreadyFired, float StartingOffset )
{
	local Vector OffsetDir;
	local float SingleShotOffset;

	OffsetDir = Normal( Weapon.Location - WeapDEMortar( Weapon ).TargetLocation );
	SingleShotOffset = StartingOffset / TotalShots;

	return WeapDEMortar( Weapon ).TargetLocation + SingleShotOffset * ( TotalShots - NumShotsAlreadyFired ) * OffsetDir;
}

//---------------------------------------------------------
//	GetTargetLocEffect
//---------------------------------------------------------
function Vector GetTargetLocEffect( float Offset )
{
	local Rotator OffsetRot;

	OffsetRot.Yaw = FRand() * 65535;

	return WeapDEMortar( Weapon ).TargetLocation + Offset * Vector( OffsetRot );
}

//---------------------------------------------------------
//	GetTargetLocBracketing
//---------------------------------------------------------
function Vector GetTargetLocBracketing( int TotalShots, int NumShotsAlreadyFired, float StartingOffset )
{
	local bool bRightSide;
	local Vector OffsetDir;
	local float SingleShotOffset;
	
	if ( NumShotsAlreadyFired % 2 == 0 )
		bRightSide = true;

	OffsetDir = Normal( WeapDEMortar( Weapon ).TargetLocation - Weapon.Location ) cross Vect(0,0,1);
	if ( bRightSide )
		OffsetDir *= -1;

	SingleShotOffset = StartingOffset / TotalShots;
	return WeapDEMortar( Weapon ).TargetLocation + SingleShotOffset * ( TotalShots - NumShotsAlreadyFired ) * OffsetDir;
}

//---------------------------------------------------------
//	GetTargetLocBounded
//---------------------------------------------------------
function Vector GetTargetLocBounded( float Offset )
{
	local Vector NewLoc;
	local Rotator OffsetDir;
	
	NewLoc = WeapDEMortar( Weapon ).TargetLocation;
	OffsetDir.Yaw = FRand() * 65535;
	NewLoc += ( Offset * FRand() * Vector( OffsetDir ) );

	return NewLoc;
}

//---------------------------------------------------------
//	LaunchMortar
//---------------------------------------------------------
function LaunchMortar( EMortarFire FireType, int TotalShots, int NumShotsAlreadyFired, float StartingOffset )
{
	local Vector X,Y,Z, OffsetLoc;
	local Vector TargetLocation;
	local Rotator ShellRot, TempRot;

	if ( WeapDEMortar( Weapon ).Target == None )
		return;

	GetAxes( Weapon.Rotation, X,Y,Z );

	switch ( FireType )
	{
		case MortarFire_Marching_Random:
		case MortarFire_Marching:
			TargetLocation = GetTargetLocMarching( TotalShots, NumShotsAlreadyFired, StartingOffset );
			break;
		case MortarFire_Effect:
			TargetLocation = GetTargetLocEffect( StartingOffset );
			break;
		case MortarFire_Bracketing:
			TargetLocation = GetTargetLocBracketing( TotalShots, NumShotsAlreadyFired, StartingOffset );
			break;
		case MortarFire_Bounded:
			TargetLocation = GetTargetLocBounded( StartingOffset );
			break;
		default:
			return;
	}

	ShellRot = WeapDEMortar( Weapon ).MuzzleRotation;
	TempRot = Rotator( TargetLocation - Weapon.Location );
	ShellRot.Yaw = TempRot.Yaw;

	PlayOwnedSoundCue( MortarLaunchSound, SLOT_None );
	Shell = Spawn(class'gbxMortarShell',self,, WeapDEMortar( Weapon ).MuzzleLocation, ShellRot );

	// To make the Marching less linear, we randomize the target location a bit,.
	if ( FireType == MortarFire_Marching_Random )
	{	
		OffsetLoc.x = Rand(800);
		OffsetLoc.y = Rand(500);
		Shell.Launch( TargetLocation+OffsetLoc, ShellDelay );
	}
	else
		Shell.Launch( TargetLocation, ShellDelay );

	Shell.MortarFlightSound = MortarFlightSound;
}

//---------------------------------------------------------
//	PlayReloading
//---------------------------------------------------------
simulated function PlayReloading()
{
}

//=========================================================
//	Default Properties
//=========================================================
defaultproperties
{
	AmmoClass=class'AmmoDEMortar'
	AIFireRateSP=0.1
	AIFireRateMP=0.1
	ShellDelay=3.5
	AmmoPerFire=1
	MortarLaunchSoundName="W_MORTAR.Fire"
	MortarFlightSoundName="W_MORTAR.INCOMING"	
}