class FireSubMachineGun extends InstantFire;


// loops for things like machine guns
var String FireLoopSoundName;
var String StereoFireLoopSoundName;
var String FireEndLoopSoundName;
var String StereoFireEndLoopSoundName;

// sounds
var() Sound FireLoopSound;

// cues for the loops
var Cue FireLoopSoundCue;
var Cue StereoFireLoopSoundCue;
var Cue FireEndLoopSoundCue;
var Cue StereoFireEndLoopSoundCue;

var float MinFireLoopSustainTime;
var float FireLoopSustainEndTime;
var bool  bCheckForEndSustainLoop;
var float fAmbientSoundFadeOutTime;

// From FireUSThompson.uc
var bool	bWeaponFiring;
var float	LastFireTime;
var float	ClickTime;

simulated function PostBeginPlay()
{
	// Set the sound radius for the ThirdPersonActor based on cue max distance

	Super.PostBeginPlay();

	// Loops
	if ( FireLoopSoundName != "" )
	{
		FireLoopSoundCue = LoadSound( FireLoopSoundName );
		FireLoopSound = Sound( DynamicLoadObject( FireLoopSoundName, class'Sound' ) );
	}

	if ( StereoFireLoopSoundName != "" )
	{
		StereoFireLoopSoundCue = LoadSound( StereoFireLoopSoundName );
	}
		

	if ( FireEndLoopSoundName != "" )
		FireEndLoopSoundCue = LoadSound( FireEndLoopSoundName );
    
	if ( StereoFireEndLoopSoundName != "" )
		StereoFireEndLoopSoundCue = LoadSound( StereoFireEndLoopSoundName );

	if ( FireLoopSoundCue != None )
	{
		if ( Weapon.ThirdPersonActor != None )
		{
			Weapon.ThirdPersonActor.SoundRadius = FireLoopSoundCue.MaximumDistance;
		}
	}
	
	log(self$ " FireSubMachineGun::PostBeginPlay() FireLoopSoundCue: "$FireLoopSoundCue$" FireLoopSound: " $FireLoopSound$ " Weapon: " $Weapon$ " Weapon.ThirdPersonActor: " $Weapon.ThirdPersonActor, 'DevSound');
}

// If the fire mode is being destroyed, stop any and all sounds immediatley
simulated function Destroyed()
{
    log(self$ " FireSubMachineGun::Destroyed()", 'DevSound');
    
    Super.Destroyed();
        
    // Make sure any looping sounds stop
	FireLoopSustainEndTime = 0.0;
	bCheckForEndSustainLoop = false;

	log( self$ " FireSubMachineGun::Destroyed() Weapon: "$Weapon, 'DevSound');
	
	if ( FireSoundHandle != 0 )
	{
		log(self$ " FireSubMachineGun::Destroyed() calling StopSound()!", 'DevSound');   
		StopSound( FireSoundHandle );
	}

	log(self$ " FireSubMachineGun::Destroyed() setting AmbientSound to None!", 'DevSound');   
	PlayAmbientSound(None);

    Super.Destroyed();
}

// Machine gun weapons define all their ModeDoFire() code in their class.
// This is why you won't see any calls to Super.ModeDoFire().
event ModeDoFire()
{
	local float AppliedFireRate;	
	local bool isAutomatic;
	
	LastFireTime = Level.TimeSeconds;

	if( !Instigator.Controller.AllowFire() || !AllowFire())
	{
		Weapon.InvalidateBurst();
		Weapon.StopFire(0);
		return;
	}

	if ( !Weapon.HasAmmo() )
	{
		return;
	}
	else if (Weapon.NeedsToReload() && !Weapon.bDelayReloading)
	{
		if ( !Weapon.AlwaysAutoReload() && !(Instigator.Controller.ReloadAfterWeaponFire()))
		{
			PlayDryFire();
                        return;
		}
		else if ((Weapon.ClientState != WS_Reloading) && (Instigator.Controller.IsA('gbxAIController') || Weapon.AlwaysAutoReload()))
		{
			PlayReloading();
		}
		else
		{
			if ( Weapon.IsAutomatic() && FireSoundHandle != 0 )
			{
				Weapon.ClientStopFire(0);
			}
		}
		return;
	}

    if (MaxHoldTime > 0.0)
        HoldTime = FMin(HoldTime, MaxHoldTime);

    // server
    if (Weapon.Role == ROLE_Authority)
    {
		Load = AmmoPerFire;

        Weapon.ConsumeAmmo(ThisModeNum, Load);

        DoFireEffect();

        if (Weapon == None || Instigator == None)
		{
            return;
		}
    }
	else
	{
		ClientDoFireEffect();
	}

	isAutomatic = Weapon.IsAutomatic();
	//IFiring = IsFiring();
	
    // Reset sustain flag	
    bCheckForEndSustainLoop = false;

    // client
    if (Instigator.IsLocallyControlled())
    {
		PlayFiring();
        
		if ( Instigator.Controller == Level.GetLocalPlayerController() )
		{
			//log( self$ " ModeDoFire() Instigator.Controller == Level.GetLocalPlayerController()!  About to play sound locally!", 'MP' );

			if ( FireSoundHandle == 0 )  
			{
				if ( StereoFireLoopSoundCue != None /*&& (Instigator.Controller == Level.GetLocalPlayerController())*/ )
				{
					FireSoundHandle = Weapon.PlayLocalSoundCue(StereoFireLoopSoundCue, SLOT_None, false);	
					FireLoopSustainEndTime = Level.TimeSeconds + MinFireLoopSustainTime;
					log(self$ "ModeDoFire() Setting SustainEndTime: " $FireLoopSustainEndTime, 'DevSound');  
				}
				else
				{
					log( self$ " StereoFireLoopSoundCue is NULL so MONO first person firing sound will play!  Baaaad!", 'DevSound' );				
					FireSoundHandle = Weapon.PlayOwnedSoundCue(FireLoopSoundCue, SLOT_None, TransientSoundVolume,,,,true);	
				}
			}
		}
    }

	Weapon.IncrementFlashCount(FireType);

	if (Instigator.Controller.IsA('AIController'))
	{
		if ( !Level.m_bMPStyleFatigue && Level.NetMode == NM_Standalone )
			AppliedFireRate = AIFireRateSP;
		else
			AppliedFireRate = AIFireRateMP;

		// set the next firing time. must be careful here so client and server do not get out of sync
		if (bFireOnRelease)
		{
			if (bIsFiring)
			{				
				NextFireTime += ( MaxHoldTime + AppliedFireRate*Weapon.FireRateAtten + GetRandomFireRateChange() );
			}
			else
			{
				NextFireTime = Level.TimeSeconds + AppliedFireRate*Weapon.FireRateAtten + GetRandomFireRateChange();
			}
		}
		else
		{
			NextFireTime += ( AppliedFireRate*Weapon.FireRateAtten + GetRandomFireRateChange() );
			NextFireTime = FMax(NextFireTime, Level.TimeSeconds);			
		}
	}
	else
	{
		if ( !Level.m_bMPStyleFatigue && Level.NetMode == NM_Standalone )
			AppliedFireRate = FireRateSP;
		else
			AppliedFireRate = FireRateMP;

		// set the next firing time. must be careful here so client and server do not get out of sync
		if (bFireOnRelease)
		{
			if (bIsFiring)
				NextFireTime += MaxHoldTime + AppliedFireRate*Weapon.FireRateAtten;
			else
				NextFireTime = Level.TimeSeconds + AppliedFireRate*Weapon.FireRateAtten;
		}
		else
		{
			NextFireTime += AppliedFireRate*Weapon.FireRateAtten;
			NextFireTime = FMax(NextFireTime, Level.TimeSeconds);
		}
	}

    HoldTime = 0;

    if (Instigator.PendingWeapon != Weapon && Instigator.PendingWeapon != None)
    {
        if (Weapon.PutDown())
            bIsFiring = false;

        log( self$ " ModeDoFire() setting bCheckForEndSustainLoop to true!", 'DevSound' );
        bCheckForEndSustainLoop = true;
    }

	bWaitForAnim = true;

    // Change fire states
	if ( !bWeaponFiring )
	{
		StartFiring();
	}
}

event ModeTick(float dt)
{
	local bool bNeedsToReload;
	Super.ModeTick(dt);

	bNeedsToReload = Weapon.NeedsToReload();
	
	log( self$ " ModeTick() bNeedsToReload: " $bNeedsToReload$ " bIsFiring: " $bIsFiring$ " bWeaponFiring: " $bWeaponFiring$ " Level.TimeSeconds: " $Level.TimeSeconds$ " FireLoopSustainEndTime: " $FireLoopSustainEndTime$ " FireSoundHandle: " $FireSoundHandle, 'DevSound' );

	if ( !bIsFiring && bWeaponFiring ||
		Weapon.NeedsToReload() && bIsFiring ||
		bCheckForEndSustainLoop )
	{
		bNeedsToReload = Weapon.NeedsToReload();
		log(self$ " ModeTick() bIsFiring: "$bIsFiring$" bWeaponFiring: " $bWeaponFiring$" bNeedsToReload: " $bNeedsToReload$ " bCheckForEndSustainLoop = true", 'DevSound');
		
		if ( Level.TimeSeconds > FireLoopSustainEndTime )
		{
			log(self$ " ModeTick() Level.TimeSeconds IS greater then FireLoopSustainEndTime, stopping fireloop!", 'DevSound');

			StopFiring();		// will handle calling stop on sound handle and removing ambient sound
		}
		else
		{
			log(self$ " ModeTick() Level.TimeSeconds is NOT greater then FireLoopSustainEndTime", 'DevSound');
		}
		
	}
}

// PSV: 11/29/2004 Added fire loop support for machine guns
simulated function PlayAmbientSound(Sound aSound)
{
	if (Weapon.ThirdPersonActor == None) 
	{
        log(self$ " FireSubMachineGun::PlayAmbientSound() Weapon: " $Weapon$ " Weapon.ThirdPersonActor == NULL!!!  BAD!!", 'DevSound');
		return;		
	}

	// Because users can fire and release super fast, we need to sustain the fireloop long enough 
	// so that at least one fire "bang" sound is played
    if ( aSound == None )
	{		
		log(self$ "PlayAmbientSound() aSound: "$aSound$ " Level.TimeSeconds: "$Level.TimeSeconds$ " IS greater then SustainEndTime: "$FireLoopSustainEndTime$ " , setting AmbientSound to "$aSound, 'DevSound');
		
        if ( Level.TimeSeconds > FireLoopSustainEndTime )
		{
			if  ( Weapon.ThirdPersonActor.AmbientSound != None )
			{
				// Set a fade-out time for the fire looping sound
				Weapon.ThirdPersonActor.m_fAmbientSoundFadeOutTime = fAmbientSoundFadeOutTime;
				Weapon.ThirdPersonActor.AmbientSound = aSound;
			}

			FireLoopSustainEndTime = 0.0;
			bCheckForEndSustainLoop = false;
		}
        else
	    {
			log(self$ "PlayAmbientSound() Level.TimeSeconds: " $Level.TimeSeconds$ " is NOT greater then SustainEndTime: " $FireLoopSustainEndTime, 'DevSound');

			// Set flag so we can check for end in Tick()
			bCheckForEndSustainLoop = true;
	    }
    }
	else
	{
		FireLoopSustainEndTime = Level.TimeSeconds + MinFireLoopSustainTime;
		log(self$ "PlayAmbientSound() Setting SustainEndTime: " $FireLoopSustainEndTime, 'DevSound');   

		Weapon.ThirdPersonActor.AmbientSound = aSound;
		bCheckForEndSustainLoop = false;
	}
}

event ModeHoldFire()
{
	Super.ModeHoldFire();
}

// Called locally to stop first person fire loop and play one off shot
function StopFiring()
{
	Super.StopFiring();

	log(self$ " FireSubMachineGun::StopFiring() FireSoundHandle: " $FireSoundHandle, 'DevSound' );

	if ( Weapon.IsAutomatic() )
	{
		// Because users can fire and release super fast, we need to sustain the fireloop long enough 
		// so that at least one fire "bang" sound is played
		if ( Level.TimeSeconds > FireLoopSustainEndTime )
		{		
			log(self$ " FireSubMachineGun::StopFiring() Level.TimeSeconds: "$Level.TimeSeconds$ " IS greater then SustainEndTime: "$FireLoopSustainEndTime$ " , setting AmbientSound to None!", 'DevSound');
		
			FireLoopSustainEndTime = 0.0;
			bCheckForEndSustainLoop = false;

			if (Instigator.IsLocallyControlled())
			{
				if ( FireSoundHandle != 0 )
				{
					log( self$ " FireSubMachineGun::StopFiring() LOCALLY CONTROLLED!", 'DevSound');
					if ( StereoFireEndLoopSoundCue != None && (Instigator.Controller == Level.GetLocalPlayerController()) )
					{
						log( self$ " FireSubMachineGun::StopFiring() Playing stereo one off shot", 'DevSound');	
						Weapon.PlayLocalSoundCue(StereoFireEndLoopSoundCue, SLOT_None, false);	
					}
					else
					{
						log( self$ " FireSubMachineGun::StopFiring() Missing stereo cue?!  Playing mono one off shot", 'DevSound');	
						Weapon.PlayLocalSoundCue(FireEndLoopSoundCue, SLOT_None, false);	
					}
				}
			}
			if ( Weapon.ThirdPersonActor.AmbientSound != None )
			{
				// This shouldn't be the case for the local guy!!!!!!!! BAD!!!!
				log( self$ " FireSubMachineGun::StopFiring() Playing mono one off shot", 'DevSound');
				Weapon.PlayRemoteSoundCue(FireEndLoopSoundCue, SLOT_None, true, false);
			}
			else
			{
				log(self$ " FireSubMachineGun::StopFiring() Not playing one off end shot because Weapon.ThirdPersonActor.AmbientSound is NULL!");
			}
			
			// Was the stereo loop playing?
			if ( FireSoundHandle != 0 )					
				StopSound( FireSoundHandle );

			// Stop the mono loop
			PlayAmbientSound(None);

			FireSoundHandle = 0;
		}
		else
		{
			log(self$ " FireSubMachineGun::StopFiring() Level.TimeSeconds: " $Level.TimeSeconds$ " is NOT greater then SustainEndTime: " $FireLoopSustainEndTime, 'DevSound');
		
			// Set flag so we can check for end in Tick()
			bCheckForEndSustainLoop = true;
		}
	}
}

auto state Idle
{
    function BeginState()
    {
        PlayAmbientSound(None);
    }

    function EndState()
    {
    }

    function StartFiring()
    {
		bWeaponFiring = true;
        GotoState('FireLoop');
    }
}

state FireLoop
{
    function BeginState()
    {
		log(self$ " FireSubMachineGun::FireLoop::BeginState() Calling PlayAmbientSound("$FireLoopSound$")", 'DevSound');
		PlayAmbientSound(FireLoopSound);
    }

    function StopFiring()
    {
		Global.StopFiring();
	
		bWeaponFiring = false;
		GotoState('Idle');
    }

    function EndState()
    {
    }
}

defaultproperties
{
	MinFireLoopSustainTime = 0.0
	bCheckForEndSustainLoop = false
	FireLoopSustainEndTime = 0.0
	fAmbientSoundFadeOutTime = 0.1
}