class FireUSBazooka extends WeaponFire;

var class<DamageType>	DamageType;
var float				TraceRange;
var float				Momentum;
var float				HitEffectProb;
var float				FireTraceBackStep; // Distance to subtract from the start location of a fire trace

// Burst-count min/max values to use when firing; categorized by suppression level (low, med., high).
var Range Burst_High_SP;
var Range Burst_High_MP;
var Range Burst_Low_SP;
var Range Burst_Low_MP;
var Range Burst_Op_SP;
var Range Burst_Op_MP;

var Range BurstLength_High;
//var Range BurstLength_Med;
var Range BurstLength_Low;

var float				m_flRandomBurstRange;
var ProjectileUSBazooka	FiredRocket;

const AMMO_INVENTORY_SHOW_TIME = 3.0;

simulated function PostNetBeginPlay()
{
	Super.PostNetBeginPlay();
}

event ModeDoFire()
{
	Super.ModeDoFire();

	if ( Weapon.IsA('gbxWeapon') && gbxWeapon(Weapon).m_fZoomed)
	{
		bWaitForAnim = false;
	}
}

function PlayFiring()
{
	Super.PlayFiring();

	//SJP: Kick the view
	if ( Instigator.Weapon.Ammo[0] != None )
	{
		if( gbxAmmo(Instigator.Weapon.Ammo[0]) != None )
		{
			//gbxAmmo(Instigator.Weapon.Ammo[0]).ApplyKick();
			//gbxWeaponAttachment(gbxWeapon(Instigator.Weapon).ThirdPersonActor).EjectShell( gbxAmmo(Instigator.Weapon.Ammo[0]).AmmoShellClass );
		}

		if (Weapon.ClipAmt <= 0)
		{
			if (gbxWeapon(Weapon) != None)
			{
				gbxWeapon(Weapon).flAmmoFadeTime = AMMO_INVENTORY_SHOW_TIME;
				gbxWeapon(Weapon).flLastAmmoFadeTime = Level.TimeSeconds;
			}
		}

		gbxWeapon(Weapon).flLastFireTime = Level.TimeSeconds;
	}
	Weapon.AnimBlendToAlpha( 1, 0.0, 0.0 );	// Reset the breathing animation so it doesn't override the fire animation
	if (gbxWeapon(Weapon) != None)
	{
		gbxWeapon(Weapon).m_fResetBreathe = true;
	}
	
	gbxWeapon(Weapon).m_fPlayingFire = true;
}

function DoFireEffect()
{
    local Vector StartRocket;
    local Rotator Aim;
	local Vector HitLocation, HitNormal;
	local Actor HitActor;
	local float lookdist;

    Instigator.MakeNoise(1.0);

	if (!Level.Game.IsA('WargameMultiplayer'))
	{
		StartRocket = Weapon.GetMuzzleLocationWorld();				
	}
	else
	{
		if (Instigator.IsHumanControlled())
		{
			StartRocket = Instigator.Location + Instigator.EyePosition();
			
		}
		else
		{
			StartRocket = Weapon.GetMuzzleLocationWorld();
		}
	}	

	// Get the modified aim rotator based on the weapon aim error (as well as any other error values along the way).
	Aim = AdjustAimNew(StartRocket);

	if (gbxPawn(Instigator) != None)
		gbxPawn(Instigator).LastAimRotator = Aim;

	if (FiredRocket != None)
	{
		FiredRocket.Destroy();
	}
	
	FiredRocket = Spawn( class'ProjectileUSBazooka', Instigator,, StartRocket, Aim );
	
	if ((FiredRocket != None) && Weapon.IsA('gbxWeapon'))
	{
		// Use safety launch to make sure the rocket spawns and travels effectively.  However, if the player is actually aiming at a point
		// Closer than the safety launch distance, bring that distance down so the rocket will blow up in his face.
		HitActor = Trace( HitLocation, HitNormal, StartRocket + ( Vector( Aim ) * ( FiredRocket.SafeLaunchDist + 26.0 ) ), StartRocket - ( Vector( Aim ) * 26.0 ), true );

		lookdist = VSize( StartRocket - HitLocation ) - 26.0;

		if( HitLocation == vect(0,0,0) || lookdist > FiredRocket.SafeLaunchDist )
		{
			FiredRocket.UseSafetyLaunch();
		}
		else
		{
			FiredRocket.SetCollision( true, false, false );
			FiredRocket.bCollideWorld = true;
			FiredRocket.HitWall( HitLocation, HitNormal, HitActor );
		}

		FiredRocket.m_aGuideTarget = gbxWeapon(Weapon).m_aGuideTarget;
	}

	ClientDoFireEffect();
}

simulated function ClientDoFireEffect()
{
	if ( Instigator.IsHumanControlled() && Instigator.IsA( 'gbxPawn' ) )
		Instigator.Controller.JogFatigue( gbxPawn( Instigator ).FatigueJog_PlayerFiredShot );	
}

simulated function PlayReloading()
{
	local gbxPlayerController gbxPC;

	if (Weapon.ClipAmt >= Weapon.AmmoClipSize)
		return;

	if (Instigator.IsA('Mind') && Instigator.IsMoving() && Instigator.bIsCrouched)
		return;
		
	gbxWeapon(Weapon).m_fAIWeaponWaitingToFire = false;
	gbxWeapon(Weapon).flAmmoFadeTime = AMMO_INVENTORY_SHOW_TIME;
	gbxWeapon(Weapon).flLastAmmoFadeTime = Level.TimeSeconds;

	gbxPC = gbxPlayerController(Instigator.Controller);

	if( gbxWeapon(Weapon).m_fZoomed && gbxPC != None )
	{
		gbxPC.PendingDelayedAction = ReloadAfterZoomOut;
		gbxWeapon(Weapon).ForceUnzoom();
		//gbxPC.ChangeView(0);
	}
	else
	{
		Super.PlayReloading();
	}
}

function FlashMuzzleFlash()
{
	local gbxMuzzleFlash MuzFlash;

	MuzFlash = gbxMuzzleFlash(GetMuzzleFlash());
	MuzFlash.SetLocation(Weapon.Location);
	MuzFlash.Flash();
}

// GEARBOX: SJP - Look at RenderOverlays in gbxWeapon
// For first-person muzzle flashes only
function DrawMuzzleFlash(Canvas Canvas)
{
}

simulated function bool ReadyToFire()
{
	return AllowFire();
}

simulated function bool AllowFire()
{
	if( Super.AllowFire() )
	{
		if (Pawn(Weapon.Owner) != None)
		{
			if (Pawn(Weapon.Owner).bIsSprinting)
			{
				Pawn(Weapon.Owner).SetSprinting( false );
			}

			if (Weapon.IsA('gbxWeapon') && Pawn(Weapon.Owner).Controller.bIsHumanPlayer)
			{
				return !(gbxWeapon(Weapon).bAimingAtFriendly);
			}
		}
		
		return true;
	}

	return false;
}


function Rotator GetLastAimRotator()
{
	if (gbxPawn(Instigator) != None)
		return gbxPawn(Instigator).LastAimRotator;
	return Rot(0,0,0);
}

function GetBestBurstCount( ESuppressionLevel SuppressionLevel, out int MinShots, out int MaxShots )
{
	MinShots = 1;
	MaxShots = 1;
}

function GetOpBurstCount( out int MinShots, out int MaxShots )
{
	MinShots = 1;
	MaxShots = 1;
}

simulated function Tick(float DeltaTime)
{
	Super.Tick(DeltaTime);

	if (Weapon == None)
		return;
	
//	if (!Weapon.bDelayReloading && (Weapon.Ammo[0].AmmoAmount > 0) && (Weapon.ClipAmt <= 0) && (Weapon.ClientState != WS_Reloading) && !gbxWeapon(Weapon).m_fPlayingFire)
//	{
//		PlayReloading();
//	}
}

defaultproperties
{
    AmmoClass=class'AmmoUSBazooka'
    AmmoPerFire=1
    DamageType=class'DamageUSBazooka'
    bPawnRapidFireAnim=true
	Load=1

    FireAnim=fire
    FireEndAnim=None
    FireLoopAnim=None
    FireAnimRate=1.0
	ReloadAnim=reload
	ReloadAnimRate=1.0

    FlashEmitterClass=class'gbxEffects.MuzzleFlashBazookaView'

	//FireSound=Sound'WeaponSounds.AssaultRifle.AssaultRifleFire'
    FireForce="AssaultRifleFire"   // jdf

	PreFireTime=0.0
	FireRateSP=0.01
	FireRateMP=0.01
	AIFireRateSP=0.5
	AIFireRateMP=0.5
	AIFireRateVarianceSP=0.25		// Up to a 1/4 second difference in either direction.
	AIFireRateVarianceMP=0.25		// Up to a 1/4 second difference in either direction.
//    FireRate=0.5
    //FireRate = 0.25
    bModeExclusive=true

    BotRefireRate=0.5
    //AimErrorAI_SP=(Min=300,Max=300)	// Singleplayer: US weapons do not change with difficulty settings.
	//AimErrorAI_MP=(Min=300,Max=300)
    //AimErrorHuman_SP=300
	//AimErrorHuman_MP=300

    ShakeOffsetMag=(X=0.0,Y=0.0,Z=0.0)
    ShakeOffsetRate=(X=1000.0,Y=1000.0,Z=1000.0)
    ShakeOffsetTime=2
    ShakeRotMag=(X=10.0,Y=10.0,Z=10.0)
    ShakeRotRate=(X=10000.0,Y=10000.0,Z=10000.0)
    ShakeRotTime=2

    TweenTime = 0.0

	bWaitForRelease=true

	// Burst count levels.
	Burst_High_SP=(Min=1,Max=3)
	Burst_High_MP=(Min=1,Max=3)
	Burst_Low_SP=(Min=3,Max=5)
	Burst_Low_MP=(Min=3,Max=5)
	Burst_Op_SP=(Min=8,Max=8)
	Burst_Op_MP=(Min=8,Max=8)

	// This really sucks, but constants are supported here, so I'm just setting this to the value of the constant.
	FireType=1

	FireSoundName="W_USBAZOOKA.FIRE"
	StereoFireSoundName="W_USBAZOOKA.FIRE_ST"
}

