class InstantFire extends WeaponFire
	native;

cpptext
{
	virtual FLOAT MaxRange();
}

var class<DamageType>	DamageType;
var int					TracesPerFire;
var Range				Damage_Human_SP;
var Range				Damage_Human_MP;
var Range				Damage_AI_SP;
var Range				Damage_AI_MP;
var float				TraceRange;
var float				Momentum;
var float				HitEffectProb;
var float				FireTraceBackStep; // Distance to subtract from the start location of a fire trace

// Burst-count min/max values to use when firing; categorized by suppression level (low, med., high).
var Range Burst_High_SP;
var Range Burst_High_MP;
var Range Burst_Low_SP;
var Range Burst_Low_MP;
var Range Burst_Op_SP;
var Range Burst_Op_MP;

var Range BurstLength_High;		// GBX:naj - Deprecate this?
var Range BurstLength_Low;		// GBX:naj - Deprecate this?

var float				m_flRandomBurstRange;

const AMMO_INVENTORY_SHOW_TIME = 3.0;


function float MaxRange()
{
	return TraceRange;
}

event ModeDoFire()
{
	Super.ModeDoFire();

	if (Weapon.ClipAmt <= 0 && !(Instigator.Controller.ReloadAfterWeaponFire()))
	{
		Instigator.ReceiveLocalizedMessage(class'gbxUseKeyHUDMessage', EContextUseType.ReloadWeaponButton);  // display the USE message
	}
	
	if ( Weapon.IsA('gbxWeapon') && gbxWeapon(Weapon).m_fZoomed)
	{
		bWaitForAnim = false;
	}
}

function PlayFiring()
{
	Super.PlayFiring();

	//SJP: Kick the view
	if ( Instigator.Weapon.Ammo[0] != None )
	{
		if( gbxAmmo(Instigator.Weapon.Ammo[0]) != None )
		{
			//gbxAmmo(Instigator.Weapon.Ammo[0]).ApplyKick();
			//gbxWeaponAttachment(gbxWeapon(Instigator.Weapon).ThirdPersonActor).EjectShell( gbxAmmo(Instigator.Weapon.Ammo[0]).AmmoShellClass );
		}

		if (Weapon.ClipAmt <= 0)
		{
			if (gbxWeapon(Weapon) != None)
			{
				gbxWeapon(Weapon).flAmmoFadeTime = AMMO_INVENTORY_SHOW_TIME;
				gbxWeapon(Weapon).flLastAmmoFadeTime = Level.TimeSeconds;
			}
		}

		gbxWeapon(Weapon).flLastFireTime = Level.TimeSeconds;

		if (gbxWeapon(Weapon).m_nAIWeaponBurstCount > 0)
		{
			NextFireTime += (FRand() * m_flRandomBurstRange);
		}
	}
	Weapon.AnimBlendToAlpha( 1, 0.0, 0.0 );	// Reset the breathing animation so it doesn't override the fire animation
	if (gbxWeapon(Weapon) != None)
	{
		gbxWeapon(Weapon).m_fResetBreathe = true;
	}
	
	gbxWeapon(Weapon).m_fPlayingFire = true;
}

function DoFireEffect()
{
	Instigator.MakeNoise(1.0);

	ClientDoFireEffect();
}

function DoBFG()
{
	local gbxPawn aGbxP;
	local Vector Momentum;

	if ( Instigator == None || Instigator.Controller == None )
		return;

	foreach dynamicactors( class'gbxPawn', aGbxP )
	{
		if ( aGbxP.PawnTeam == Instigator.PawnTeam )
			continue;

		if ( Normal( aGbxP.Location - Instigator.Location ) dot Vector( Instigator.Controller.Rotation ) > 0.7071 &&
			VSize( aGbxP.Location - Instigator.Location ) <= 20000.0 )
		{
			Momentum = 5000.0 * Normal( aGbxP.Location - Instigator.Location );
			Momentum.Z += 2000.0;
			aGbxP.Velocity = Momentum;
			aGbxP.TakeDamage( 200, Instigator, aGbxP.Location, Momentum, DamageType );			
		}
	}	
}

simulated function ClientDoFireEffect()
{
    local Vector StartTrace;
    local Rotator Aim;
	local int bSkipDoTrace;
	local bool bIsHumanControlled;

	if ( Level.bWeaponsAreBFG && Instigator.IsHumanControlled() )
	{
		DoBFG();
	}

    // the to-hit trace always starts right in front of the eye
    //StartTrace = Instigator.Location + Instigator.EyePosition();

	// GBX:naj - no it doesn't; it starts from the muzzle
	// unless the pawn is the player.

	// GBX:sjp - well, now the GetMuzzleLocationWorld() handles
	// first person view as well.

	// SDJ 04/30/2004: Part 3 of the "Where the hell does the bullet come from saga".  In multiplayer having the bullet
	//                 originate at the muzzle location is a bad idea since the muzzle location is first/third person view
	//                 dependent.  This was causing clients to be horribly inaccurate since on the server they are technically
	//                 in third person.  So in multiplayer we'll use the old unreal logic for the bullet origin for players.

	// GBX:sjp 12/20/2004 - Screw it; use the eyeposition in first person for both SP and MP.  This fixes several SP bugs.

	// GBX:naj - 5/18/04 - Separated AI and human weapon aim error into different values.
	// GBX:naj - 1/06/05 - Substituted IsHumanControlled() for IsPlayerPawn().  IsPlayerPawn() returns true for all pawns, human- and AI-controlled.


	bIsHumanControlled = Instigator.IsHumanControlled();
	if ( bIsHumanControlled && !Instigator.bIsManningTurret )
	{
		StartTrace = Instigator.Location + Instigator.EyePosition();
	}
	else
	{
		StartTrace = Weapon.GetMuzzleLocationWorld();
	}

	// Get the modified aim rotator based on the weapon aim error (as well as any other error values along the way).
	Aim = AdjustAimNew(StartTrace, bSkipDoTrace);

	if (gbxPawn(Instigator) != None)
		gbxPawn(Instigator).LastAimRotator = Aim;

	if ( bSkipDoTrace != 1 )
	{
		bTraceWater = true;	//-BB
		DoTrace(StartTrace, Aim);
		bTraceWater = false;
	}

	if ( bIsHumanControlled && Instigator.IsA( 'gbxPawn' ) )
	{
		Instigator.Controller.JogFatigue( gbxPawn( Instigator ).FatigueJog_PlayerFiredShot );	
	}
}

native function DoTrace(Vector Start, Rotator Dir, optional out Vector End, optional out Vector EndDir);

function SpawnBeamEffect(Vector Start, Rotator Dir, Vector HitLocation, Vector HitNormal, int ReflectNum)
{
}

simulated function PlayReloading()
{
	local gbxPlayerController gbxPC;

	if (Weapon.ClipAmt >= Weapon.AmmoClipSize){
		gbxWeapon(Weapon).flAmmoFadeTime = AMMO_INVENTORY_SHOW_TIME;  //NAC: added this so that anytime the player presses the reload key, it shows the number of clips available.
		gbxWeapon(Weapon).flLastAmmoFadeTime = Level.TimeSeconds;
		return;
	}

	if (Instigator.IsA('Mind') && Instigator.IsMoving() && Instigator.bIsCrouched)
		return;
		
//	if (gbxWeapon(Weapon).m_fZoomed)
//		gbxWeapon(Weapon).ForceUnzoom();

	gbxWeapon(Weapon).m_fAIWeaponWaitingToFire = false;
	gbxWeapon(Weapon).flAmmoFadeTime = AMMO_INVENTORY_SHOW_TIME;
	gbxWeapon(Weapon).flLastAmmoFadeTime = Level.TimeSeconds;

	gbxPC = gbxPlayerController(Instigator.Controller);

	//DHW + -- Added to prevent player from entering reload anim if he's out of ammo
	if (Weapon.Ammo[0].AmmoAmount <= Weapon.ClipAmt)
	// GBX:PAD: The previous check was preventing partial clips from loading if the gun only had 1 partial clip
	// left.
//	if (Weapon.Ammo[0].AmmoAmount <= 0)
		return;
	//DHW -

	if( gbxWeapon(Weapon).m_fZoomed && gbxPC != None )
	{
		gbxPC.PendingDelayedAction = ReloadAfterZoomOut;
		gbxWeapon(Weapon).ForceUnzoom();
		//gbxPC.ChangeView(0);
	}
	else
	{
		Super.PlayReloading();
	}
}

function FlashMuzzleFlash()
{
	local gbxMuzzleFlash MuzFlash;

	MuzFlash = gbxMuzzleFlash(GetMuzzleFlash());
	MuzFlash.SetLocation(Weapon.Location);
	MuzFlash.Flash();
}

function DrawMuzzleFlash(Canvas Canvas)
{
}

simulated function bool AllowFire()
{
	if( Super.AllowFire() )
	{
		if (Pawn(Weapon.Owner) != None)
		{
			if (Pawn(Weapon.Owner).bIsSprinting)
			{
				Pawn(Weapon.Owner).SetSprinting( false );
			}

			if (Weapon.IsA('gbxWeapon') && Instigator.Controller.bIsHumanPlayer )
			{
				return !(gbxWeapon(Weapon).bAimingAtFriendly);
			}
		}

		if ( ShotCountMod > 0 && ShotCount > 0 )
		{
			ShotCount = ( ShotCount + 1 ) % ShotCountMod;
			return true;
		}
		else
		{
			ShotCount = ( ShotCount + 1 ) % ShotCountMod;
			return Instigator.ApproveShot();
		}
	}

	return false;
}


function Rotator GetLastAimRotator()
{
	if (gbxPawn(Instigator) != None)
		return gbxPawn(Instigator).LastAimRotator;
	return Rot(0,0,0);
}

function GetBestBurstCount( ESuppressionLevel SuppressionLevel, out int MinShots, out int MaxShots )
{
	switch ( SuppressionLevel )
	{
	case SL_UNSUPPRESSED:
		if ( !Level.m_bMPStyleFatigue && Level.NetMode == NM_Standalone )
		{
			MinShots = Burst_Low_SP.Min;
			MaxShots = Burst_Low_SP.Max;
		}
		else
		{
			MinShots = Burst_Low_MP.Min;
			MaxShots = Burst_Low_MP.Max;
		}
		break;
	case SL_SUPPRESSED:
		if ( !Level.m_bMPStyleFatigue && Level.NetMode == NM_Standalone )
		{
			MinShots = Burst_High_SP.Min;
			MaxShots = Burst_High_SP.Max;
		}
		else
		{
			MinShots = Burst_High_MP.Min;
			MaxShots = Burst_High_MP.Max;
		}
		break;
	}
}

function GetOpBurstCount( out int MinShots, out int MaxShots )
{
	if ( !Level.m_bMPStyleFatigue && Level.NetMode == NM_Standalone )
	{
		MinShots = Burst_Op_SP.Min;
		MaxShots = Burst_Op_SP.Max;
	}
	else
	{
		MinShots = Burst_Op_MP.Min;
		MaxShots = Burst_Op_MP.Max;
	}
}

function SetInstaKill( bool bInstaKill )
{
	if ( bInstaKill )
	{
		Damage_Human_SP.min=1000;
		Damage_Human_SP.max=1000;
		Damage_Human_MP.min=1000;
		Damage_Human_MP.max=1000;
		Damage_AI_SP.min=1000;
		Damage_AI_SP.max=1000;
		Damage_AI_MP.min=1000;
		Damage_AI_MP.max=1000;
	}
	else
	{
		Damage_Human_SP=Default.Damage_Human_SP;
		Damage_Human_MP=Default.Damage_Human_MP;
		Damage_AI_SP=Default.Damage_AI_SP;
		Damage_AI_MP=Default.Damage_AI_MP;
	}
}

simulated function Tick(float DeltaTime)
{
	local bool bReloadAfterFireAllowed;

	Super.Tick(DeltaTime);

	if (Weapon == None || Instigator == None || Instigator.Controller == None)
		return;
	
	if ( Instigator != None && Instigator.Controller != None )
		bReloadAfterFireAllowed = Instigator.Controller.ReloadAfterWeaponFire();
	else
		bReloadAfterFireAllowed = true;

//	if ( Weapon.Ammo[0] != None && Weapon.IsA( 'gbxWeapon' ) )
//	{
//		if ( bReloadAfterFireAllowed && !Weapon.bDelayReloading && (Weapon.Ammo[0].AmmoAmount > 0) && (Weapon.ClipAmt <= 0) && (Weapon.ClientState != WS_Reloading) && !gbxWeapon(Weapon).m_fPlayingFire)
//		{
//			PlayReloading();
//		}
//	}
}

defaultproperties
{
	Momentum=25000
    TracesPerFire=1	
    TraceRange=10000
//    NoAmmoSound=Sound'WeaponSounds.P1Reload5'
	bInstantHit=true
    AutoAim=0
    HitEffectProb=0.75
	FireTraceBackStep=125
	
	m_flRandomBurstRange=0.1
	ShotCountMod=0
}
