class ProjectileRocket extends gbxProjectile;

var Vector	FireVector;
var bool	bHitWater;
var bool	bActive;
var bool	bUseTrail;
var bool	bUsingSafeLaunches;
var vector  Direction;
var float	m_flTravelTime;
var float	RocketDrag;
var float	SafeLaunchDist, PlayerSafeLaunchDist;

var	BazookaRocketTrail trail;

var   class<Projector> ExplosionDecalVehicle;

simulated function PostNetBeginPlay()
{
	Direction = vector(Rotation);
	Velocity = speed * Direction;
	if (PhysicsVolume.bWaterVolume)
	{
		bHitWater = True;
		Velocity=0.6*Velocity;
	}
	bActive = true;

	if ( (Pawn(Owner).IsLocallyControlled() ) || (ROLE!=ROLE_Authority) )
	{
		if ( bUseTrail )
		{
			Trail = Spawn(class'BazookaRocketTrail',self);
			log(self$".PostNetBeginPlay() - "$Trail);
			Trail.SetBase(Self);
		}
	}
	
	// GBX:naj - fix bug 9223: Humans and NPCs should use different safe-launch distances (50 for player; 520 for NPCs).
	if ( Pawn(Owner).IsHumanControlled() )
	{
		SafeLaunchDist = Default.PlayerSafeLaunchDist;
	}

	Super.PostNetBeginPlay();

	SetStaticMesh(StaticMesh(DynamicLoadObject("s_npc_weapon_us.us_rocket_projectile", class'StaticMesh')));	
}

simulated function BeginPlay()
{
	if ( (Level.DetailMode != DM_Low) && !Level.bDropDetail )
		SmokeRate = 0.03;
	else
		SmokeRate = 0.15;
}

// Ensures that the shell will not explode close to the shooter.
simulated function UseSafetyLaunch()
{
	bUsingSafeLaunches = true;
	bCollideWorld = false;
	SetCollision( false, false, false );
	bActive = false;
}

simulated function PhysicsVolumeChange( PhysicsVolume NewVolume )
{
	if ( !NewVolume.bWaterVolume || bHitWater )
		return;

	bHitWater = True;
	Spawn(class'WaterSplashLarge',,,,rot(16384,0,0));			//-BB
	Velocity=0.6*Velocity;
}

simulated function ProcessTouch( actor Other, vector HitLocation )
{
	if (!bActive)
		return;

	if ( Other!=instigator )
	{
		Explosion(HitLocation);
		bActive = false;
	}
}

simulated function HitWall( vector HitLocation, vector HitNormal, actor Wall )
{
	local Projector Decal;
	local ProjectorRelative DecalRelative;
	local vector vHit, TraceEnd;
	local float Length;

	if (!bActive)
		return;

	Velocity = Vect( 0, 0, 0 );

	vHit = HitLocation - Location;
	Length = VSize(vHit);
	vHit = Normal(vHit);

	TraceEnd = Location + (vHit * (Length + 1.0f));

	if (TraceHitBackDrop(TraceEnd, Location))
	{
		Destroy();
		return;
	}

	if ( Role == ROLE_Authority )
	{
		if ( Mover(Wall) != None )
			Wall.TakeDamage( Damage, instigator, Location, MomentumTransfer * Normal(Velocity), MyDamageType);

		MakeNoise(1.0);
	}

	Explosion(Location + ExploWallOut * HitNormal);

	if (Wall.IsA('gbxSVehicle'))
	{
		if ( (ExplosionDecalVehicle != None) && (Level.NetMode != NM_DedicatedServer) )
		{
			Decal = Spawn(ExplosionDecalVehicle,self,,Location, rotator(-HitNormal));
			DecalRelative = ProjectorRelative(Decal);
			if (DecalRelative != None)
			{
			    DecalRelative.SetRelativeProjector(Wall, Location, rotator(-HitNormal));
			}
		}
	}
	else
	{
		if ( (ExplosionDecal != None) && (Level.NetMode != NM_DedicatedServer) )
		{
			Decal = Spawn(ExplosionDecal,self,,Location, rotator(-HitNormal));
		}
	}
}

function ServerExplosion(vector HitLocation)
{
	Explosion( HitLocation );
}

simulated function Destroyed()
{
	if ( Trail != None )
	{
		Trail.ShutdownTrail();
	}
	
	Super.Destroyed();
}

simulated function HurtRadius( float DamageAmount, float DamageRadius, class<DamageType> DamageType, float Momentum, vector HitLocation )
{
	local actor Victims;
	local float damageScale, dist;
	local vector dir;
	local Vector HitLoc, HitNormal;
	local name HitBoxName;
	local Actor HitActor;

	if( bHurtEntry )
		return;

	bHurtEntry = true;

	// For Bazookas use 'RadiusActors()' instead of 'VisibleCollidingActors()' to slightly damage
	// Pawns that are behind nearby cover...
	foreach RadiusActors( class 'Actor', Victims, DamageRadius, HitLocation )
	{
		// don't let blast damage affect fluid - CollisingActors doesn't really work for them - jag
		if( (Victims != self) && (Victims.Role == ROLE_Authority) )
		{
			dir = Victims.Location - HitLocation;
			dist = FMax(1,VSize(dir));
			dir = dir/dist;
			damageScale = 1 - FMax(0,(dist - Victims.CollisionRadius)/DamageRadius);

			HitActor = Trace( HitLoc, HitNormal, Victims.Location, HitLocation, true,,,true,, HitBoxName );

			if (HitActor == Victims)  // victim was exposed, gets full brunt of blast...
			{
				Victims.TakeDamage
				(
					damageScale * DamageAmount,
					Instigator,
					Victims.Location - 0.5 * (Victims.CollisionHeight + Victims.CollisionRadius) * dir,
					(damageScale * Momentum * dir),
					DamageType,
					HitBoxName
				);
			}
			else  // victim was only partially exposed (or completely hidden), gets 1/2 of blast...
			{
				damageScale = damageScale * 0.5f;

				Victims.TakeDamage
				(
					damageScale * DamageAmount,
					Instigator,
					Victims.Location - 0.5 * (Victims.CollisionHeight + Victims.CollisionRadius) * dir,
					(damageScale * Momentum * dir),
					DamageType,
					HitBoxName
				);
			}

			if( dist < HeavyTraumaRadius && Level.Game.IsA('WargameSinglePlayer') && Victims.IsA('gbxPlayerPawn') && gbxPlayerController(Pawn(Victims).Controller) != None )
			{
				gbxPlayerController(Pawn(Victims).Controller).DoHeavyTrauma();
			}
		}
	}
	bHurtEntry = false;
}

simulated function Tick(float DeltaTime)
{
	Super.Tick( DeltaTime );
	
	if ( bUsingSafeLaunches && Instigator != None )
	{
		if ( VSize( Location - Instigator.Location ) > SafeLaunchDist )
		{
			bCollideWorld = true;
			SetCollision( true, true, true );
			bUsingSafeLaunches = false;
			bActive = true;
		}
	}

	if ((m_aGuideTarget != None) && (m_flNextGuideTime < Level.TimeSeconds))
	{
		Velocity = VSize( Velocity ) * Normal( m_aGuideTarget.Location - Location );
		m_flNextGuideTime = Level.TimeSeconds + GUIDE_DELAY_TIME;
	}

	m_flTravelTime += DeltaTime;

	// Add drag
	Velocity -= Direction * RocketDrag * DeltaTime;

	// Add gravity over time
	Velocity.Z += PhysicsVolume.Gravity.Z * 0.5 * m_flTravelTime * DeltaTime;

	// Rotation changes
	SetRotation( Rotator( Velocity ) );
}

defaultproperties
{
	ExplosionDecal=class'BlastMark'
	ExplosionDecalVehicle=class'BlastMarkSmallVehicle'
	MyDamageType=class'DamageUSBazooka'
    speed=6000.0
	MaxSpeed=6000.000000
	Damage=500.000000
    RemoteRole=ROLE_SimulatedProxy
    LifeSpan=8.0
	DamageRadius=375.000000
	MomentumTransfer=50000
	Physics=PHYS_Projectile
//	StaticMesh=StaticMesh's_npc_weapon_us.us_rocket_projectile'
	DrawScale=1.00000
	AmbientGlow=64
	bBounce=false
    bCollideWorld=true
	bFixedRotationDir=True
	DrawType=DT_STATICMESH

    ShakeRadius=2000.000000
	ViewRollTime=0.200000
	RollMag=20.000000
	RollRate=100.000000
	OffsetMagVertical=20.000000
	OffsetRateVertical=800.000000
	OffsetMagHorizontal=20.000000
	OffsetRateHorizontal=800.000000
	OffsetIterations=10.000000

	bBlowingUp=false

	FuseTimeMin=5.0
	FuseTimeMax=6.0

	HeavyTraumaRadius=768.0
	
	bUseTrail=false
	bActive=false
	bDestroyWhenDone=true

	SafeLaunchDist=520.0f	// Damage radius plus a safety threshold
	PlayerSafeLaunchDist=50.0f
}