class TNT extends gbxExplosive;

#exec OBJ LOAD FILE=..\staticmesh\s_military_us.usx

var float MessageInterval;
var float CheckZInterval, NextCheckZTime;
var bool  bShowClockTimer;

function StartExplosionTimer()
{
	Super.StartExplosionTimer();
	SetTimer(MessageInterval, true);
	bShowClockTimer = true;
}

simulated function Timer()
{
	local gbxMessage GrenadeMessage;

	GrenadeMessage = new( None ) class<gbxMessage>( DynamicLoadObject( "gbxAI.PerceptionGrenade", class'Class' ));
	GrenadeMessage.Sender = self;
	GrenadeMessage.bUseSenderLocation = true;
	GrenadeMessage.bHasLocation = true;
	GrenadeMessage.BroadcastRadius = DamageRadius;		
	GrenadeMessage.bBroadcastLOSBlocked = true;
	PostGBXMessage( GrenadeMessage );
}

// Don't do anything if this thing hit a wall or the floor.
simulated function HitWall (vector HitLocation, vector HitNormal, actor Wall) {}

event Landed( vector HitLocation, vector HitNormal )
{
	//bHidden = false;	// Now keeping satchel invisible until the player dismounts.
}

// Make sure to handle rare cases where the TNT falls through the world.
function Tick( float DeltaTime )
{
	local	PlayerController	aPlayer;
	
	//DHW + Moved code out of the Parent Tick to prevent the fusetimer being updated

	//removed to prevent TNT ticking fusetime
	//Super.Tick( DeltaTime );

	//Moved up and modified from gbxExplosive to explode TNT when fusetime is up
	if ( !bHidden ) 
	{
		if ( bActivated )
		{
			if (Role == ROLE_Authority)
			{
				if ( flFuseTime <= 0.0 )
				{
					flFuseTime = 0.0;
					iReplicatedFuseTime = 0;
					bShouldExplode = true;
					Explosion( Location );
				}
			}
		}

		if( bBlowingUp )
		{
			if ( bHideAfterExplosion )	{	bHidden = true;	}
			else						{	Destroy();		}
		}
	}
	//DHW -

	if ( NextCheckZTime < 0.00001f || Level.TimeSeconds > NextCheckZTime )
	{
		if ( ( Instigator.Location.Z - Self.Location.Z > 1000.0f ) )
		{
			TNTTrigger( TheTrigger ).NotifyFellThroughWorld();
			Self.Destroy();
		}

		NextCheckZTime = Level.TimeSeconds + CheckZInterval;
	}
	
	if (!bHidden && bShowClockTimer)
	{
		if (Level.NetMode == NM_StandAlone)
		{
			aPlayer = Level.GetLocalPlayerController();
			if ((aPlayer != None) && (gbxHUD(aPlayer.MyHUD) != None))
			{
				gbxHUD(aPlayer.MyHUD).SetHUDClock( flFuseTime );
			}
		}
		bShowClockTimer = false;
	}
}

simulated function Explode(vector HitLocation, vector HitNormal) {}

function Explosion(vector HitLocation)
{
	//DHW + -- Alert the LDs that the explosion is going off soon
	TNTTrigger( TheTrigger ).NotifyPreExplosion();
	//DHW -
	Super.Explosion( HitLocation );
	TNTTrigger( TheTrigger ).NotifyExploded();
}

defaultproperties
{
	DrawType=DT_StaticMesh
	//StaticMesh=s_military_us.weapon.satchel_charge
	StaticMesh=s_military_us.weapon.explosive_charge
	bHidden=true
	
	DrawScale=1.5
	MyDamageType=class'DamageGammonBomb'	

	FuseTimeMin=10.0
	FuseTimeMax=10.0
	MessageInterval=0.5
	
	CollisionRadius=2.0

	Physics=PHYS_Falling

	ExplosionDecal=class'BlastMark'
	Damage=500.000000
	DamageRadius=750.000000
	HeavyTraumaRadius=400.0
	MomentumTransfer=50000

	ShakeRadius=2000.000000
	ViewRollTime=0.200000
	RollMag=20.000000
	RollRate=100.000000
	OffsetMagVertical=20.000000
	OffsetRateVertical=800.000000
	OffsetMagHorizontal=20.000000
	OffsetRateHorizontal=800.000000
	OffsetIterations=10.000000	

	CheckZInterval=1.0
}