class UsableHandGrenade extends gbxUsableItem;



var class<gbxGrenade>	GrenadeClass;
var	Vector				ThrowVector;	// Velocity vector to launch the grenade
var gbxGrenade			Grenade;		// Grenade object
var float				FuseStartDelay;
const GRENADE_THROW_DELAY = 1.56f;
var bool				fWaitForThrow;
var float				GrenadeThrowTime;

function Prepare()
{
	if (ROLE==ROLE_Authority)
	{
		// Spawn the grenade
		Grenade = Spawn(GrenadeClass, self,,Instigator.Location + Instigator.EyePosition());
		Grenade.RemoteRole = ROLE_None;

		// Log(name$".Prepare() - "$Grenade.Owner);
		// Attach it to my instigator
		Grenade.AttachToPawn( Instigator );

		// Making noise when the pin is pulled ensures the grenade will almost never work.
		// Instead, make noise when it is actually thrown

		// Start the fuse after a little delay
		SetTimer( FuseStartDelay, false );
		
		GrenadeThrowTime = Level.TimeSeconds + GRENADE_THROW_DELAY;
		fWaitForThrow = true;
	}
}

simulated function Use( float Value )
{
	// Log(name$".Use( "$Value$" ) - "$Grenade);
	if (Pawn( Owner ) != None)
	{
		ProjectileFire();
	}
}

simulated function name GetUseAnimation()
{
	return 'grenade_throw';
}

function ServerProjectileFire()
{
	if (Role < ROLE_Authority)
		return;

	Owner.MakeNoise(1.0);

	if( Grenade != None )
	{
		if( Instigator != None ) // None == dead - don't launch the grenade.
		{
			if( ThrowVector != Vect(0,0,0) )
				Grenade.ThrowGrenade( ThrowVector );
			else
				Grenade.ThrowGrenade();
		}
		else
		{
			Grenade.ThrowGrenade( vect(0,0,1.0) );
		}
	}
}

simulated function ProjectileFire()
{
	ServerProjectileFire();
//	Log(name$".ProjectileFire() - "$Grenade);
//	if( Grenade != None )
//	{
//		if( ThrowVector != Vect(0,0,0) )
//			Grenade.ThrowGrenade( ThrowVector );
//		else
//			Grenade.ThrowGrenade();
//	}
}

function Timer()
{
	// Light it up!
	Grenade.StartFuse();
}

function Tick(float Delta)
{
	Super.Tick( Delta );
	
	if ( fWaitForThrow && (GrenadeThrowTime < Level.TimeSeconds))
	{
		ProjectileFire();
		if ((gbxPawn(Instigator) != None) && (gbxWeapon(gbxPawn(Instigator).Weapon) != None))
			gbxWeapon(gbxPawn(Instigator).Weapon).m_fCanZoom = true;
		fWaitForThrow = false;
	}
}

defaultproperties
{
	FuseStartDelay = 1.10;
}
