class WargameWeapon extends gbxWeapon
	abstract
	native;

replication
{
	reliable if ( Role == ROLE_Authority )
		ClientPlantGammonBombOnTarget, ClientDenyPlantGammonBombOnTarget;

	reliable if ( Role < ROLE_Authority )
		ServerPlantGammonBombOnTarget;
}

var bool bDisableCrosshair;
var Rotator m_rotDesiredWeaponDir;
var Rotator m_rotLastWeaponDir;
var Rotator	m_rotWeaponDir;
var Rotator m_rotBreathDir;
var Rotator m_rotLastBaseAngle;
var Vector	m_vecDesiredDirRate;
var Vector	m_vecFatigueDirRate;
var Vector	m_vecChangeDirRate;
var float	m_flLastAccelTime;
var float	m_flDirStartTime;
var float	m_flDecelStartTime;
var float	m_flWeaponDirRate;

var name	m_szFromHipAngleBone;
var name	m_szZoomedAngleBone;

const ROT_MOMENTUM_RATE = 15;
const ROT_CHARACTER_DIR_RATE = 1.0f;
const MAX_ROT_ACCEL_RATE = 300;

var float	m_flMaxRotAccelRate;
var float	m_flRotFatigueTime;
var float	m_flErrorScale;

var bool	m_fDisableFatigue;


simulated native function AdjustWeaponAngle( int nError );

simulated function DrawCrossHair( canvas Canvas )
{
	local float XLength, YLength;
	local  PlayerController PlayerOwner;
	local HUD	Hud;

	
	if( bDisableCrosshair )
		return;

	if ( Instigator != None && Instigator.IsHumanControlled() )
	{
		PlayerOwner = PlayerController(Instigator.Controller);

		if ( PlayerOwner.IsZoomed() )
			return;
			
		if ( PlayerOwner.DisableCrosshair() )
			return;
	
		if ( PlayerOwner.iCrossHairIndex == 0 )
			return;
		
		if ( Level.NetMode != NM_StandAlone )
			return;

		Hud = PlayerOwner.MyHud;

		if (Hud == None)
		{
			XLength = 32;
			YLength = 32;
		}
		else
		{
			XLength = 32 * Hud.flMinScaleX;
			YLength = 32 * Hud.flMinScaleY;
		}
		Canvas.bNoSmooth = False;
		//Canvas.SetPos(0.503 * (Canvas.ClipX - XLength), 0.504 * (Canvas.ClipY - XLength));

		Canvas.SetPos(0.5 * (Canvas.ClipX - XLength), 0.5 * (Canvas.ClipY - YLength));

		Canvas.CurY -= (YLength *0.3 );   //NAC: Why the magic number?  Since all other icons in the game use the "Rotator" it causes them to be displaced a little...instead of changing all of them, this was MUCH easier!

	//	Canvas.Style = ERenderStyle.STY_Translucent;
		Canvas.Style = ERenderStyle.STY_Additive;
		Canvas.SetDrawColor(255,255,255,96);

		Canvas.DrawTile(m_TestCrosshairs[PlayerOwner.iCrossHairIndex - 1], XLength, YLength, 0, 0, m_TestCrosshairs[PlayerOwner.iCrossHairIndex - 1].USize, m_TestCrosshairs[PlayerOwner.iCrossHairIndex - 1].VSize);

		Canvas.bNoSmooth = True;
		Canvas.Style = Style;
	}		
}

exec function ToggleCrosshair()
{
	bDisableCrosshair = !bDisableCrosshair;
}

// Compare self to current weapon.  If better than current weapon, then switch
simulated function ClientWeaponSet(bool bOptionalSet)
{
	local gbxPawn	aPlayer;

	Instigator = Pawn(Owner); //weapon's instigator isn't replicated to client

	if ( m_eWeaponType == WeaponPrimary )
	{
		aPlayer = gbxPawn( Instigator );
		Super.ClientWeaponSet( bOptionalSet );
		return;
	}

	if (gbxPawn( Instigator ) != None)
	{
		aPlayer = gbxPawn( Instigator );

		if ( aPlayer.SecondaryWeapon == None )
		{
			aPlayer.PendingSecondaryWeapon = self;
			aPlayer.ChangedSecondaryWeapon();
		}
	}

	GotoState('');
}

simulated function ForceUnzoom()
{
	local PlayerController	aPlayer;

	if (PlayerController(Instigator.Controller) != None)
	{
		aPlayer = PlayerController(Instigator.Controller);

		aPlayer.DesiredFOV = aPlayer.DefaultFOV;

		if (gbxPawn(aPlayer.Pawn) != None)
		{
			aPlayer.Pawn.eWeaponAimedState = WAS_Sighted; // GBX:naj - 3/24/04
			aPlayer.Pawn.flLastAimedStateChange = Level.TimeSeconds;
			SwitchZoomView( false );
		}
	}
}

function PlantGammonBombOnTarget()
{
	ServerPlantGammonBombOnTarget();
}

// This is called by the server when a client use's an explosive trigger and is allowed to begin planting bomb
simulated function ClientPlantGammonBombOnTarget()
{
	ServerPlantGammonBombOnTarget();		
}

simulated function ClientDenyPlantGammonBombOnTarget()
{
}

// This is called by the client to tell the server we're done animating
function ServerPlantGammonBombOnTarget()
{
	local gbxPlayerController aController;
	
	aController = gbxPlayerController(Instigator.Controller);
	if (aController != None)
	{
		if ( !aController.AttachBombToTarget( Instigator ))
		{
			// The server decided that you cannot plant teh bomb
			ClientDenyPlantGammonBombOnTarget();
		}
	}
	else
	{
		// The server decided that you cannot plant teh bomb
		ClientDenyPlantGammonBombOnTarget();
	}
}

exec notinship function DisableWeaponFatigue( bool fDisable )
{
	m_fDisableFatigue = fDisable;
	m_rotWeaponDir = rot( 0, 0, 0 );
	SetBoneRotation( m_szZoomedAngleBone, m_rotWeaponDir, 0, 1.0f );
}

exec function SetFatigueAccelRate( float flMaxRotAccelRate )
{
	m_flMaxRotAccelRate = flMaxRotAccelRate;
}
exec function SetFatigueFatigueTime( float flRotFatigueTime )
{
	m_flRotFatigueTime = flRotFatigueTime;
}
exec function SetFatigueErrorScale( float flErrorScale )
{
	m_flErrorScale = flErrorScale;
}

defaultproperties
{
	bDisableCrosshair = false
	m_flWeaponDirRate = 0.0f

	m_szFromHipAngleBone = ""
	m_szZoomedAngleBone = "Gun"

	m_flMaxRotAccelRate = 300
	m_flRotFatigueTime = 1.0f
	m_flErrorScale = 1.0f

	Icon=Texture't_interface_mb.hud.interface_comp'
}
