class WeapDEFlakvierling extends TurretWeapon;

//---------------------------------------------------------
//	Variables
//---------------------------------------------------------
var Actor	TargetActor;		// If this is None, the controller will assign random targets.
var	Name	BaseBone, GunBone, CoGunBone, HorizCrankBone, VertCrankBone;
var Name	GunnerAttach, LeftLoaderAttach, RightLoaderAttach;
var Name	GunnerToDraft, LeftLoaderToDraft, RightLoaderToDraft;
var Name	DeathEvent;
var float	YawRotSpeed, PitchRotSpeed;
var float	NextCrewCheckTime;
var float	CrewCheckInterval;
var int		CurBaseYaw, CurGunPitch, PitchOffset;
var int		MaxPitch,MinPitch;
var bool	bDontRecruit;
var bool	bCrewHasBeenDrafted;
var bool	bTurningLeft, bTurningRight, bTurningUp, bTurningDown, bWasFiring;
var Rotator	DesiredRot, LastRequestedRot;
var gbxPawn	Gunner, LeftLoader, RightLoader;
var Vector	GunnerOffset, LeftLoaderOffset, RightLoaderOffset;
var FlakvierlingBrokenTurret BrokenTurret;

var const int		HorizRotChannel, VertRotChannel;

var enum ELoaderSide
{
	Loader_Left,
	Loader_Right

} LastSideToReload;

//---------------------------------------------------------
//	PostNetBeginPlay
//---------------------------------------------------------
function PostNetBeginPlay()
{
	Super.PostNetBeginPlay();
	GiveAmmo(0);

	Mesh = Mesh(DynamicLoadObject("k_prop.de_flakv", class'Mesh'));
	LinkMesh( Mesh );
	
	PlayAnim( 'Idle' );	
}

//---------------------------------------------------------
//	IsAntiAir
//---------------------------------------------------------
function bool IsAntiAir() 
{ 
	return true; 
}

//---------------------------------------------------------
//	GetBracingAnim
//---------------------------------------------------------
// TBD: Get rid of this ugly function - move animation names into ItemPawnAnimInfo.
function Name GetBracingAnim( gbxPawn Loader )
{
	if ( Loader == LeftLoader )
		return 'Flakv_L_loader_fire';
	else if ( Loader == RightLoader )
		return 'Flakv_R_loader_fire';
	else
		return '';
}

//---------------------------------------------------------
//	SetTargetActor
//---------------------------------------------------------
function SetTargetActor( Actor NewTargetActor )
{
	TargetActor = NewTargetActor;
}

//---------------------------------------------------------
//	GetTargetActor
//---------------------------------------------------------
function Actor GetTargetActor()
{
	return TargetActor;	// May be None.
}

//---------------------------------------------------------
//	FaceRotation
//---------------------------------------------------------
function FaceRotation( Rotator Rot )
{
	Rot.Pitch = Rot.Pitch & 65535;

	if ( bHasBlownUp )
		return;

	// Hacky damper code: Throw out the request if it's only a little bit different from the current one.
	if ( abs( Rot.Pitch - LastRequestedRot.Pitch ) <= 5 && abs( Rot.Yaw - LastRequestedRot.Yaw ) <= 5 )
		return;

	LastRequestedRot = Rot;

	// Make sure the requested pitch does not go outside the bounds of the weapon.
	if ( Rot.Pitch > MaxPitch ) 
	{
		if ( Rot.Pitch > 32768 )
		{
			Rot.Pitch = 0;
		}
		else
		{
			Rot.Pitch = MaxPitch;
		}
	}

	// Make sure the requested pitch isn't to low
	if(Rot.Pitch < MinPitch)
	{
		Rot.Pitch = MinPitch;
	}
	
	DesiredRot = Rot;	
	DesiredRot.Pitch -= PitchOffset;		// Because at pitch=0, the gun may not actually be pointing horizontally	

	DesiredRot.Pitch = DesiredRot.Pitch & 65535;
	DesiredRot.Yaw = DesiredRot.Yaw & 65535;
}

//---------------------------------------------------------
//	GetAdjustedRotation
//---------------------------------------------------------
function int GetAdjustedRotation( int CurPosition, int DesiredPosition, float Speed, float DeltaTime, out int bRotatingLeft )
{
	local int Diff, NewPos, Offset;
	local bool bIsNegative, bDidSubtract, bSnap;
	local float AdjustedSpeed;

	Diff = DesiredPosition - CurPosition;
	bIsNegative = ( Diff < 0 );

	Diff = abs( Diff );

	if ( Diff > 32767)
	{
		Diff = 65536 - Diff;
		bDidSubtract = true;
	}

	AdjustedSpeed = Speed;
	Offset = AdjustedSpeed*DeltaTime;
	if ( Offset >= Diff )
	{
		do
		{
			AdjustedSpeed *= 0.75;
			Offset = AdjustedSpeed*DeltaTime;
		}
		until( AdjustedSpeed < 0.25*Speed || Offset <= Diff );
		
		if ( Offset >= Diff )
		{
			bSnap = true;
		}
	}
	
	if ( ( bDidSubtract && bIsNegative ) || ( !bDidSubtract && !bIsNegative ) )
	{
		if ( bSnap )
			NewPos = DesiredPosition;
		else
			NewPos = (CurPosition + Offset ) & 65535;
		
		bRotatingLeft = 0;
	}
	else
	{
		if ( bSnap )
			NewPos = DesiredPosition;
		else
			NewPos = (CurPosition - Offset ) & 65535;

		bRotatingLeft = 1;
	}

	return NewPos;	
}

//---------------------------------------------------------
//	PawnIsInCrew
//---------------------------------------------------------
function bool PawnIsInCrew( Pawn P ) 
{ 
	return P == Gunner || P == LeftLoader || P == RightLoader;
}

//---------------------------------------------------------
//	DraftCrew
//---------------------------------------------------------
function DraftCrew()
{
	local gbxPawn aDraftee;
	local class<gbxAIController> MindClass;
	
	ForEach DynamicActors(class'gbxPawn', aDraftee )
	{
		if ( aDraftee.Tag == GunnerToDraft && Gunner == None )
		{
			Gunner = aDraftee;
			MindClass = class<gbxAIController>( DynamicLoadObject( "gbxAI.MindFlakvierlingGunner", class'Class' ) );
			Gunner.CrewTurret( Self, MindClass, Job_Flakv_Gunner );
		}
		else if ( aDraftee.Tag == LeftLoaderToDraft && LeftLoader == None  )
		{
			LeftLoader = aDraftee;
			MindClass = class<gbxAIController>( DynamicLoadObject( "gbxAI.MindFlakvierlingLoader", class'Class' ) );
			LeftLoader.CrewTurret( Self, MindClass, Job_Flakv_LeftLoader );
		}
		else if ( aDraftee.Tag == RightLoaderToDraft && RightLoader == None  )
		{
			RightLoader = aDraftee;
			MindClass = class<gbxAIController>( DynamicLoadObject( "gbxAI.MindFlakvierlingLoader", class'Class' ) );
			RightLoader.CrewTurret( Self, MindClass, Job_Flakv_RightLoader );
		}
	}

	if ( Gunner != None )
		bCrewHasBeenDrafted = true;
}

//---------------------------------------------------------
//	GetFireRotation
//---------------------------------------------------------
function Rotator GetFireRotation()
{
	local Rotator Temp;
	Temp.Pitch = CurGunPitch;
	Temp.Pitch = ( Temp.Pitch + 5385 ) & 65535;
	Temp.Yaw = CurBaseYaw;
	return Temp;
}

//---------------------------------------------------------
//	GetViewOrigin
//---------------------------------------------------------
function Vector GetViewOrigin()
{
	if ( Gunner != None )
		return Gunner.Location;
}

//---------------------------------------------------------
//	Tick
//---------------------------------------------------------
function Tick( float DeltaTime )
{
	local name AnimName;
	local float AnimFrame, AnimRate;
	local Rotator BaseRot, GunRot, CrankRot;;
	local int RotatingLeft, RotatingUp;

	Super.Tick( DeltaTime );

	if ( bHasBlownUp )
		return;

	// Make sure that this weapon is manned.
	//if ( !bCrewHasBeenDrafted )
	//{
	if ( !bDontRecruit && !bCrewHasBeenDrafted && Level.TimeSeconds > NextCrewCheckTime )
	{
		NextCrewCheckTime = Level.TimeSeconds + CrewCheckInterval;
		DraftCrew();		
	}

	if ( Gunner == None )
		return;

	bTurningLeft = false;
	bTurningRight = false;
	bTurningUp = false;
	bTurningDown = false;
	
	// Determine which direction (if any) the weapon is turning.
	if ( abs( DesiredRot.Yaw - CurBaseYaw ) > 0 )
	{		
		CurBaseYaw = GetAdjustedRotation( CurBaseYaw, DesiredRot.Yaw, YawRotSpeed, DeltaTime, RotatingLeft );
		BaseRot.Pitch = Rotation.Yaw-CurBaseYaw;	// Yes, I must use the pitch component to yaw the base.  Confusing, isn't it?
		SetBoneRotation( BaseBone, BaseRot, 1.0 );
		
		if ( RotatingLeft == 1 )
		{
			bTurningLeft = true;
		}
		else
		{
			bTurningRight = true;
		}
	}
	else
	if ( abs( DesiredRot.Pitch - CurGunPitch ) > 0 )
	{
		CurGunPitch = GetAdjustedRotation( CurGunPitch, DesiredRot.Pitch, PitchRotSpeed, DeltaTime, RotatingUp );
		GunRot.Yaw = ( (65536-CurGunPitch) ) & 65535;
		SetBoneRotation( GunBone, GunRot, 1.0 );
		SetBoneRotation( CoGunBone, GunRot, 1.0 );
		
		if ( RotatingUp == 1 )
			bTurningUp = true;
		else
			bTurningDown = true;
	}	
	
	// Play the appropriate hand-cranking animation if the weapon is turning.
	if ( bTurningLeft )
	{		
		PlayCrankAnimations( 'Flakv_gunner_crank_left' );
	}
	else if ( bTurningRight )
	{		
		PlayCrankAnimations( 'Flakv_gunner_crank_right' );
	}
	else if ( bTurningUp )
	{
		PlayCrankAnimations( 'Flakv_gunner_crank_up' );
	}
	else if ( bTurningDown )
	{
		PlayCrankAnimations( 'Flakv_gunner_crank_down' );
	}
	
	// Synchronize the crank bones with the gunner's cranking animation, if any.
	Gunner.GetAnimParams( 0, AnimName, AnimFrame, AnimRate );
	if ( AnimName == 'Flakv_gunner_crank_left' )
	{
		CrankRot.Pitch = AnimFrame * 65535;
		SetBoneRotation( HorizCrankBone, CrankRot, 1.0 );
		SetBoneRotation( VertCrankBone, Rot(0,0,0), 1.0 );
	}
	else if ( AnimName == 'Flakv_gunner_crank_right' )
	{
		CrankRot.Pitch = 65536 - AnimFrame * 65535;
		SetBoneRotation( HorizCrankBone, CrankRot, 1.0 );
		SetBoneRotation( VertCrankBone, Rot(0,0,0), 1.0 );
	}
	else if ( AnimName == 'Flakv_gunner_crank_up' )
	{
		CrankRot.Yaw = AnimFrame * 65535;
		SetBoneRotation( HorizCrankBone, Rot(0,0,0), 1.0 );
		SetBoneRotation( VertCrankBone, CrankRot, 1.0 );
	}
	else if ( AnimName == 'Flakv_gunner_crank_down' )
	{
		CrankRot.Yaw = 65536 - AnimFrame * 65535;
		SetBoneRotation( HorizCrankBone, Rot(0,0,0), 1.0 );
		SetBoneRotation( VertCrankBone, CrankRot, 1.0 );
	}		
	else
	{
		SetBoneRotation( HorizCrankBone, Rot(0,0,0), 1.0 );
		SetBoneRotation( VertCrankBone, Rot(0,0,0), 1.0 );
	}

	if ( !FireMode[0].bIsFiring && bWasFiring )
	{
		Self.LoopAnim( 'Idle', 1.0, 0.25 );
		gbxAIController( Gunner.Controller ).NotifyStopFiring();
		gbxAIController( LeftLoader.Controller ).NotifyStopFiring();
		gbxAIController( RightLoader.Controller ).NotifyStopFiring();
		bWasFiring = false;
	}
}

//---------------------------------------------------------
//	PlayCrankAnimations
//---------------------------------------------------------
simulated function PlayCrankAnimations( Name GunnerAnim )
{
	local name AnimName;
	local float AnimFrame, AnimRate;

	Gunner.GetAnimParams( 0, AnimName, AnimFrame, AnimRate );
	if ( AnimName != GunnerAnim )
	{
		Gunner.AddAnimToQueue( GunnerAnim, 96.0, true, false,, 2.0, 0.15 );
	}	
}

//---------------------------------------------------------
//	PlayFiring
//---------------------------------------------------------
simulated function PlayFiring()
{
	local name AnimName;
	local float AnimFrame, AnimRate;	

	Gunner.GetAnimParams( 0, AnimName, AnimFrame, AnimRate );

	gbxAIController( Gunner.Controller ).NotifyBeginFiring();
	gbxAIController( LeftLoader.Controller ).NotifyBeginFiring();
	gbxAIController( RightLoader.Controller ).NotifyBeginFiring();

	FireDEFlakvierling( FireMode[0] ).FlashMuzzleFlashes( true );
	
	SetTimer( 0.2, false );
	bWasFiring = true;
}

//---------------------------------------------------------
//	Timer
//---------------------------------------------------------
function Timer()
{
	if ( bHasBlownUp )		//If the timer was set before a dismount it will fire again.  
	{
		SetTimer( 0.0, false );
		return;
	}

	Super.Timer();
	FireDEFlakvierling( FireMode[0] ).FlashMuzzleFlashes( false );
}

//---------------------------------------------------------
//	CompleteMount
//---------------------------------------------------------
function CompleteMount( gbxPawn gbxP )
{
	if ( gbxP == Gunner )
	{
		Gunner.SetBase( None );
		Gunner.SetPhysics( PHYS_None );
		AttachToBone( Gunner, GunnerAttach );		
		Gunner.SetRelativeRotation( Rot( 0,32768,16384 ) );
		Gunner.SetRelativeLocation( GunnerOffset );
		Gunner.FlushAnimQueueChannel( 0,, true );
		Gunner.StopAnimating();
		Gunner.LoadMountAnimations( Mount_MountFlakVGunner );
		Gunner.bDoTorsoTwist = false;
		Gunner.SetWeapon( Self );

		Instigator = Gunner;
		if ( Ammo[0] != None )
			Ammo[0].Instigator = Instigator;
		if ( FireMode[0] != None )
			FireMode[0].Instigator = Instigator;
		if ( ThirdPersonActor != None )
			ThirdPersonActor.Instigator = Instigator;

	}
	else if ( gbxP == LeftLoader )
	{
		LeftLoader.SetBase( None );
		LeftLoader.SetPhysics( PHYS_None );
		AttachToBone( LeftLoader, LeftLoaderAttach );
		LeftLoader.SetRelativeRotation( Rot( 0,32768,16384 ) );
		LeftLoader.SetRelativeLocation( LeftLoaderOffset );
		LeftLoader.FlushAnimQueueChannel( 0,, true );
		LeftLoader.StopAnimating();
		LeftLoader.LoadMountAnimations( Mount_MountFlakVLoader );
		LeftLoader.bDoTorsoTwist = false;
		LeftLoader.SetWeapon( None );
	}
	else if ( gbxP == RightLoader )
	{
		RightLoader.SetBase( None );
		RightLoader.SetPhysics( PHYS_None );
		AttachToBone( RightLoader, RightLoaderAttach );
		RightLoader.SetRelativeRotation( Rot( 0,32768,16384 ) );
		RightLoader.SetRelativeLocation( RightLoaderOffset );
		RightLoader.FlushAnimQueueChannel( 0,, true );
		RightLoader.StopAnimating();
		RightLoader.LoadMountAnimations( Mount_MountFlakVLoader );
		RightLoader.bDoTorsoTwist = false;
		RightLoader.SetWeapon( None );
	}	

	gbxP.PlayWaiting();
}

//---------------------------------------------------------
//	PreparePawnToDismount
//---------------------------------------------------------
function PreparePawnToDismount( gbxPawn gbxP )
{
	gbxP.SetPhysics( PHYS_Falling );

	//make sure we suspend spawning of smoke when they exit
	FireDEFlakvierling( FireMode[0] ).StopFiring();
	FireDEFlakvierling( FireMode[1] ).StopFiring();
}

//---------------------------------------------------------
//	CompleteDismount
//---------------------------------------------------------
function CompleteDismount( gbxPawn gbxP )
{
	//make sure we suspend spawning of smoke when they exit
	FireDEFlakvierling( FireMode[0] ).StopFiring();
	//gbxP.Weapon = None;  // hack so SetWeapon() will call ChangedWeapon() - jlb
	//gbxP.SetWeapon( gbxP.LastWeapon );
}

//---------------------------------------------------------
//	ForceCrewDismount
//---------------------------------------------------------
function ForceCrewDismount( optional Pawn InstigatedBy )
{
	bDontRecruit = true;

	bTurningLeft = false;
	bTurningRight = false;
	bTurningUp = false;
	bTurningDown = false;

	if ( Gunner != None )
	{
		StopFiringWeapon();
		gbxAIController( Gunner.Controller ).BreakOutJustMe( InstigatedBy );
	}

	if ( LeftLoader != None )
	{
		gbxAIController( LeftLoader.Controller ).BreakOutJustMe( InstigatedBy );
	}

	if ( RightLoader != None )
	{
		gbxAIController( RightLoader.Controller ).BreakOutJustMe( InstigatedBy );
	}

	Gunner = None;
	LeftLoader = None;
	RightLoader = None;
}

// Special override fixes a rare bug where the FiringController tells the weapon to stop firing right after it has consumed its last
// bullet.  This had prevented the weapon from ever firing again.
function StopFiringWeapon()
{
	Super.StopFiringWeapon();
	if ( NeedsToReload() )
	{
		PlayReloading();
	}
}

//---------------------------------------------------------
//	PlayReloading
//---------------------------------------------------------
simulated function PlayReloading() 
{
	gbxAIController( LeftLoader.Controller ).ReloadTurret();
	gbxAIController( RightLoader.Controller ).ReloadTurret();	
}

//---------------------------------------------------------
//	ReloadFinished
//---------------------------------------------------------
function ReloadFinished()
{
	ReloadClip();
	gbxAIController( LeftLoader.Controller ).NotifyReloadFinished();
	gbxAIController( RightLoader.Controller ).NotifyReloadFinished();
}

//---------------------------------------------------------
//	NotifyForceDismount
//---------------------------------------------------------
function NotifyForceDismount( optional bool bDoNothingIfHumanControlled )
{
	ForceCrewDismount();
}

//---------------------------------------------------------
//	SetBlownUp
//---------------------------------------------------------
function SetBlownUp()
{
	local Rotator NewTurretRot;

	Super.SetBlownUp();

	StopFiringWeapon();

	bTurningLeft = false;
	bTurningRight = false;
	bTurningUp = false;
	bTurningDown = false;

	if ( Gunner != None )
	{
		Gunner.Destroy();
	}
		
	if ( LeftLoader != None )
	{
		LeftLoader.Destroy();
	}
	
	if ( RightLoader != None )
	{
		RightLoader.Destroy();
	}

	SetDrawType( DT_StaticMesh );
	SetStaticMesh( StaticMesh(DynamicLoadObject("s_military_de.Weapon.flakvierling_dmg_base", class'StaticMesh')) );
	
	BrokenTurret = Spawn( class'FlakvierlingBrokenTurret' );
	
	NewTurretRot.Yaw = CurBaseYaw;
	NewTurretRot.Pitch = CurGunPitch;
	BrokenTurret.SetLocation( Self.Location );
	BrokenTurret.SetRotation( NewTurretRot );
}

//---------------------------------------------------------
//	BlowUp
//---------------------------------------------------------
function BlowUp( Pawn EventInstigator )
{
	local Rotator NewTurretRot, HurtOffsetRot;

	Super.BlowUp( EventInstigator );

	bTurningLeft = false;
	bTurningRight = false;
	bTurningUp = false;
	bTurningDown = false;

	SetDrawType( DT_StaticMesh );
	SetStaticMesh( StaticMesh(DynamicLoadObject("s_military_de.Weapon.flakvierling_dmg_base", class'StaticMesh')) );
	
	BrokenTurret = Spawn( class'FlakvierlingBrokenTurret' );
	
	NewTurretRot.Yaw = CurBaseYaw;
	NewTurretRot.Pitch = CurGunPitch;
	BrokenTurret.SetLocation( Self.Location );
	BrokenTurret.SetRotation( NewTurretRot );

	HurtOffsetRot = NewTurretRot;
	HurtOffsetRot.Pitch = 0;
	
	HurtRadius( 500.0,500.0, class'DamageTankCannon', 50000, Location - 100.0*Vector( HurtOffsetRot ) );

	if ( DeathEvent != '' )
	{
		TriggerEvent( DeathEvent, Self, EventInstigator );
	}
}

//---------------------------------------------------------
//	CanBeDestroyedBy
//---------------------------------------------------------
function bool CanBeDestroyedBy( class<DamageType> DamageType ) 
{ 
	return (
		DamageType == class'DamageGammonBomb' || 
		DamageType == class'DamageTankCannon' ||
		DamageType == class'DamageDEPanzerfaust' ||
		DamageType == class'DamageUSBazooka' 
		);
}

//---------------------------------------------------------
//	AlwaysAutoReload
//---------------------------------------------------------
function bool AlwaysAutoReload()
{
	return true;
}

simulated event RenderOverlays( Canvas Canvas ) {}

//=========================================================
//	Default Properties
//=========================================================
defaultproperties
{
	DrawType=DT_Mesh
//	Mesh=Mesh'k_prop.de_flakv'
//	StaticMesh=StaticMesh's_military_de.Weapon.flakvierling_dmg_base'
	Mesh=None
	StaticMesh=None
	Texture=Texture'Engine.S_Weapon'

	FireModeClass(0)=FireDEFlakvierling
	FireModeClass(1)=FireDEFlakvierling
	AttachmentClass=class'AttachmentDEFlakvierling'

	BaseBone=Dummy01
	GunBone=Dummy04
	CoGunBone=Dummy03
	HorizCrankBone=Dummy14
	VertCrankBone=Dummy12

	GunnerAttach=Gunner
	LeftLoaderAttach=Left
	RightLoaderAttach=Right

	YawRotSpeed=5000.0
	PitchRotSpeed=5000.0
	PitchOffset=5385
	MaxPitch=16384
	MinPitch=8192

	GunnerOffset=(x=0,y=7.0,z=0)
	LeftLoaderOffset=(x=0,y=0,z=0)
	RightLoaderOffset=(x=0,y=0,z=0)
	
	HorizRotChannel=1
	VertRotChannel=2

	ClientState=WS_Hidden
	AmmoClipSize=20

	Physics=PHYS_Falling
	bStatic=false
	bCollideActors=true
	bBlockActors=true
	bBlockPlayers=true
	bCollideWorld=true
	bCollideSkeletalMesh=true
	bBlockZeroExtentTraces=true
	bBlockNonZeroExtentTraces=true
	CollisionHeight=0.0

	CrewCheckInterval=3.0

	BreatheProfileClass=class'gbxSupportedBreatheProfile'

	bOpaqueForSituationalAwarenessView=true
}