//=============================================================================
// WeapDEMG42Static
//=============================================================================
class WeapDEMG42Static extends WargameWeapon
    config(user);


var Cue				FireLoopSound;
var String			FireLoopSoundName;
var	int				FireLoopSoundHandle;
var Cue				FireOneBulletSound;
var String			FireOneBulletSoundName;

var bool			bWeaponFiring;

simulated function PostBeginPlay()
{
	if ( FireLoopSoundName != "" )
		FireLoopSound = LoadSound(FireLoopSoundName);
	if ( FireOneBulletSoundName != "" )
		FireOneBulletSound = LoadSound(FireOneBulletSoundName);
		
	Super.PostBeginPlay();
}

simulated function LoadWeaponMesh()
{
	Super.LoadWeaponMesh();
	
	Mesh = Mesh( DynamicLoadObject( "k_view_model_us.view_thompson", class'Mesh' ) );
	m_NormalViewMesh = Mesh( DynamicLoadObject( "k_view_model_us.view_thompson", class'Mesh' ) );
	m_ZoomViewMesh = Mesh( DynamicLoadObject( "k_view_model_us.view_thompson_crosshair", class'Mesh' ) );
	LinkMesh( Mesh );
}

function byte BestMode()
{
	return 0;
}

//---------------------------------------------------------
//	StartFiringWeapon
//---------------------------------------------------------
function StartFiringWeapon()
{
	if ( bWeaponFiring )
		return;

	// RSC: dont fire if we are issuing a command
	if( !Instigator.Controller.AllowFire() )
		return;

	FireLoopSoundHandle = PlaySoundCue( FireLoopSound, SLOT_NONE, TransientSoundVolume,,,,false);

	if ( FireMode[0].IsA( 'FireDEMG42_' ) )
		FireDEMG42_( FireMode[0] ).StartFiringWeapon();

	Super.StartFiringWeapon();

	bWeaponFiring = true;
}

//---------------------------------------------------------
//	StopFiringWeapon
//---------------------------------------------------------
function StopFiringWeapon()
{
	if ( FireMode[0].IsA( 'FireDEMG42_' ) )
		FireDEMG42_( FireMode[0] ).StopFiringWeapon();

	if ( !bWeaponFiring )
		return;

	if (FireLoopSoundHandle != 0)
	{
		StopSound( FireLoopSoundHandle );
		FireLoopSoundHandle = 0;

		PlaySoundCue( FireOneBulletSound, SLOT_NONE, TransientSoundVolume,,,,false);
	}

	Super.StopFiringWeapon();

	bWeaponFiring = false;
}

//---------------------------------------------------------
//	Tick
//---------------------------------------------------------
function Tick( float DeltaTime )
{
	local gbxMuzzleFlash MuzFlash;

	// Hide the muzzle flash if in first-person mode because we only want 
	// to render it during PostRender().
	MuzFlash = gbxMuzzleFlash( FireDEMG42Static( FireMode[0] ).GetMuzzleFlash() );
	if (MuzFlash != None)
	{
		if ( m_fZoomed )
			MuzFlash.bHidden = true;
		else
			MuzFlash.bHidden = false;
	}

	if ( FireMode[0] != None )
	{
		if ( FireMode[0].bIsFiring && !bWeaponFiring )
		{
			StartFiringWeapon();
		}
		else if ( !FireMode[0].bIsFiring && bWeaponFiring )
		{
			StopFiringWeapon();
		}
	}

	Super.Tick( DeltaTime );
}

//---------------------------------------------------------
//	StopFire
//---------------------------------------------------------
simulated event StopFire(int Mode)
{
	if (FireLoopSoundHandle != 0)
	{
		StopSound( FireLoopSoundHandle );
		FireLoopSoundHandle = 0;

		PlaySoundCue( FireOneBulletSound, SLOT_NONE, TransientSoundVolume,,,,false);
	}

	bWeaponFiring = false;
	StopFiringWeapon();
	Super.StopFire( Mode );
}

defaultproperties
{
    ItemName="German MG42 Machinegun"

    FireModeClass(0)=FireDEMG42Static
    FireModeClass(1)=FireDEMG42Static
    FireModeClass(2)=FireMelee
    InventoryGroup=2
//	Mesh=Mesh'K_view_model_us.view_thompson'
//	m_NormalViewMesh=Mesh'K_view_model_us.view_thompson'
//	m_ZoomViewMesh=Mesh'K_view_model_us.view_thompson_crosshair'
	m_nCharacterBodyIdx=4
    BobDamping=0.5
    PickupClass=class'PickupWeapDEMP40_'
    EffectOffset=(X=100.0,Y=25.0,Z=-10.0)
	DisplayFOV=+50.0
    AttachmentClass=class'AttachmentDEMP40_'

    DrawScale=1.0
	PlayerViewOffset=(X=20.000,Y=0.000000,Z=-0.50000)
    SelectSound=None

	AIRating=+0.4
    CurrentRating=0.4

	FireLoopSoundName="W_MG42A.LOOP"
	FireOneBulletSoundName="W_MG42A.SINGLE"
	
	m_eWeaponType=WeaponPrimary
	MyMeleeDamageType=class'DamageUSGunButtMelee'

	DefaultAmmoCount(0)=400

	ZoomInAnim=aim_up
	ZoomOutAnim=aim_down
	SelectAnim=deploy
	PutDownAnim=holster
	IdleAnim(0)=idle_01
	IdleAnim(1)=idle_02
	IdleAnim(2)=idle_03

	WeaponBoneName=rifle_hold

	m_flPauseTimeExposed=0.7

	AmmoClipSize=400

	m_flEncumberance=1.0
	m_fCanSprint=true
	m_bIsAutomatic=true

	nIconX=334
	nIconY=0
	nIconW=24
	nIconH=99
}
