//=============================================================================
// German MG42 turret-mounted machine gun.
//
// Author: Neil Johnson, Gearbox Software.
//=============================================================================

class WeapDEMG42_ extends TurretWeapon;

//---------------------------------------------------------
//	Configurable Properties
//---------------------------------------------------------
var ()	Name	GunnerMountTag;				// Tag of the PathNode where the gunner should mount.
var ()	Name	MountEvent;
var ()	Name	DismountEvent;

//---------------------------------------------------------
//	Variables / Constants
//---------------------------------------------------------
var gbxContextUseTrigger	MyTrigger;			// The context-use trigger used to get on/off this weapon.
var Name			GunnerToDraft;
var Vector			BaseVector;
var Vector			LocalX, LocalY, LocalZ;		// The local axes of the mount.
var Vector			GunnerOffset;				// Used to position the gunner mesh.
var Vector			FeederOffset;				// Used to position the ammo feeder.
var Vector			SpotterOffset;				// Used to position the spotter.
var Vector			CameraStartOffset;
var Vector			CameraMountedOffset;
var Vector			CameraDismountedOffset;
var Vector			CameraOffset;
var Vector			CameraOffsetZoomed;
var bool			bIsMounting;
var bool			bIsDismounting;
var bool			bWeaponFiring;
var bool			bFirstPersonZoom;
var bool			bFreeCamera;
var bool			bDontRecruit;
var float			MountStartStamp;
var float			NextCrewCheckTime;			// The next timestamp at which to check for crew candidates.
var float			MaxRotSpeed;
var float			LastTimeSecs;
var float			CameraMountSpeed, CameraDismountSpeed;
var float			HorizCosTheta, VertCosTheta;
var float			CrossHairNextDrawTime, CrosshairDrawFreq;
var gbxPawn			Crew[3];					// The pawns manning this weapon.
var gbxPawn			Claimants[3];				// The pawns claiming this weapon for gunning, feeding, and spotting.
var Array<gbxPawn>	CandidateList;				// List of candidates for this weapon's crew.
var ActorLite	GunnerMountPoint;
var ActorLite	FeederMountPoint;
var ActorLite	SpotterMountPoint;
var MG42Factory	MyFactory;

var Range	PauseTime_Low;
var Range	PauseTime_High;

var Rotator	MyLastRot;

var const int HorizFieldUU;
var const int VertFieldUU;
var const int CameraPitchUU;

// The names of the animations to use nine-point blending with
// when using this weapon.
var Name	PlayerMountAnim, PlayerDismountAnim;
var Name	PawnForwardDownAnim, PawnForwardLevelAnim, PawnForwardUpAnim,
			PawnLeftDownAnim, PawnLeftLevelAnim, PawnLeftUpAnim,
			PawnRightDownAnim, PawnRightLevelAnim, PawnRightUpAnim;

enum VerticalFacing		{VF_Up, VF_Center, VF_Down};
enum HorizontalFacing	{HF_Left, HF_Center, HF_Right};

var const int	GUNNER;

simulated function LoadWeaponMesh()
{
	Super.LoadWeaponMesh();

	Mesh = Mesh( DynamicLoadObject( "k_view_model_de.mg42_crosshair", class'Mesh' ) );
	LinkMesh( Mesh );
	log(self$".LoadWeaponMesh() - "$Mesh);
}

//---------------------------------------------------------
//	PostNetBeginPlay
//---------------------------------------------------------
simulated function PostNetBeginPlay()
{
	Super.PostNetBeginPlay();
		
	GiveAmmo(0);

	if ( ThirdPersonActor == None )
	{
		ThirdPersonActor = Spawn(AttachmentClass,Owner);		
		InventoryAttachment(ThirdPersonActor).InitFor(self);
	}

	if ( FireLoopSoundName != "" )
		FireLoopSound = Sound( DynamicLoadObject( FireLoopSoundName, class'Sound' ) );
	if ( FireOneBulletSoundName != "" )
		FireOneBulletSound = Sound( DynamicLoadObject( FireOneBulletSoundName, class'Sound' ) );
	if ( StereoFireLoopSoundName != "" )
		StereoFireLoopSound = Sound( DynamicLoadObject( StereoFireLoopSoundName, class'Sound' ) );
	if ( StereoFireOneBulletSoundName != "" )
		StereoFireOneBulletSound = Sound( DynamicLoadObject( StereoFireOneBulletSoundName, class'Sound' ) );

	PlayAnim( 'Idle', 1.0 );
}

//---------------------------------------------------------
//	IsMachineGun
//---------------------------------------------------------
function bool IsMachineGun() 
{ 
	return true; 
}

//---------------------------------------------------------
//	SetFactory
//---------------------------------------------------------
function SetFactory( MG42Factory NewFactory )
{
	MyFactory = NewFactory;
	SetBase(None);

	BaseVector = Vector( MyFactory.Rotation );
}

//---------------------------------------------------------
//	AttachPathNodes
//---------------------------------------------------------
function AttachPathNodes()
{
	local ActorLite NavPoint;
	local name NavPointTag;

	// Locate the mount nodes.
	for (NavPoint=Level.NavigationPointList; NavPoint!=None; NavPoint=NavPoint.nextNavigationPoint)
	{
		if (NavPoint.bIsNavigationPoint)
			NavPointTag = NavigationPoint(NavPoint).Tag;
		else if (NavPoint.bIsNavigationPointLite)
			NavPointTag = NavigationPointLite(NavPoint).Tag;

		if ( NavPointTag == GunnerMountTag )
		{
			GunnerMountPoint = NavPoint;
			break;
		}
	}	

	if ( GunnerMountPoint != None )
	{
		if (GunnerMountPoint.bIsNavigationPoint)
			NavigationPoint(GunnerMountPoint).SetLocation( GetMyMountPoint() );
		else if (GunnerMountPoint.bIsNavigationPointLite)
			NavigationPointLite(GunnerMountPoint).SetLocation( GetMyMountPoint() );
	}
}

//---------------------------------------------------------
//	Landed
//---------------------------------------------------------
event Landed( vector HitLocation, vector HitNormal )
{
	// Adjust the location of the gunner mount to be _precisely_ where we want them to be.
	if ( GunnerMountPoint != None )
	{
		if (GunnerMountPoint.bIsNavigationPoint)
			NavigationPoint(GunnerMountPoint).SetLocation( GetMyMountPoint() );
		else if (GunnerMountPoint.bIsNavigationPointLite)
			NavigationPointLite(GunnerMountPoint).SetLocation( GetMyMountPoint() );
	}
}

//---------------------------------------------------------
//	HasPendingGunner
//---------------------------------------------------------
function bool HasPendingGunner()
{
	return (Claimants[GUNNER] != None);
}

//---------------------------------------------------------
//	PawnIsInCrew
//---------------------------------------------------------
function bool PawnIsInCrew( Pawn P ) 
{ 
	return P == Crew[GUNNER];
}

//---------------------------------------------------------
//	GetBestBurstLength
//---------------------------------------------------------
function GetBestBurstLength( ESuppressionLevel SuppressionLevel, out float MinLen, out float MaxLen )
{
	FireMode[0].GetBestBurstLength( SuppressionLevel, MinLen, MaxLen );
}

//---------------------------------------------------------
//	GetBestPauseTime
//---------------------------------------------------------
function GetBestPauseTime( ESuppressionLevel SuppressionLevel, out float MinLen, out float MaxLen )
{
	if ( SuppressionLevel == SL_UNSUPPRESSED )
	{
		MinLen = PauseTime_Low.Min;
		MaxLen = PauseTime_Low.Max;
	}
	else
	{
		MinLen = PauseTime_High.Min;
		MaxLen = PauseTime_High.Max;
	}
}

//---------------------------------------------------------
//	Tick
//---------------------------------------------------------
function Tick( float DeltaTime )
{
	local int x;
	local class<gbxAIController> MindClass;
	local gbxPawn ClosestPawn;
	local gbxMuzzleFlash MuzFlash;

	// Hide the muzzle flash if a players is in first-person mode because we only want 
	// to render it during PostRender().
	MuzFlash = gbxMuzzleFlash( FireDEMG42_( FireMode[0] ).GetMuzzleFlash() );
	if ( bFirstPersonZoom && Crew[GUNNER].IsA( 'gbxPlayerPawn' ) )
	{
		MuzFlash.bHidden = true;		
	}
	else
	{
		MuzFlash.bHidden = false;
	}

	if ( FireMode[0] != None )
	{		
		// Ensure that a gunner-less MG is not firing ad infinitum.
		if ( bWeaponFiring && ( Crew[GUNNER] == None || Crew[GUNNER].IsDead() ) )
		{
			StopFiringWeapon();
		}
		else if ( FireMode[0].bIsFiring && !bWeaponFiring && !bIsDismounting )
		{			
			StartFiringWeapon();
		}
		else if ( !FireMode[0].bIsFiring && bWeaponFiring )
		{
			StopFiringWeapon();
		}
	}

	if ( Level.TimeSeconds > NextCrewCheckTime )
	{
		NextCrewCheckTime = Level.TimeSeconds + 5.0;

		// Check if the claimants/crew are still alive.
		for(x=0; x < 3; ++x)
		{
			if ( !bIsMounting && ( bDontRecruit || ( Claimants[x] != None && Claimants[x].IsDead() ) ) )
			{
				Claimants[x] = None;
			}
		}
		for(x=0; x < 3; ++x)
		{
			if ( Crew[x] == None || Crew[x].IsDead() )
			{
				if ( x == GUNNER && Crew[GUNNER] != None )
				{
					StopFiringWeapon();
					Instigator = None;
					Ammo[0].Instigator = None;
					FireMode[0].Instigator = None;
				}

				Crew[x] = None;
			}
		}

		// Fill vacant crew positions.
		if ( Crew[GUNNER] == None && CheckOKToRecruit() )		
		{
			ClosestPawn = GetClosestCandidate();
			if ( ClosestPawn != None && ( Claimants[GUNNER] == None || ClosestPawn == Claimants[GUNNER] ) && ClosestPawn.CanCrewTurret( Self ) )
			{				
				MindClass = class<gbxAIController>( DynamicLoadObject( "gbxAI.MindMG42Gunner", class'Class' ) );
				Claimants[GUNNER] = ClosestPawn;
				ClosestPawn.CrewTurret( Self, MindClass, Job_MG42_Gunner );
			}
		}

	}

	Super.Tick( DeltaTime );
}

function ActivateOracle()
{
	if ( MyFactory.bHidden )  // is the 3rd person weapon currently hidden?
	{
		bFirstPersonZoom = false;
		MyFactory.ShowFactory();
	}
}

//---------------------------------------------------------
//	CheckOKToRecruit
//---------------------------------------------------------
function bool CheckOKToRecruit()
{
	return ( !bDontRecruit && ( CandidateList.Length > 0 || GunnerToDraft != '' ) );
}

//---------------------------------------------------------
//	GetDraftedGunner
//---------------------------------------------------------
function gbxPawn GetDraftedGunner()
{
	local gbxPawn DraftedGunner;

	if ( GunnerToDraft != '' )
	{
		foreach DynamicActors( class 'gbxPawn', DraftedGunner, GunnerToDraft )
		{
			if ( DraftedGunner.IsDead() )
			{
				GunnerToDraft = '';
				return None;
			}
			else
			{
				return DraftedGunner;
			}
		}
	}

	return None;
}

//---------------------------------------------------------
//	GetClosestCandidate
//---------------------------------------------------------
function gbxPawn GetClosestCandidate()
{
	local Pawn MountPointClaimant;
	local gbxPawn ClosestPawn;
	local float OneDist, ClosestDist;
	local int x;
	local bool bClosestPawnIsDraftee;

	ClosestPawn = None;
	ClosestDist = 99999;	

	// Get the draftee, if he exists.
	ClosestPawn = GetDraftedGunner();

	if ( ClosestPawn != None )
	{
		bClosestPawnIsDraftee = true;
	}
	else
	{
		bClosestPawnIsDraftee = false;

		for(x = 0; x < CandidateList.Length; ++x)
		{
			if ( CandidateList[x].IsDead() )
			{
				CandidateList.Remove( x, 1 );
				continue;
			}

			if ( /*CandidateList[x] == Claimants[GUNNER] ||*/
				CandidateList[x] == Crew[GUNNER] ||
				!CandidateList[x].CanCrewTurret( Self ) )
			{
				continue;
			}

			OneDist = VSizeIgnoreZ( CandidateList[x].Location - Self.Location );
			if ( OneDist < ClosestDist )
			{
				ClosestDist = OneDist;
				ClosestPawn = CandidateList[x];
			}
		}
	}

	// Check if anyone has claimed the mount point.
	if ( GunnerMountPoint.bIsNavigationPointLite )
		MountPointClaimant = NavigationPointLite( GunnerMountPoint ).Claimant;
	else
		MountPointClaimant = NavigationPoint( GunnerMountPoint ).Claimant;

	// If the mount point's claimant is the guy we picked, groovy, go ahead and recruit him.
	// If the mount point's claimant is NOT the guy we picked, skip recruiting altogether if we have a
	// handpicked draftee in mind.  If we don't have a draftee, hell, just try to recruit the claimant.
	if ( MountPointClaimant == None || MountPointClaimant == ClosestPawn )
		return ClosestPawn;
	else if ( !bClosestPawnIsDraftee && MountPointClaimant.IsA( 'gbxPawn' ) && gbxPawn( MountPointClaimant ).CanCrewTurret( Self ) )
		return gbxPawn( MountPointClaimant );
	else
		return None;
}

//---------------------------------------------------------
//	CalculateLerpedRotator
//---------------------------------------------------------
function Rotator CalculateLerpedRotator( Rotator Rot1, Rotator Rot2, float LerpSpeed, out int YawDiff )
{
	local bool bDidSubtract;
	local bool bIsNegative;
	local Rotator TempRot;
	
	YawDiff = Rot1.Yaw - Rot2.Yaw;
	bIsNegative = (YawDiff < 0);

	YawDiff = abs(YawDiff);

	if ( YawDiff > 32768)
	{
		YawDiff = 65536 - YawDiff;
		bDidSubtract = true;
	}

	if ( LastTimeSecs == 0 )
		LastTimeSecs = Level.TimeSeconds;

	if ( ( bDidSubtract && bIsNegative ) || ( !bDidSubtract && !bIsNegative ) )
	{
		// Rot1 is to the RIGHT of Rot2; YawDiff is the angle (in Unreal Units).
		TempRot.Yaw = (Rot1.Yaw - LerpSpeed*(Level.TimeSeconds - LastTimeSecs )*YawDiff) & 65535;
	}
	else
	{
		// Rot1 is to the LEFT of Rot2; YawDiff is the angle (in Unreal Units).
		TempRot.Yaw = (Rot1.Yaw + LerpSpeed*(Level.TimeSeconds - LastTimeSecs )*YawDiff) & 65535;
	}

	TempRot.Pitch = Rot2.Pitch;
	TempRot.Roll = Rot2.Roll;

	LastTimeSecs = Level.TimeSeconds;
	return TempRot;
}

//---------------------------------------------------------
//	FaceRotation
//---------------------------------------------------------
// Make the turret weapon rotate to face a particular direction.
function FaceRotation( Rotator Rot ) {

	// OK, now the aim is to end up with two alpha values for
	// animation channels 1 and 2.  These channels, along with
	// base channel 0, will be blended to produce a smooth transitioning
	// of the gunner's pose as the user moves the mouse.
	//
	// Channel 0 plays the 'level-forward' gunning animation at all times.
	// Channel 1 plays an 'edge animation' (i.e. 'level-left', 'level-right',
	// 'up-forward', or 'down-forward').  Channel 2 plays a 'diagonal animation'
	// (i.e. 'up-left', 'up-right', 'down-left', or 'down-right').
	//
	// The blending occurs progressively; that is, the edge animation is
	// blended over the base animation first, and then the diagonal animation
	// is blended over that result in turn.
	//
	local Vector Aim, AimXY, AimXZ, D, V;
	local Float DotH, DotV, DotCamera, Alpha1, Alpha2, m;
	local HorizontalFacing HorizFace;
	local VerticalFacing VertFace;
	local NinePointDir DirDiag, DirEdge;
	local Rotator LerpedRot;
	local int YawDiff;

	// Sanity check.
	if (Crew[GUNNER] == None)
	{
		log("Error: MG42 Gunner is null!", 'ScriptWarning');
		return;
	}
	if ( bIsDismounting )
	{
		return;
	}

	GetAxes(MyFactory.Rotation, LocalX, LocalY, LocalZ);

	// Smoothly interpolate the rotation.
	//
	// If the camera is facing to the rear of the weapon, interpolate nice and slowly when the camera crosses the Prime Meridian.
	//	
	// Lerp iff: the gunner is AI-controlled, the difference in movement since the last tick is large, or the camera is facing to the rear.
	if ( !Crew[GUNNER].IsHumanControlled() || ( !bFirstPersonZoom && (YawDiff > 2000 || DotCamera < 0) ) )
	{
		DotCamera = Vector(Rot) dot Vector(MyFactory.Rotation);
		if ( DotCamera < 0 )
			MaxRotSpeed = 1;	
		else
			MaxRotSpeed = Default.MaxRotSpeed;	
		
		LerpedRot = CalculateLerpedRotator( MyLastRot, Rot, MaxRotSpeed, YawDiff );
		Rot = LerpedRot;		
	}
	
	RestrictYawAndPitch( Rot );	
	
	// Adjust the gun's actual rotation.
	SetRotation(Rot);
	SetLocation( MyFactory.Rebalance( Rot ) );
	
	MyLastRot = Rot;

	// Next, we'll calculate DotH and DotV again, only using the gun's actual
	// current orientation (due to the motion-delay placed on the gun, the
	// actual orientation may differ from the requested orientation.
	Aim = Vector(Rotation);
	AimXZ = Aim - ((Aim dot LocalY) * LocalY);
	AimXY = Aim - ((Aim dot LocalZ) * LocalZ);
	AimXZ = Normal(AimXZ);
	AimXY = Normal(AimXY);

	DotH = AimXY dot LocalY;
	DotV = AimXZ dot LocalZ;

	// The extants of movement are -45deg to 45deg in the horizontal direction,
	// and -15deg to 15deg in the vertical direction,
	// so the min/max dot products will be +/-0.7071 and +/-0.2588.
	// Remap the dot-prods into the space -1.0 to 1.0.
	DotH /= HorizCosTheta;
	DotV /= VertCosTheta;

	DotH = FClamp( DotH, -1.0, 1.0 );
	DotV = FClamp( DotV, -1.0, 1.0 );

	// Determine what quadrant the aim vector is in.
	if (DotV < 0)
		VertFace = VF_Down;
	else
		VertFace = VF_Up;

	//if (DotH < 0)
	if (DotH > 0)
		HorizFace = HF_Right;
	 else
		HorizFace = HF_Left;


	// Absolute-value the dot prods so that we can simplify calculations below.
	DotH = abs(DotH);
	DotV = abs(DotV);

	//_DotH = DotH;
	//_DotV = DotV;

	// From the quadrant information, determine which two
	// diagonal and edge animations to use.
	if (VertFace == VF_Down) {
		if (HorizFace == HF_Left) {
			DirDiag = NPD_LowLeft;
			if (DotH >= DotV)
				DirEdge = NPD_MidLeft;
			else
				DirEdge = NPD_LowCenter;
		} else {
			DirDiag = NPD_LowRight;
			if (DotH >= DotV)
				DirEdge = NPD_MidRight;
			else
				DirEdge = NPD_LowCenter;
		}
	} else {	// VF_Up
		if (HorizFace == HF_Left) {
			DirDiag = NPD_UpLeft;
			if (DotH >= DotV)
				DirEdge = NPD_MidLeft;
			else
				DirEdge = NPD_UpCenter;
		} else {
			DirDiag = NPD_UpRight;
			if (DotH >= DotV)
				DirEdge = NPD_MidRight;
			else
				DirEdge = NPD_UpCenter;
		}
	}

	// Step 4: Figure the alpha blending values to use for the
	// two channels.  Point D is the point in 2D space that we found
	// above.  Point V is a point of intersection between the line thru
	// D and the nearest corner with the x- or the y-axis.
	//
	// Alpha1 is found by finding the distance from the origin to V.
	// Alpha2 is the distance between V and D.
	//
	// By absolute-valuing the dot prods above, we reduce the problem
	// to quadrant 1 (at this point, we have the two animations we need.
	// Except for knowing whether our edge-animation is horizontal or vertical,
	// quadrant information is no longer relevant.
	//
	D.X = DotH;	// 0.0 to 1.0, no negative values.
	D.Y = DotV;	// 0.0 to 1.0, no negative values.
	D.Z = 0;

	if (D.X == 1.0)	{	// Avoid divide-by-zero errors in the slope formula.
		Alpha1 = 1.0;
		Alpha2 = D.Y;
	} else {
		// m is the slope of the line through D and the corner (1,1).
		m = (1.0 - D.Y) / (1.0 - D.X);

		if (DirEdge == NPD_MidRight || DirEdge == NPD_MidLeft) {

			// The edge-animation is horizontal, so find x when
			// the line intercepts the x-axis (when y=0).
			Alpha1 = D.X - (D.Y / m);	// -b/m in intercept-slope formula.
			V.x = Alpha1;
			V.y = 0;
			V.z = 0;

		} else {	// Vertical edge (NPD_UpCenter or NPD_LowCenter).
			// The edge-animation is vertical, so find y when
			// the line intercepts the y-axis (when x=0).
			Alpha1 = D.Y - (m * D.X);	// 'b' in intercept-slope formula.
			V.x = 0;
			V.y = Alpha1;
			V.z = 0;
		}

		// Alpha2 is the proportion of the length of vector D - V
		// to that of vector (1,1) - V.  The equation used below
		// is the optimized form of the one in comments.
		Alpha2 = VSize(D - V) / (1.41421 - 0.41421 * Alpha1);	// 0.41421 == sqrt(2)
		//Alpha2 = VSize(D - V) / VSize(Vect(1,1,0) - V);		// Keep this for reference.
	}

	// Play the gunning animations, supplying the edge-animation,
	// the diagonal-animation, and their blending factors.
	Crew[GUNNER].DoNinePointBlending( DirEdge, Alpha1, DirDiag, Alpha2 );
	MyFactory.DoNinePointBlending( DirEdge, Alpha1, DirDiag, Alpha2 );
}

//---------------------------------------------------------
//	PreparePawnToMount
//---------------------------------------------------------
function PreparePawnToMount(gbxPawn gbxP)
{
	if ( gbxP != Claimants[GUNNER] )
	{
		log("Error! Pawn "$gbxP$" attempting to call WeapDEMG42_::PreparePawnToMount() without claiming first.", 'ScriptWarning');
		return;
	}

	// Do some initial cleanup.
	CleanUpProperties();

	if ( gbxP.IsHumanControlled() )
		gbxP.SetTurretAnims( PlayerMountAnim, PlayerDismountAnim );

	gbxP.PrepareToMount();

	bIsMounting = true;
}

//---------------------------------------------------------
//	CompleteMount
//---------------------------------------------------------
function CompleteMount( gbxPawn gbxP )
{
	local int Mode;

	if ( gbxP == Claimants[GUNNER] )
	{
		if ( gbxP != Claimants[GUNNER] )
		{
			log("Pawn "$gbxP$" attempting to call WeapDEMG42_::CompleteMount() without claiming first.", 'ScriptWarning');
			return;
		}

		SetInstaKill( false );	// Just in case it was left with insta-kill on.

		Crew[GUNNER] = gbxP;
		Claimants[GUNNER] = None;

		Instigator = Crew[GUNNER];
		if ( Ammo[0] != None )
			Ammo[0].Instigator = Crew[GUNNER];
		if ( FireMode[0] != None )
			FireMode[0].Instigator = Instigator;
		if ( ThirdPersonActor != None )
			ThirdPersonActor.Instigator = Instigator;
		
		Crew[GUNNER].SetNinePointAnimations(
			PawnForwardDownAnim, PawnForwardLevelAnim, PawnForwardUpAnim,
			PawnLeftDownAnim, PawnLeftLevelAnim, PawnLeftUpAnim,
			PawnRightDownAnim, PawnRightLevelAnim, PawnRightUpAnim);

		Crew[GUNNER].Velocity = vect(0,0,0);
		Crew[GUNNER].bRotateToDesired = false;
		Crew[GUNNER].DesiredRotation = MyFactory.Rotation;		

		GetAxes(MyFactory.Rotation, LocalX, LocalY, LocalZ);
		Crew[GUNNER].SetLocation( GetMyMountPoint() );
		Crew[GUNNER].SetRotation( MyFactory.Rotation );

		//Crew[GUNNER].ResetLookAtImmediate(true, true, true);
		//Crew[GUNNER].StopLookAt( true, true, true );

		Self.SetRotation( MyFactory.Rotation );
		MyLastRot = MyFactory.Rotation;

		bIsMounting = false;

		for( Mode = 0; Mode < NUM_FIRE_MODES; Mode++ )
			if (FireMode[Mode] != None)
//				FireMode[Mode].InitEffects();
				FireMode[Mode].GetMuzzleFlash();

		Crew[GUNNER].SetWeapon( Self );

		if ( Crew[GUNNER].MyUnit != None )
		{
			Crew[GUNNER].MyUnit.MemberGainedMachineGun( Crew[GUNNER] );
		}

		// Send out a mount event to anyone who may be interested.
		TriggerEvent( MountEvent, Self, Crew[GUNNER] );	

		Crew[GUNNER].PlayWaiting();

		// Force player to be zoomed in after mount.
		SetFirstPersonZoom( true );

		MyTrigger.bDontDisplayMessage = true;
	}
}

//---------------------------------------------------------
//	CenterWeapon
//---------------------------------------------------------
function MountFailed( gbxPawn gbxP )
{
	Super.MountFailed( gbxP );

	// Reactivate the trigger.
	MyTrigger.bDontDisplayMessage = false;
}

//---------------------------------------------------------
//	CenterWeapon
//---------------------------------------------------------
function CenterWeapon()
{
	SetRotation( MyFactory.Rotation );
	SetLocation( MyFactory.Rebalance( MyFactory.Rotation ) );
	MyFactory.StopBlending();	
}

//---------------------------------------------------------
//	PreparePawnToDismount
//---------------------------------------------------------
function PreparePawnToDismount( gbxPawn gbxP )
{
	// Check that the pawn passed in is the gunner.
	if (gbxP != Crew[GUNNER])
		return;	

	StopFiringWeapon();

	gbxP.StopNinePointBlending();
	gbxP.SetRotation( MyFactory.Rotation );
	gbxP.Controller.SetRotation( MyFactory.Rotation );

	if (gbxP.bUseRootMotion)
		gbxP.SetPhysics(PHYS_RootMotion);
	else
		gbxP.SetPhysics(PHYS_Walking);

	SetFirstPersonZoom( false );

	bIsDismounting = true;
}

//---------------------------------------------------------
//	CompleteDismount
//---------------------------------------------------------
function CompleteDismount( gbxPawn gbxP )
{
	if ( gbxP == Crew[GUNNER] )
	{
		Instigator = None;
		Ammo[0].Instigator = None;
		FireMode[0].Instigator = None;

		if ( Crew[GUNNER].MyUnit != None )
		{
			Crew[GUNNER].MyUnit.MemberLostMachineGun( Crew[GUNNER] );
		}

		SetInstaKill( false );	// Just in case it was left with insta-kill on.		
		CleanUpProperties();
		CameraOffset.X = 0.0;
		CameraOffset.Z = 0.0;

		// Send out a dismount event to anyone who may be interested.
		TriggerEvent( DismountEvent, Self, gbxP );

		// Reactivate my trigger, which was made inactive when used.
		MyTrigger.bDontDisplayMessage = false;
	}

	gbxP.Weapon = None;  // hack so SetWeapon() will call ChangedWeapon() - jlb

	if ( gbxP.LastWeapon != None )
		gbxP.SetWeapon( gbxP.LastWeapon );
	else
		gbxP.Controller.SwitchToBestWeapon();
	
	CenterWeapon();
}

//---------------------------------------------------------
//	CleanUpProperties
//---------------------------------------------------------
function CleanUpProperties()
{
	Crew[GUNNER] = None;
	bFirstPersonZoom = false;
	bIsMounting = false;
	bIsDismounting = false;
	CameraOffset.X = 0.0;
	CameraOffset.Z = 0.0;
}

//---------------------------------------------------------
//	InformCrewToGetUp
//---------------------------------------------------------
function InformCrewToGetUp( gbxPawn Caller)
{
	local int x;

	//for ( x = 0; x < Crew.Length; ++x )
	for ( x = 0; x < 3; ++x )
	{
		if ( Crew[x] != None && Crew[x] != Caller )
			Crew[x].BreakOutOfJob();
	}
}

//---------------------------------------------------------
//	GunnerDied
//---------------------------------------------------------
function GunnerDied( gbxPawn gbxP )
{
	if ( gbxP == Crew[GUNNER] )
	{
		bFirstPersonZoom = false;
		bIsMounting = false;
		bIsDismounting = false;

		SetRelativeRotation(Rot(0,0,0));

		Crew[GUNNER].bCollideWorld = true;
		Crew[GUNNER].SetLocation( GetMyMountPoint() );

		if ( Crew[GUNNER].MyUnit != None )
		{
			Crew[GUNNER].MyUnit.MemberLostMachineGun( Crew[GUNNER] );
		}

		SetInstaKill( false );	// Just in case it was left with insta-kill on.

		Crew[GUNNER] = None;
		Claimants[GUNNER] = None;

		// Reactivate my trigger, which was made inactive when used.
		MyTrigger.bDontDisplayMessage = false;
	}
}

//---------------------------------------------------------
//	GetMyMountPoint
//---------------------------------------------------------
function Vector GetMyMountPoint()
{
	local Vector X,Y,Z;
	GetAxes( MyFactory.Rotation, X,Y,Z );
	
	return MyFactory.Location + GunnerOffset.X*X + GunnerOffset.Y*Y + GunnerOffset.Z*Z;
}

//---------------------------------------------------------
//	GetMountPoint
//---------------------------------------------------------
function ActorLite GetMountPoint()
{
	return GunnerMountPoint;
}

//---------------------------------------------------------
//	Trigger
//---------------------------------------------------------
event Trigger(Actor Other, Pawn EventInstigator)
{
	local gbxPawn gbxP;
	local Actor EncroachedActor;
	local bool bEncroached;

	gbxP = gbxPawn( EventInstigator );

	// Skip the action instigator is in the middle of a reload.
	if ( gbxP.IsTransitioning() || (gbxP.Weapon != Self &&( gbxP.Weapon.IsReloading() || gbxP.Weapon.IsThrowingGrenade())))  //NAC:  3/10/05 added additional check to see if we are throwing a grenade
		return;

	if ( Crew[GUNNER] == gbxP )	// Gunner wishes to dismount.
	{
		Crew[GUNNER].DismountTurretPlayer(true);
		MountStartStamp=Level.TimeSeconds;
	}
	else						// Gunner wishes to mount.
	if ( Crew[GUNNER] == None )
	{
		Claimants[GUNNER] = gbxP;

		if ( Claimants[GUNNER].IsHumanControlled() )
		{
			bEncroached = Claimants[GUNNER].CheckEncroachment( GetMyMountPoint(), MyFactory.Rotation, EncroachedActor );

			if ( bEncroached && EncroachedActor.IsA( 'Pawn' ) && Pawn( EncroachedActor ).IsDead() )
			{
				// TBD: Move dead pawn out of the way somehow
			}
			else
			if ( bEncroached && !EncroachedActor.IsA( 'Pawn' ) )
			{
				bEncroached = false;
			}

			if ( !bEncroached && TryToPlaceGunner( Claimants[GUNNER] ) )
			{
				PreparePawnToMount( Claimants[GUNNER] );
				Claimants[GUNNER].MountTurretPlayer( Self, true );
				if ( Claimants[GUNNER].bHasStartedMountAnim )
				{					
					Claimants[GUNNER].SetRotation( MyFactory.Rotation );
					Claimants[GUNNER].Controller.SetRotation( MyFactory.Rotation );
				}
				MountStartStamp=Level.TimeSeconds;
			}
			else
			{
				MyTrigger.bDontDisplayMessage = false;	// We have to do this because gbxContextUseTriggers disable themselves when UsedBy.
				log("WARNING: MG42 encroachment check failed.  EncroachingActor=="$EncroachedActor, 'ScriptWarning' );
			}
		}		
	}
}

function bool TryToPlaceGunner( gbxPawn TheGunner )
{
	local int i;
	local Vector TrialMountLocation;

	TrialMountLocation = GetMyMountPoint();

	for ( i = 0; i < 10; ++i )
	{
		if ( TheGunner.SetLocation( TrialMountLocation ) )
		{
			return true;
		}
		else
		{		
			log("NAJ**** "$Self.Name$" Failed to mount location; adding 5 units to Z-height.", 'naj' );
			TrialMountLocation.Z += 5;
		}
	}

	return false;
}

//---------------------------------------------------------
//	StartFiringWeapon
//---------------------------------------------------------
function StartFiringWeapon()
{
	if (bAimingAtFriendly && Instigator.IsA('gbxPawn'))
	{
		bFiringAtFriendly = true;
	}
	
	if ( bWeaponFiring || bIsDismounting )
		return;

	// RSC: Don't fire if we are issuing a command
	if( !Instigator.Controller.AllowFire() )
		return;

	if( !BAimingAtFriendly ) { // RSC: dont play a sound if aiming at a friendly
		if( Instigator.IsLocallyControlled() )
		{
			if ( StereoFireLoopSound != None && (Instigator.Controller == Level.GetLocalPlayerController()) )
			{
				FireLoopSoundHandle = PlaySound( StereoFireLoopSound, SLOT_None, TransientSoundVolume,,,,false);
			}
			else
			{
				FireLoopSoundHandle = PlaySound( FireLoopSound, SLOT_WeaponFire1, TransientSoundVolume,,,,true);
			}
		}
		else
		{
			FireLoopSoundHandle = PlaySound( FireLoopSound, SLOT_WeaponFire1, TransientSoundVolume,,,,true);
		}
	}
			
	if ( FireMode[0].IsA( 'FireDEMG42_' ) )
		FireDEMG42_( FireMode[0] ).StartFiringWeapon();

	if ( bFirstPersonZoom )
		LoopAnim( 'Fire', 1.0, 0.0 );

	Super.StartFiringWeapon();

	bWeaponFiring = true;
}

//---------------------------------------------------------
//	StopFiringWeapon
//---------------------------------------------------------
function StopFiringWeapon()
{
	if ( FireMode[0].IsA( 'FireDEMG42_' ) )
		FireDEMG42_( FireMode[0] ).StopFiringWeapon();

	if ( !bWeaponFiring )
		return;

	if (FireLoopSoundHandle != 0)
	{
		//StopSound( FireLoopSoundHandle );
		//FireLoopSoundHandle = 0;

		if (Instigator.IsLocallyControlled())
		{
			if ( StereoFireOneBulletSound != None && (Instigator.Controller == Level.GetLocalPlayerController()) )
			{
				PlaySound( StereoFireOneBulletSound, SLOT_NONE, TransientSoundVolume,,,,false);
			}
			else
			{
				PlaySound( FireOneBulletSound, SLOT_NONE, TransientSoundVolume,,,,true);
			}
		}
		else
		{
			PlaySound( FireOneBulletSound, SLOT_NONE, TransientSoundVolume,,,,true);
		}

		//PlaySound( FireOneBulletSound, SLOT_NONE, TransientSoundVolume,,,,false);

		StopSound( FireLoopSoundHandle );
		FireLoopSoundHandle = 0;
	}

	if ( bFirstPersonZoom )
		PlayAnim( 'Idle', 1.0 );

	Super.StopFiringWeapon();

	bWeaponFiring = false;
}

function ConsumeAmmo(int Mode, float load) {}

simulated event RenderOverlays( Canvas Canvas ) {}

simulated function DrawHud(canvas Canvas, gbxHud Hud, FontInfo Fonts, int nPass) {}

simulated function DrawCrossHair( canvas Canvas )
{
	local float XLength, YLength;
	local  PlayerController PlayerOwner;
	local Vector HitLoc, HitNorm;
	local Actor HitActor;
	local HUD	Hud;

	if ( bFirstPersonZoom )
		return;

	if ( Instigator != None && Instigator.IsHumanControlled() )
	{
		PlayerOwner = PlayerController(Instigator.Controller);
			
		if ( PlayerOwner.DisableCrosshair() )
			return;	

		if ( PlayerOwner.iCrossHairIndex == 0)
			return;

		Hud = PlayerOwner.MyHud;

		if (Hud == None)
		{
			XLength = 32;
			YLength = 32;
		}
		else
		{
			XLength = 32 * Hud.flMinScaleX;
			YLength = 32 * Hud.flMinScaleY;
		}

		if ( Level.TimeSeconds >= CrosshairNextDrawTime )
		{
			HitActor = Trace( HitLoc, HitNorm, Location + 15000 * Vector( Rotation ), Location, false );
			
			if ( HitActor != None )
			{
				Canvas.Style = ERenderStyle.STY_Additive;
				Canvas.SetDrawColor(255,255,255,96);
				Canvas.DrawTile3D( m_TestCrosshairs[PlayerOwner.iCrossHairIndex - 1], HitLoc, XLength, YLength, 0, 0, m_TestCrosshairs[PlayerOwner.iCrossHairIndex - 1].USize, m_TestCrosshairs[PlayerOwner.iCrossHairIndex - 1].VSize);
			}

			CrossHairNextDrawTime = Level.TimeSeconds + CrosshairDrawFreq;
		}
	}		
}

//---------------------------------------------------------
//	FlashMuzzleFlash
//---------------------------------------------------------
// For 3rd person only.  See PostRender() in this class for 1st person muzzle flashes.
function FlashMuzzleFlash()
{
	local gbxMuzzleFlash MuzFlash;

	if ( Crew[GUNNER] == None || bIsDismounting )
		return;

	if ( FireMode[0].IsA( 'FireDEMG42_' ) )
	{
		MuzFlash = gbxMuzzleFlash( FireDEMG42_( FireMode[0] ).GetMuzzleFlash() );

		// We only want to hide the flash if a players is firing it
		if( bFirstPersonZoom && Crew[GUNNER].IsA( 'gbxPlayerPawn' ))	
		{
			MuzFlash.bHidden = true;		// PostRender() will unhide it and render it.
			MuzFlash.Flash();
		}
		else
		{
			MuzFlash.SetLocation( GetMuzzleLocationWorld() );
			MuzFlash.SetRotation( Rotation );
			MuzFlash.Flash();
		}
	}
}

//---------------------------------------------------------
//	AnimEnd
//---------------------------------------------------------
simulated function AnimEnd(int channel)
{
	if ( bFirstPersonZoom )
	{
		if ( bWeaponFiring )
		{
			//PlayAnim( 'Fire', 1.0 );
		}			
		else
		{
			PlayAnim( 'Idle', 1.0 );
		}
	}
	else
		Super.AnimEnd( channel );
}

//---------------------------------------------------------
//	Fire
//---------------------------------------------------------
simulated function Fire(float F)
{	
	// RSC: Don't fire if we are issuing a command. BB: Or if it is being dismounted
	if( !Instigator.Controller.AllowFire() || bIsDismounting )
		return;

	StartFiringWeapon();
	Super.Fire(F);	
}

//---------------------------------------------------------
//	PlayFiringAnim
//---------------------------------------------------------
simulated function PlayFiringAnim()
{
	// Deprecate. 
}

//---------------------------------------------------------
//	StopFire
//---------------------------------------------------------
simulated event StopFire(int Mode)
{
	if (FireLoopSoundHandle != 0)
	{
		//StopSound( FireLoopSoundHandle );
		//FireLoopSoundHandle = 0;

		if (Instigator.IsLocallyControlled())
		{
			if ( StereoFireOneBulletSound != None && (Instigator.Controller == Level.GetLocalPlayerController()) )
			{
				PlaySound( StereoFireOneBulletSound, SLOT_NONE, TransientSoundVolume,,,,false);
			}
			else
			{
				PlaySound( FireOneBulletSound, SLOT_NONE, TransientSoundVolume,,,,true);
			}
		}
		else
		{
			PlaySound( FireOneBulletSound, SLOT_NONE, TransientSoundVolume,,,,true);
		}

		//PlaySound( FireOneBulletSound, SLOT_NONE, TransientSoundVolume,,,,false);
		StopSound( FireLoopSoundHandle );
		FireLoopSoundHandle = 0;
	}

	bWeaponFiring = false;
	StopFiringWeapon();
	Super.StopFire( Mode );
}

//---------------------------------------------------------
//	SetFirstPersonZoom
//---------------------------------------------------------
function SetFirstPersonZoom( bool bZoomed )
{
	local PlayerController PC;

	// don't change zoom settings if currently in OracleCam (SA View) mode...
	if( Level.Game.bOracleCamActive )
		return;

	bFirstPersonZoom = bZoomed;

	if ( Crew[GUNNER] == None || !Crew[GUNNER].IsA( 'gbxPlayerPawn' ) )
		return;

	if ( bZoomed )
	{
		MyFactory.HideFactory();
		//Crew[GUNNER].bHidden = true;

		PC = PlayerController( Crew[GUNNER].Controller );
		PC.DesiredFOV = DisplayFOV;
	}
	else
	{
		MyFactory.ShowFactory();		
		//Crew[GUNNER].bHidden = false;

		PC = PlayerController( Crew[GUNNER].Controller );
		PC.DesiredFOV = PC.DefaultFOV;
	}
}

//---------------------------------------------------------
//	RestrictYawAndPitch
//---------------------------------------------------------
function RestrictYawAndPitch( out Rotator ControllerRot )
{
	local int Diff;
	local bool bIsNegative, bDidSubtract;
	local int FactoryYaw, FactoryPitch;

	ControllerRot.Yaw = ControllerRot.Yaw & 65535;
	ControllerRot.Pitch = ControllerRot.Pitch & 65535;
	FactoryYaw = MyFactory.Rotation.Yaw & 65535;
	FactoryPitch = MyFactory.Rotation.Pitch & 65535;

	Diff = ControllerRot.Yaw - FactoryYaw;

	bIsNegative = (Diff < 0);

	Diff = abs(Diff);

	if ( Diff > 32768)
	{
		Diff = 65536 - Diff;
		bDidSubtract = true;
	}
	else
	{
		bDidSubtract = false;
	}

	if ( Diff > HorizFieldUU )
	{
		if ( ( bDidSubtract && bIsNegative ) || ( !bDidSubtract && !bIsNegative ) )
		{
			// ControllerRot is to the RIGHT of MyFactory.
			ControllerRot.Yaw = (FactoryYaw + HorizFieldUU) & 65535;
		}
		else
		{
			// ControllerRot is to the LEFT of MyFactory.
			ControllerRot.Yaw = (FactoryYaw - HorizFieldUU) & 65535;
		}
	}

	// Pitch.
	Diff = ControllerRot.Pitch - FactoryPitch;
	bIsNegative = ( Diff < 0 );
	Diff = abs( Diff );

	if ( Diff > 32768 )
	{
		Diff = 65536 - Diff;
		bDidSubtract = true;
	}
	else
	{
		bDidSubtract = false;
	}

	if ( Diff > VertFieldUU )
	{
		if ( ( bDidSubtract && bIsNegative ) || ( !bDidSubtract && !bIsNegative ) )
			ControllerRot.Pitch = FactoryPitch + VertFieldUU;
		else
			ControllerRot.Pitch = FactoryPitch - VertFieldUU;
	}
}

//---------------------------------------------------------
//	RestrictPitchOnly
//---------------------------------------------------------
function RestrictPitchOnly( out Rotator ControllerRot )
{
	local int Diff;
	local bool bIsNegative, bDidSubtract;
	local int FactoryYaw, FactoryPitch;

	ControllerRot.Yaw = ControllerRot.Yaw & 65535;
	ControllerRot.Pitch = ControllerRot.Pitch & 65535;
	FactoryYaw = MyFactory.Rotation.Yaw & 65535;
	FactoryPitch = MyFactory.Rotation.Pitch & 65535;

	// Pitch.
	Diff = ControllerRot.Pitch - FactoryPitch;
	bIsNegative = ( Diff < 0 );
	Diff = abs( Diff );

	if ( Diff > 32768 )
	{
		Diff = 65536 - Diff;
		bDidSubtract = true;
	}

	if ( Diff > VertFieldUU )
	{
		if ( ( bDidSubtract && bIsNegative ) || ( !bDidSubtract && !bIsNegative ) )
			ControllerRot.Pitch = FactoryPitch + VertFieldUU;
		else
			ControllerRot.Pitch = FactoryPitch - VertFieldUU;
	}
}

//---------------------------------------------------------
//	CalcThirdPersonView
//---------------------------------------------------------
function CalcThirdPersonView( gbxPawn thePawn, out actor ViewActor, out vector CameraLocation, out rotator CameraRotation )
{
	local Vector X,Y,Z;
	local Rotator ControllerRot;
	local float T;

	ViewActor = thePawn;

	CameraLocation = thePawn.Location;

	if ( bFirstPersonZoom )
	{
		CameraOffset = CameraOffsetZoomed;
	}
	else
	if ( bIsMounting )
	{
		// Player is mounting turret.  Zoom out.
		T = FClamp( ( Level.TimeSeconds - MountStartStamp ) * CameraMountSpeed, 0, 1 );
		CameraOffset = CameraStartOffset + T*( CameraMountedOffset - CameraStartOffset );
	}
	else
	if ( bIsDismounting )
	{
		// Player is dismounting turret.  Zoom in.
		T = FClamp( ( Level.TimeSeconds - MountStartStamp ) * CameraDismountSpeed, 0, 1 );
		CameraOffset = CameraMountedOffset + T*( CameraDismountedOffset - CameraMountedOffset );
	}
	else		
	{
		CameraOffset = CameraMountedOffset;
	}

	// Restrict the yaw and pitch to the cone of fire.
	if ( Crew[GUNNER] != None )
	{
		ControllerRot = Crew[GUNNER].Controller.Rotation;

		// Yaw.
		if ( !bFreeCamera )
		{
			RestrictYawAndPitch( ControllerRot );
		}
		else
		{
			//ControllerRot.Yaw = ControllerRot.Yaw & 65535;
			RestrictPitchOnly( ControllerRot );
		}

		Crew[GUNNER].Controller.SetRotation( ControllerRot );
	}

	if ( !bIsMounting && !bIsDismounting )
	{
		CameraRotation = ControllerRot;
	}

	GetAxes( CameraRotation, X,Y,Z );
	CameraLocation = CameraLocation - CameraOffset.X*X + CameraOffset.Y*Y + CameraOffset.Z*Z;

	CameraRotation.Pitch -= CameraPitchUU;
}

//---------------------------------------------------------
//	CalcFirstPersonView
//---------------------------------------------------------
function CalcFirstPersonView( gbxPawn thePawn, out actor ViewActor, out vector CameraLocation, out rotator CameraRotation )
{
	local Vector X,Y,Z;
	local Rotator ControllerRot;

	ViewActor = Self;

	CameraOffset.X = CameraOffsetZoomed.X;
	CameraOffset.Y = CameraOffsetZoomed.Y;
	CameraOffset.Z = CameraOffsetZoomed.Z;

	GetAxes( Self.Rotation, X,Y,Z );
	CameraLocation = Self.Location + CameraOffset.X*X + CameraOffset.Y*Y + CameraOffset.Z*Z;

	// Restrict the yaw to the cone of fire.
	if ( Crew[GUNNER] != None )
	{
		ControllerRot = Crew[GUNNER].Controller.Rotation;

		RestrictYawAndPitch( ControllerRot );

		CameraRotation = ControllerRot;

		Crew[GUNNER].Controller.SetRotation( ControllerRot );
	}
}

//---------------------------------------------------------
//	SpecialCalcView
//---------------------------------------------------------
simulated function SpecialCalcView( gbxPawn thePawn, out actor ViewActor, out vector CameraLocation, out rotator CameraRotation )
{
	if ( bFirstPersonZoom )
		CalcFirstPersonView( thePawn, ViewActor, CameraLocation, CameraRotation );
	else
		CalcThirdPersonView( thePawn, ViewActor, CameraLocation, CameraRotation );
}

//---------------------------------------------------------
//	ClientStartFire
//---------------------------------------------------------
// Needed to override this because Owner is different for this weapon than most other weapons.
simulated event ClientStartFire(int Mode)
{
	if (!Crew[GUNNER].Controller.CanFire() || bIsDismounting )
        return;

    if (Role < ROLE_Authority)
    {
        if (StartFire(Mode))
        {
            ServerStartFire(Mode);
        }
    }
    else
    {
        StartFire(Mode);
    }
}

//---------------------------------------------------------
//	GetFireRotation
//---------------------------------------------------------
function Rotator GetFireRotation()
{
	return Rotation;
}

//---------------------------------------------------------
//	GetViewOrigin
//---------------------------------------------------------
function Vector GetViewOrigin()
{
	if ( Crew[GUNNER] != None )
		return Crew[GUNNER].Location;
}

//---------------------------------------------------------
//	AdjustAimNew
//---------------------------------------------------------
// Used only with player pawns; AI pawns use the AdjustAimNew() in FiringController.
function Rotator AdjustAimSpecial( Ammunition Ammo, Vector ProjStart, Rotator AimRotator, int WeaponAimError )
{
	local float FinalErrorMax;

	FinalErrorMax = WeaponAimError;

	// Yaw.
	if ( Rand(2) == 0 )
		AimRotator.Yaw += FRand() * FinalErrorMax;
	else
		AimRotator.Yaw -= FRand() * FinalErrorMax;

	// Pitch
	if ( Rand(2) == 0 )
		AimRotator.Pitch += FRand() * FinalErrorMax;
	else
		AimRotator.Pitch -= FRand() * FinalErrorMax;

	return AimRotator;
}

//---------------------------------------------------------
//	CandidateAcquired
//---------------------------------------------------------
function CandidateAcquired( gbxPawn Candidate )
{
	local int x;

	// Here's how it works: add this candidate only if:
	//	1) The pawn is an AI-controlled pawn.
	//	2) The list is currently empty;
	//  3) The first candidate in the list is on this candidate's team.

	if ( Candidate.IsA( 'gbxPlayerPawn' ) )
		return;

	if ( CandidateList.Length == 0 || CandidateList[0].SameTeamAs( Candidate ) )
	{
		for( x = 0; x < CandidateList.Length; ++x )
		{
			if ( CandidateList[x] == Candidate )
				return;
		}

		// Add.
		CandidateList[CandidateList.Length] = Candidate;
	}
}

//---------------------------------------------------------
//	CandidateLost
//---------------------------------------------------------
function CandidateLost( gbxPawn Candidate )
{
	local int x;

	for( x = 0; x < CandidateList.Length; ++x )
	{
		if ( CandidateList[x] == Candidate )
		{
			CandidateList.Remove( x, 1 );
			return;
		}
	}
}

//---------------------------------------------------------
//	NeverAllowShotProtectionFor
//---------------------------------------------------------
function bool NeverAllowShotProtectionFor( class<DamageType> DamageType )
{
	return (
		DamageType == class'DamageGammonBomb' || 
		DamageType == class'DamageTankCannon' ||
		DamageType == class'DamageDEPanzerfaust' ||
		DamageType == class'DamageUSBazooka' ||
		DamageType == class'DamageGrenade' 
		);
}

//---------------------------------------------------------
//	RejectDamageRequest
//---------------------------------------------------------
function bool RejectDamageRequest( Pawn Shooter, Pawn Victim, class<DamageType> DamageType )
{
	local Vector MyFacingVect, ToShooterVect;
	local float DistToShooter;

	if ( !bUseShotProtection || Level.bWeaponsAreBFG )
		return false;

	// No rejection for heavy explosives.
	if ( NeverAllowShotProtectionFor( DamageType ) )
		return false;

	if ( !PawnIsInCrew( Victim ) || Victim.IsHumanControlled() )
		return false;

	MyFacingVect = Vector( MyFactory.Rotation );
	ToShooterVect = Shooter.Location - Location;

	MyFacingVect.Z = 0;
	ToShooterVect.Z = 0;

	DistToShooter = VSizeIgnoreZ( ToShooterVect );

	MyFacingVect = Normal( MyFacingVect );
	ToShooterVect = Normal( ToShooterVect );

	if ( MyFacingVect dot ToShooterVect < ShotProtectionCosTheta )	// 45 degrees
		return false;

	if ( DistToShooter  > NoShotProtectionDist )
		return true;
	else
		return false;
}

//---------------------------------------------------------
//	CanSpawnShellCasing
//---------------------------------------------------------
function bool CanSpawnShellCasing()
{
	return ( Crew[GUNNER].IsHumanControlled() && !bFirstPersonZoom );
}

//---------------------------------------------------------
//	ShouldPlayShellSound
//---------------------------------------------------------
function bool ShouldPlayFirstPersonShellSound()
{
	return ( Crew[GUNNER].IsHumanControlled() && bFirstPersonZoom );
}

//---------------------------------------------------------
//	IsOnCrew
//---------------------------------------------------------
function bool IsOnCrew( gbxPawn gbxP )
{
	local int x;

	for( x = 0; x < 3; ++x )
		if ( Crew[x] == gbxP )
			return true;
	return false;
}

//---------------------------------------------------------
//	GetAttachment
//---------------------------------------------------------
function AttachmentDEMG42_ GetAttachment()
{
	return AttachmentDEMG42_( ThirdPersonActor );
}

//---------------------------------------------------------
//	PlayZoomIn
//---------------------------------------------------------
simulated function PlayZoomIn()
{
	if ( !bIsMounting && !bIsDismounting )
		SetFirstPersonZoom( true );
}

//---------------------------------------------------------
//	PlayZoomOut
//---------------------------------------------------------
simulated function PlayZoomOut()
{
	if ( !bIsMounting && !bIsDismounting )
		SetFirstPersonZoom( false );
}

//---------------------------------------------------------
//	IsZoomed
//---------------------------------------------------------
event bool IsZoomed()
{
	return bFirstPersonZoom;
}

//---------------------------------------------------------
//	CanZoom
//---------------------------------------------------------
simulated function bool CanZoom()
{
	return true;
}

//---------------------------------------------------------
//	GetBaseVector
//---------------------------------------------------------
function Vector GetBaseVector()
{
	return BaseVector;
}

//---------------------------------------------------------
//	NotifyCrewMemberDied
//---------------------------------------------------------
function NotifyCrewMemberDied( gbxPawn gbxP )
{
	CandidateLost( gbxP );

	if ( gbxP == Crew[GUNNER] )
	{
		StopFiringWeapon();
		GunnerDied( gbxP );
		
		NextCrewCheckTime = 0.0;	// Force the weapon to look for a new gunner during the next tick.		
	}
}

//---------------------------------------------------------
//	NotifyForceDismount
//---------------------------------------------------------
function NotifyForceDismount( optional bool bDoNothingIfHumanControlled )
{
	local int x;
	for (x=0; x<3; ++x)
		if ( Crew[x] != None )
		{
			if ( Crew[x].Controller.IsA( 'gbxAIController' ) )
			{
				gbxAIController( Crew[x].Controller ).BreakOutJustMe();
			}
			else
			if ( Crew[x].Controller.bIsHumanPlayer && !bDoNothingIfHumanControlled )
			{
				Crew[GUNNER].DismountTurretPlayer(true);
			}
		}
}

//---------------------------------------------------------
//	PostRender
//---------------------------------------------------------
function PostRender( Canvas TheCanvas )
{
	local gbxMuzzleFlash MuzFlash;

	if ( bFirstPersonZoom )
	{		
		MuzFlash = gbxMuzzleFlash( FireDEMG42_( FireMode[0] ).GetMuzzleFlash() );
		if ( MuzFlash != None )
		{
			MuzFlash.bHidden = false;
			MuzFlash.SetLocation( GetMuzzleLocationWorld() );
			MuzFlash.SetRotation( Rotation );
			TheCanvas.DrawActor( MuzFlash, 50 );
		}
		TheCanvas.DrawActor(self, DisplayFOV);
	}	

	CalcOffsetWeaponCollision();
}

function bool AlwaysAutoReload()
{
	return true;
}

event bool TargetIsInMyConeOfFire( gbxPawn aTarget, optional bool bUseNarrowHysteresis )
{
	return PointIsInMyConeOfFire( aTarget.Location, bUseNarrowHysteresis );
}

function bool PointIsInMyConeOfFire( Vector TargetLoc, optional bool bUseNarrowHysteresis ) 
{ 
	local Vector ToTarget, BaseVectorAdjusted;

	// Quick dot-product check first to see if the enemy is behind this weapon.
	ToTarget = TargetLoc - Self.Location;
	ToTarget.Z = 0;
	ToTarget = Normal( ToTarget );
	BaseVectorAdjusted = BaseVector;
	BaseVectorAdjusted.Z = 0;
	BaseVectorAdjusted = Normal( BaseVectorAdjusted );

	if ( bUseNarrowHysteresis )
		return ( BaseVectorAdjusted dot ToTarget >= ( HorizCosTheta + 0.03f ) );
	else
		return ( BaseVectorAdjusted dot ToTarget >= ( HorizCosTheta - 0.1f ) );
}

//=========================================================
//	Default Properties
//=========================================================
defaultproperties
{
	DrawType=DT_Mesh
//	Mesh=Mesh'k_view_model_de.mg42_crosshair'
	bHidden=true
	DisplayFOV=+35.0
	bAcceptsProjectors=false
	DrawScale=1.0
	ItemName="German MG42 Machinegun"
	Physics=PHYS_None

	MyMountAction=Mount_MountMG42Gunner
	MyDismountAction=Mount_DismountMG42Gunner
	PlayersJob=Job_MG42_Gunner

	FireModeClass(0)=FireDEMG42_
	FireModeClass(1)=FireDEMG42_
	AttachmentClass=class'AttachmentDEMG42_'

	DefaultAmmoCount(0)=400	

	// Nine-point blending animations.
	PlayerMountAnim=PLAYER_mg42_mount
	PlayerDismountAnim=PLAYER_mg42_dismount

	PawnForwardDownAnim=mg42_forward_down
	PawnForwardLevelAnim=mg42_forward_level
	PawnForwardUpAnim=mg42_forward_up
	PawnLeftDownAnim=mg42_left_down
	PawnLeftLevelAnim=mg42_left_level
	PawnLeftUpAnim=mg42_left_up
	PawnRightDownAnim=mg42_right_down
	PawnRightLevelAnim=mg42_right_level
	PawnRightUpAnim=mg42_right_up

	HorizCosTheta=0.7071
	VertCosTheta=0.2588

	FireLoopSoundName="W_MG42A.FIRELOOP"
	FireOneBulletSoundName="W_MG42A.FIRELOOPEND"

	StereoFireLoopSoundName="W_MG42A.FIRELOOP_ST"
	StereoFireOneBulletSoundName="W_MG42A.FIRELOOPEND_ST"

	MaxRotSpeed=11.0

	GunnerOffset=(X=0,Y=0,Z=0)

	GUNNER=0

	Crew(0)=None
	Crew(1)=None
	Crew(2)=None
	Claimants(0)=None
	Claimants(1)=None
	Claimants(2)=None

	GunnerMountTag=None	

	CameraMountSpeed=2.0
	CameraDismountSpeed=1.3
	CameraStartOffset=(X=30.0,Y=0.0,Z=150.0)
	//CameraMountedOffset=(X=90,Y=0.1,Z=100.0)
	CameraMountedOffset=(X=175,Y=0.1,Z=100.0)
	CameraDismountedOffset=(X=25.0,Y=0.0,Z=85.0)

	CameraOffsetZoomed=(X=-35.8,Y=0.05,Z=0.86)

	bUseShotProtection=true
	NoShotProtectionDist=1000.0

	PauseTime_Low=(Min=0.15,Max=1.0)
	PauseTime_High=(Min=3.0,Max=5.0)

	HorizFieldUU=8192
	VertFieldUU=2731
	CameraPitchUU=2000

	ClientState=WS_Hidden

	AmmoClipSize=400	

	bFreeCamera=true

	BreatheProfileClass=class'gbxSupportedBreatheProfile'

	WeaponName = WN_DEMG42

	CrosshairDrawFreq=0.01

	bOpaqueForSituationalAwarenessView=true
}
