class WeapDEMortar extends TurretWeapon;

//---------------------------------------------------------
//	Variables
//---------------------------------------------------------
var bool					bCrewHasBeenDrafted, bReadyToShoot;
var bool					bDontRecruit;
var bool					bWaitingOnShell, bWaitingOnAnim;
var bool					bDismountWhenNearbyImpacts;
var bool					bDismountWhenNearbyWhizzes;
var bool					bFireRateIndependent;
var Name					GunnerToDraft, LeftLoaderToDraft, RightLoaderToDraft;
var Name					MountEvent, DismountEvent, StopFiringEvent;
var gbxPawn					Gunner, LeftLoader, RightLoader;
var Vector					GunnerOffset, LeftLoaderOffset, RightLoaderOffset;
var Vector					MuzzleLocation, MuzzleOffset;
var Vector					TargetLocation;
var Rotator					MuzzleRotation;
var Actor					Target;
var EMortarFire				FireType;
var int						TotalShots, NumShotsSoFar;
var int						MaxCrewSize, CurCrewSize;
var float					TimeBetweenShots;
var float					OffsetFromTarget;
var float					LastCheckedProximityTime, CheckProximityInterval;
var float					NextCrewCheckTime;
var float					CrewCheckInterval;
var float					NoticeBulletImpactsDist;		// GBX:naj - Fixes bug 3876
const QUEUE_FIRE_PRIORITY	= 96;

//---------------------------------------------------------
//	PostBeginPlay
//---------------------------------------------------------
function PostNetBeginPlay()
{
	local Vector X,Y,Z;

	Super.PostNetBeginPlay();
	
	SetStaticMesh(StaticMesh(DynamicLoadObject("s_military_de.Weapon.mortar", class'StaticMesh')));
	
	GetAxes( Rotation, X,Y,Z );
	MuzzleLocation = Self.Location + MuzzleOffset.X*X + MuzzleOffset.Y*Y + MuzzleOffset.Z*Z;
	MuzzleRotation.Yaw = Rotation.Yaw;
	MuzzleRotation.Pitch = 13000;

	GiveAmmo(0);
}

//---------------------------------------------------------
//	IsMortar
//---------------------------------------------------------
function bool IsMortar() 
{ 
	return true; 
}

//---------------------------------------------------------
//	FaceRotation
//---------------------------------------------------------
function FaceRotation( Rotator Rot ) {}

//---------------------------------------------------------
//	PawnIsInCrew
//---------------------------------------------------------
function bool PawnIsInCrew( Pawn P ) 
{ 
	return P == Gunner || P == LeftLoader || P == RightLoader;
}

//---------------------------------------------------------
//	DraftCrew
//---------------------------------------------------------
function DraftCrew()
{
	local gbxPawn aDraftee;
	local class<gbxAIController> MindClass;

	// Ensure that no more than the maximum number of crew members is drafted.
	if ( MaxCrewSize < 3 )
		RightLoaderToDraft = '';
	if ( MaxCrewSize < 2 )
		LeftLoaderToDraft = '';

	ForEach DynamicActors(class'gbxPawn', aDraftee )
	{		
		if ( Gunner == None && aDraftee.Tag == GunnerToDraft )
		{
			Gunner = aDraftee;
			MindClass = class<gbxAIController>( DynamicLoadObject( "gbxAI.MindMortarGunner", class'Class' ) );
			Gunner.CrewTurret( Self, MindClass, Job_Mortar_Gunner );			
			CurCrewSize++;
		}
		else if ( LeftLoader == None && aDraftee.Tag == LeftLoaderToDraft )
		{
			LeftLoader = aDraftee;
			MindClass = class<gbxAIController>( DynamicLoadObject( "gbxAI.MindMortarLoader", class'Class' ) );
			LeftLoader.CrewTurret( Self, MindClass, Job_Mortar_LeftLoader );
			CurCrewSize++;
		}
		else if ( RightLoader == None && aDraftee.Tag == RightLoaderToDraft )
		{
			RightLoader = aDraftee;
			MindClass = class<gbxAIController>( DynamicLoadObject( "gbxAI.MindMortarLoader", class'Class' ) );
			RightLoader.CrewTurret( Self, MindClass, Job_Mortar_RightLoader );
			CurCrewSize++;
		}
	}

	if ( CurCrewSize >= MaxCrewSize )
	{
		bCrewHasBeenDrafted = true;
		return;
	}
}

// Keep spawning the crew member upwards until you find a clear spot; then use gravity to drop him down in place.
function SetPawnOnMountLocation( gbxPawn CrewMember, Vector IdealOffset )
{
	local Vector LocalX, LocalY, LocalZ;
	GetAxes( Rotation, LocalX, LocalY, LocalZ );

	if ( CrewMember.SetLocation( Self.Location + IdealOffset.X*LocalX + IdealOffset.Y*LocalY + IdealOffset.Z*LocalZ ) )
		return;
	
	IdealOffset.Z += 10.0;

	if ( CrewMember.SetLocation( Self.Location + IdealOffset.X*LocalX + IdealOffset.Y*LocalY + IdealOffset.Z*LocalZ ) )
		return;

	IdealOffset.Z += 10.0;

	if ( CrewMember.SetLocation( Self.Location + IdealOffset.X*LocalX + IdealOffset.Y*LocalY + IdealOffset.Z*LocalZ ) )
		return;

	IdealOffset.Z += 10.0;

	if ( CrewMember.SetLocation( Self.Location + IdealOffset.X*LocalX + IdealOffset.Y*LocalY + IdealOffset.Z*LocalZ ) )
		return;

	IdealOffset.Z += 20.0;

	if ( CrewMember.SetLocation( Self.Location + IdealOffset.X*LocalX + IdealOffset.Y*LocalY + IdealOffset.Z*LocalZ ) )
		return;

	IdealOffset.Z += 50.0;

	if ( CrewMember.SetLocation( Self.Location + IdealOffset.X*LocalX + IdealOffset.Y*LocalY + IdealOffset.Z*LocalZ ) )
		return;

	IdealOffset.Z += 100.0;

	CrewMember.SetLocation( Self.Location + IdealOffset.X*LocalX + IdealOffset.Y*LocalY + IdealOffset.Z*LocalZ );
}

//---------------------------------------------------------
//	CompleteMount
//---------------------------------------------------------
function CompleteMount( gbxPawn gbxP )
{	
	if ( gbxP == Gunner )
	{
		SetPawnOnMountLocation( Gunner, GunnerOffset );
		Gunner.SetRotation( Self.Rotation );
		LeftLoader.SetPhysics( PHYS_Falling );
		Gunner.StartGoingToCrouch( true );
		Gunner.FlushAnimQueueChannel( 0,, true );
		Gunner.LoadMountAnimations( Mount_MountMortarGunner );
		Gunner.bDoTorsoTwist = false;
		Gunner.bNoTurnBody = true;
		Gunner.SetWeapon( Self );

		Instigator = Gunner;
		if ( Ammo[0] != None )
			Ammo[0].Instigator = Instigator;
		if ( FireMode[0] != None )
			FireMode[0].Instigator = Instigator;
		if ( ThirdPersonActor != None )
			ThirdPersonActor.Instigator = Instigator;

		// Send out a mount event to anyone who may be interested.
		TriggerEvent( MountEvent, Self, Gunner );

	}
	else if ( gbxP == LeftLoader )
	{
		SetPawnOnMountLocation( LeftLoader, LeftLoaderOffset );
		LeftLoader.SetRotation( Self.Rotation );
		LeftLoader.SetPhysics( PHYS_Falling );
		LeftLoader.StartGoingToCrouch( true );
		LeftLoader.FlushAnimQueueChannel( 0,, true );
		LeftLoader.LoadMountAnimations( Mount_MountMortarLoader );
		LeftLoader.bDoTorsoTwist = false;
		LeftLoader.bNoTurnBody = true;
		LeftLoader.SetWeapon( None );
	}
	else if ( gbxP == RightLoader )
	{	
		SetPawnOnMountLocation( RightLoader, RightLoaderOffset );
		RightLoader.SetRotation( Self.Rotation );
		LeftLoader.SetPhysics( PHYS_Falling );
		RightLoader.StartGoingToCrouch( true );
		RightLoader.FlushAnimQueueChannel( 0,, true );
		RightLoader.LoadMountAnimations( Mount_MountMortarLoader );
		RightLoader.bDoTorsoTwist = false;
		RightLoader.bNoTurnBody = true;
		RightLoader.SetWeapon( None );
	}	
	
	gbxP.PlayWaiting();
}

//---------------------------------------------------------
//	Tick
//---------------------------------------------------------
function Tick( float DeltaTime )
{	
	Super.Tick( DeltaTime );

	// Make sure that this weapon is manned.
	if ( !bDontRecruit && !bCrewHasBeenDrafted && Level.TimeSeconds > NextCrewCheckTime )
	{
		NextCrewCheckTime = Level.TimeSeconds + CrewCheckInterval;
		DraftCrew();		
	}	

	if ( Gunner == None )
		return;
}

//---------------------------------------------------------
//	ForceCrewDismount
//---------------------------------------------------------
function ForceCrewDismount( optional Pawn InstigatedBy )
{
	bDontRecruit = true;

	if ( Gunner != None )
	{
		StopFiringWeapon();
		gbxAIController( Gunner.Controller ).BreakOutJustMe( InstigatedBy );
	}

	if ( LeftLoader != None )
	{
		gbxAIController( LeftLoader.Controller ).BreakOutJustMe( InstigatedBy );
	}	

	if ( RightLoader != None )
	{
		gbxAIController( RightLoader.Controller ).BreakOutJustMe( InstigatedBy );
	}
}


//---------------------------------------------------------
//	SetFireParameters
//---------------------------------------------------------
function SetFireParameters( Actor NewTarget, EMortarFire NewFireType, int NewNumShots, float NewTimeBetweenShots, float NewOffsetFromTarget )
{
	Target = NewTarget;
	FireType = NewFireType;
	TotalShots = NewNumShots;
	ClipAmt = NewNumShots;	
	TimeBetweenShots = NewTimeBetweenShots;
	OffsetFromTarget = NewOffsetFromTarget;	
	NumShotsSoFar = 0;
}

//---------------------------------------------------------
//	ClientStartFire
//---------------------------------------------------------
// Needed to override this because Owner is different for this weapon than most other weapons.
simulated event ClientStartFire(int Mode)
{
    if (!Gunner.Controller.CanFire())
        return;

    if (Role < ROLE_Authority)
    {
        if (StartFire(Mode))
        {
            ServerStartFire(Mode);
        }
    }
    else
    {
        StartFire(Mode);
    }
}

//---------------------------------------------------------
//	DoFireEffect
//---------------------------------------------------------
function DoFireEffect() {}

//---------------------------------------------------------
//	PlayFiring
//---------------------------------------------------------
simulated function PlayFiring()
{	
	if ( bWaitingOnShell || bWaitingOnAnim )
	{
		return;
	}
	
	if ( !bFireRateIndependent )
	{
		bWaitingOnShell = true;
	}

	if ( Gunner != None )
	{
		Gunner.AddAnimToQueue( 'de_mortar_gunner_fire', QUEUE_FIRE_PRIORITY, true, false,,, 0.15 );
	}

	if ( LeftLoader != None && !bFireRateIndependent )
	{
		LeftLoader.AddAnimToQueue( 'de_mortar_loader_1_fire', QUEUE_FIRE_PRIORITY, true, false,,, 0.15, class'MortarCallback' );
		bWaitingOnAnim = true;
	}
	else
	{
		// No left loader, so we can't rely on an animation notify to launch the shell.
		NotifyReadyToLaunch();
	}

	if ( RightLoader != None && !bFireRateIndependent )
	{
		RightLoader.AddAnimToQueue( 'de_mortar_loader_2_fire', QUEUE_FIRE_PRIORITY, true, false,,, 0.15 );
	}
}

//---------------------------------------------------------
//	NotifyForceDismount
//---------------------------------------------------------
function NotifyForceDismount( optional bool bDoNothingIfHumanControlled )
{
	ForceCrewDismount();
}

//---------------------------------------------------------
//	NotifyReadyToLaunch
//---------------------------------------------------------
function NotifyReadyToLaunch()
{
	log("NAJ**** NotifyReadyToLaunch; setting bWaitingOnShell to false.", 'naj' );
	bWaitingOnShell = false;

	if ( NumShotsSoFar == 0 )
	{
		TargetLocation = Target.Location;
	}
	
	Spawn( class'MuzzleFlashMortar', Self,, MuzzleLocation, MuzzleRotation );
	NumShotsSoFar++;	
	FireDEMortar( FireMode[0] ).LaunchMortar( FireType, TotalShots, NumShotsSoFar, OffsetFromTarget );	
	StopFiringWeapon();	

	if ( NumShotsSoFar == TotalShots )
	{
		TriggerEvent( StopFiringEvent, Self, Gunner );
	}	
}

//---------------------------------------------------------
//	NotifyLoadShellAnimEnd
//---------------------------------------------------------
// Safeguard in case the anim notify didn't fire.
function NotifyLoadShellAnimEnd()
{
	log("NAJ**** NotifyLoadShellAnimEnd; bWaitingOnShell=="$bWaitingOnShell$" ; setting bWaitingOnAnim to false.", 'naj' );

	if ( bWaitingOnShell )
		NotifyReadyToLaunch();

	bWaitingOnAnim = false;
}

//---------------------------------------------------------
//	ReadyToShoot
//---------------------------------------------------------
function bool ReadyToShoot()
{
	log("NAJ**** ReadyToShoot: bWaitingOnShell="$bWaitingOnShell$" ; bWaitingOnAnim="$bWaitingOnAnim, 'naj' );
	return !bWaitingOnShell && !bWaitingOnAnim && ( ClipAmt > 0 );
}

//---------------------------------------------------------
//	IndependentFireMode
//---------------------------------------------------------
function bool IndependentFireMode()
{
	return bFireRateIndependent;
}

//---------------------------------------------------------
//	CompleteDismount
//---------------------------------------------------------
function CompleteDismount( gbxPawn gbxP )
{
	if ( gbxP == Gunner )
	{
		Instigator = None;
		Ammo[0].Instigator = None;
		FireMode[0].Instigator = None;

		Gunner = None;

		// Send out a dismount event to anyone who may be interested.
		TriggerEvent( DismountEvent, Self, gbxP );		
	}	
	else if ( gbxP == LeftLoader )
	{
		LeftLoader = None;
	}
	else if ( gbxP == RightLoader )
	{
		RightLoader = None;
	}

	//gbxP.Weapon = None;  // hack so SetWeapon() will call ChangedWeapon() - jlb
	//gbxP.SetWeapon( gbxP.LastWeapon );
}

//=========================================================
//	Default Properties
//=========================================================
defaultproperties
{
	DrawType=DT_StaticMesh
//	StaticMesh=StaticMesh's_military_de.Weapon.mortar'
	StaticMesh=None
	Texture=Texture'Engine.S_Weapon'

	FireModeClass(0)=FireDEMortar
	FireModeClass(1)=FireDEMortar

	Physics=PHYS_None
	bStatic=false
	bCollideActors=true
	bBlockKarma=true
	bBlockActors=true
	bBlockPlayers=true
	bCollideWorld=false
	bBlockZeroExtentTraces=true
	bBlockNonZeroExtentTraces=true	

	GunnerOffset=(x=-92.0,y=-0.787,z=0)
	LeftLoaderOffset=(x=-12.625,y=82.999,z=0)
	RightLoaderOffset=(x=-13.032,y=-77.398,z=0)
	MuzzleOffset=(x=25,y=0,z=100)

	ClientState=WS_Hidden

	AmmoClipSize=1

	EnemyTooCloseDist=1500.0
	CrewCheckInterval=3.0
	bReadyToShoot=true

	BreatheProfileClass=class'gbxSupportedBreatheProfile'

	bDamagesVehicles=true

	bOpaqueForSituationalAwarenessView=true
}