class WeapDEPak36_ extends TurretWeapon;

//---------------------------------------------------------
//	Variables
//---------------------------------------------------------
var bool					bDontRecruit;
var bool					bCrewHasBeenDrafted, bReloadingShell;
var bool					bTurningLeft, bTurningRight, bTurningUp, bTurningDown;
var	bool					bIsMounting, bIsDismounting;
var int						CurBaseYaw, CurGunPitch, PitchOffset;
var int						MaxYawDelta, MaxPitchDeltaDown, MaxPitchDeltaUp;
var int						MaxYawModelDelta;
var int						CameraPitchUU;
var	Name					GunBone;
var Name					GunnerToDraft, LoaderToDraft;
var Name					MountEvent;
var Name					DismountEvent;
var Name					DeathEvent;
var float					YawRotSpeed, PitchRotSpeed;
var float					MountStartStamp;
var float					CameraZoomSpeed, CameraZoomOutSpeed;
var	float					MaxFiringDist;
var float					KillZoneDist;
var float					NextCrewCheckTime;
var float					CrewCheckInterval;
var gbxPawn					Gunner, Loader;
var Vector					GunnerOffset, LoaderOffset;
var Vector					CameraStartOffset;
var Vector					CameraEndOffset;
var Vector					CameraOffset;
var Rotator					DesiredRot;
var gbxContextUseTrigger	MyTrigger;

//---------------------------------------------------------
//	PostBeginPlay
//---------------------------------------------------------
function PostNetBeginPlay()
{
	Super.PostNetBeginPlay();

	GiveAmmo(0);

	Mesh = Mesh(DynamicLoadObject("k_prop.de_pak36", class'Mesh'));
	LinkMesh( Mesh );
	
	PlayAnim( 'Idle' );	

	CurBaseYaw = Rotation.Yaw;
	CurGunPitch = Rotation.Pitch;
}

//---------------------------------------------------------
//	IsAntiTank
//---------------------------------------------------------
function bool IsAntiTank() 
{ 
	return true; 
}

//---------------------------------------------------------
//	Landed
//---------------------------------------------------------
event Landed( vector HitLocation, vector HitNormal )
{
	local Vector LocalX, LocalY, LocalZ;
	
	// The weapon was set up above the ground and has fallen to the floor.
	// Make sure the crew is line up properly, otherwise they'll be left floating in the air.
	//
	GetAxes( Rotation, LocalX, LocalY, LocalZ );

	if ( Gunner != None )
	{
		Gunner.SetLocation( Self.Location + GunnerOffset.X*LocalX + GunnerOffset.Y*LocalY + GunnerOffset.Z*LocalZ );
	}

	if ( Loader != None )
	{
		Loader.SetLocation( Self.Location + LoaderOffset.X*LocalX + LoaderOffset.Y*LocalY + LoaderOffset.Z*LocalZ );
	}
}

//---------------------------------------------------------
//	PawnIsInCrew
//---------------------------------------------------------
function bool PawnIsInCrew( Pawn P ) 
{ 
	return P == Gunner || P == Loader;
}

//---------------------------------------------------------
//	DraftCrew
//---------------------------------------------------------
function DraftCrew()
{
	local gbxPawn aDraftee;
	local class<gbxAIController> MindClass;

	ForEach DynamicActors(class'gbxPawn', aDraftee )
	{
		if ( aDraftee.Tag == GunnerToDraft )
		{
			Gunner = aDraftee;
			MindClass = class<gbxAIController>( DynamicLoadObject( "gbxAI.MindPak36Gunner", class'Class' ) );
			Gunner.CrewTurret( Self, MindClass, Job_Pak36_Gunner );
		}
		else if ( aDraftee.Tag == LoaderToDraft )
		{
			Loader = aDraftee;
			MindClass = class<gbxAIController>( DynamicLoadObject( "gbxAI.MindPak36Loader", class'Class' ) );
			Loader.CrewTurret( Self, MindClass, Job_Pak36_Loader );
		}
	}

	if ( Gunner != None )
		bCrewHasBeenDrafted = true;
}

//---------------------------------------------------------
//	GetRotDelta
//---------------------------------------------------------
function GetRotDelta( int Requested, int Center, out int Diff, out int RightOfCenter )
{
	local bool bIsNegative, bDidSubtract;

	Requested = Requested & 65535;
	Center = Center & 65535;

	Diff = Requested - Center;
	bIsNegative = ( Diff < 0 );
	Diff = abs( Diff );

	if ( Diff > 32767 )
	{
		Diff = 65535 - Diff;
		bDidSubtract = true;
	}
	if ( ( bDidSubtract && !bIsNegative ) || ( !bDidSubtract && bIsNegative ) )
	{
		RightOfCenter = 0;
	}
	else if ( ( !bDidSubtract && !bIsNegative ) || ( bDidSubtract && bIsNegative ) )
	{
		RightOfCenter = 1;
	}
}

//---------------------------------------------------------
//	FaceRotation
//---------------------------------------------------------
function FaceRotation( Rotator Rot )
{
	local int Diff, RightOfCenter;

	if ( bHasBlownUp )
		return;

	Rot.Pitch = Rot.Pitch & 65535;
	Rot.Yaw = Rot.Yaw & 65535;

	// Make sure the requested pitch and yaw do not go outside the bounds of the weapon.
	GetRotDelta( Rot.Pitch, Rotation.Pitch, Diff, RightOfCenter );
	if ( RightOfCenter == 1 && Diff > MaxPitchDeltaUp )
	{
		Rot.Pitch = ( Rotation.Pitch + MaxPitchDeltaUp ) & 65535;
	}
	else if ( RightOfCenter == 0 && Diff > MaxPitchDeltaDown )
	{
		Rot.Pitch = ( Rotation.Pitch - MaxPitchDeltaDown ) & 65535;
	}	
	
	GetRotDelta( Rot.Yaw, Rotation.Yaw, Diff, RightOfCenter );
	if ( RightOfCenter == 1 && Diff > MaxYawDelta )
	{
		Rot.Yaw = ( Rotation.Yaw + MaxYawDelta ) & 65535;
	}
	else if ( RightOfCenter == 0 && Diff > MaxYawDelta )
	{
		Rot.Yaw = ( Rotation.Yaw - MaxYawDelta ) & 65535;
	}

	DesiredRot = Rot;	
	DesiredRot.Pitch -= PitchOffset;		// Because at pitch=0, the gun may not actually be pointing horizontally	

	DesiredRot.Pitch = DesiredRot.Pitch & 65535;
	DesiredRot.Yaw = DesiredRot.Yaw & 65535;
}

//---------------------------------------------------------
//	GetAdjustedRotation
//---------------------------------------------------------
function int GetAdjustedRotation( int CurPosition, int DesiredPosition, float Speed, float DeltaTime, out int bRotatingLeft )
{
	local int Diff, NewPos, Offset;
	local bool bIsNegative, bDidSubtract, bSnap;
	local float AdjustedSpeed;

	Diff = DesiredPosition - CurPosition;
	bIsNegative = ( Diff < 0 );

	Diff = abs( Diff );

	if ( Diff > 32767)
	{
		Diff = 65536 - Diff;
		bDidSubtract = true;
	}

	AdjustedSpeed = Speed;
	Offset = AdjustedSpeed*DeltaTime;
	if ( Offset >= Diff )
	{
		do
		{
			AdjustedSpeed *= 0.75;
			Offset = AdjustedSpeed*DeltaTime;
		}
		until( AdjustedSpeed < 0.25*Speed || Offset <= Diff );
		
		if ( Offset >= Diff )
		{
			bSnap = true;
		}
	}
	
	if ( ( bDidSubtract && bIsNegative ) || ( !bDidSubtract && !bIsNegative ) )
	{
		if ( bSnap )
			NewPos = DesiredPosition;
		else
			NewPos = (CurPosition + Offset ) & 65535;
		
		bRotatingLeft = 0;
	}
	else
	{
		if ( bSnap )
			NewPos = DesiredPosition;
		else
			NewPos = (CurPosition - Offset ) & 65535;

		bRotatingLeft = 1;
	}

	return NewPos;	
}

//---------------------------------------------------------
//	Tick
//---------------------------------------------------------
function Tick( float DeltaTime )
{	
	local Rotator GunRot;
	local int RotatingLeft, RotatingDown;

	Super.Tick( DeltaTime );

	if ( bHasBlownUp )
		return;

	// Make sure that this weapon is manned.
	//if ( !bCrewHasBeenDrafted )
	//{
	if ( !bDontRecruit && !bCrewHasBeenDrafted && Level.TimeSeconds > NextCrewCheckTime )
	{
		NextCrewCheckTime = Level.TimeSeconds + CrewCheckInterval;
		DraftCrew();		
	}	

	if ( Gunner == None ) 
	{
		return;
	}

	bTurningLeft = false;
	bTurningRight = false;
	bTurningUp = false;
	bTurningDown = false;

	if ( !bReloadingShell )
	{
		// Determine which direction (if any) the weapon is turning.
		if ( abs( DesiredRot.Yaw - CurBaseYaw ) > 0 )
		{		
			CurBaseYaw = GetAdjustedRotation( CurBaseYaw, DesiredRot.Yaw, YawRotSpeed, DeltaTime, RotatingLeft );		
			
			if ( RotatingLeft == 1 )
				bTurningLeft = true;
			else
				bTurningRight = true;
		}
		else
		if ( abs( DesiredRot.Pitch - CurGunPitch ) > 0 )
		{
			CurGunPitch = GetAdjustedRotation( CurGunPitch, DesiredRot.Pitch, PitchRotSpeed, DeltaTime, RotatingDown );		
			
			if ( RotatingDown == 1 )
				bTurningDown = true;
			else
				bTurningUp = true;
		}	

		if ( bTurningLeft || bTurningRight || bTurningUp || bTurningDown )
		{
			// -BB Restrict the barrel movement a bit
			if(CurBaseYaw > (Rotation.Yaw & 65535) + MaxYawModelDelta)
				GunRot.Yaw = -((Rotation.Yaw & 65535) + MaxYawModelDelta);
			else if(CurBaseYaw < (Rotation.Yaw & 65535) - MaxYawModelDelta)
				GunRot.Yaw = -((Rotation.Yaw & 65535) - MaxYawModelDelta);
			else
				GunRot.Yaw = -CurBaseYaw;

			GunRot.Roll = CurGunPitch;
			SetBoneRotation( GunBone, GunRot + Rotation, 1.0 );
		}
		
		// Play the appropriate hand-cranking animation if the weapon is turning.
		if ( bTurningLeft )
		{		
			PlayCrankAnimations( 'pak_gunner_adjust_left', 0 );
		}
		else if ( bTurningRight )
		{		
			PlayCrankAnimations( 'pak_gunner_adjust_right', 0 );
		}
		else if ( bTurningUp )
		{
			PlayCrankAnimations( 'pak_gunner_adjust_up', 0 );
		}
		else if ( bTurningDown )
		{
			PlayCrankAnimations( 'pak_gunner_adjust_down', 0 );
		}
		else
		{		
			Gunner.PlayWaiting();	
		}

		if ( !FireMode[0].bIsFiring )
		{
			Self.LoopAnim( 'Idle', 1.0, 0.25 );
		}
	}
}

//---------------------------------------------------------
//	PlayCrankAnimations
//---------------------------------------------------------
simulated function PlayCrankAnimations( Name GunnerAnim, int Channel )
{
	local name AnimName;
	local float AnimFrame, AnimRate;

	Gunner.GetAnimParams( 0, AnimName, AnimFrame, AnimRate );
	if ( AnimName != GunnerAnim )
	{
		Gunner.AddAnimToQueue( GunnerAnim, 95.0, true, false,, 2.0, 0.05,, Channel );
	}
}

//---------------------------------------------------------
//	ClientStartFire
//---------------------------------------------------------
// Needed to override this because Owner is different for this weapon than most other weapons.
simulated event ClientStartFire(int Mode)
{
    if (!Gunner.Controller.CanFire() || bHasBlownUp )
        return;

    if (Role < ROLE_Authority)
    {
        if (StartFire(Mode))
        {
            ServerStartFire(Mode);
        }
    }
    else
    {
        StartFire(Mode);
    }
}

//---------------------------------------------------------
//	PlayFiring
//---------------------------------------------------------
simulated function PlayFiring()
{	
	//Gunner.PlayAnim( 'pak_gunner_fire' );
	Gunner.AddAnimToQueue( 'pak_gunner_fire', 96.0, true, false );	
	
	if ( Loader != None )
	{
		bReloadingShell = true;
		gbxAIController( Loader.Controller ).ReloadTurret();
	}
	else
	{
		// TBD: Play player reload anim.
		ReloadFinished();
	}
}

//---------------------------------------------------------
//	AdjustAimNew
//---------------------------------------------------------
// Used only with player pawns; AI pawns use the AdjustAimNew() in FiringController.
function Rotator AdjustAimSpecial( Ammunition Ammo, Vector ProjStart, Rotator AimRotator, int WeaponAimError )
{
	// The Pak is hard enough to aim as it is--don't add any inaccuracy to the thing!!
	return AimRotator;
}

//---------------------------------------------------------
//	GetViewOrigin
//---------------------------------------------------------
function Vector GetViewOrigin()
{
	return Self.Location;
}

//---------------------------------------------------------
//	GetFireRotation
//---------------------------------------------------------
function Rotator GetFireRotation()
{
	local Rotator Temp;
	Temp.Pitch = CurGunPitch;
	Temp.Yaw = CurBaseYaw;
	return Temp;
}

//---------------------------------------------------------
//	CompleteMount
//---------------------------------------------------------
function CompleteMount( gbxPawn gbxP )
{	
	local Vector LocalX, LocalY, LocalZ;

	GetAxes( Rotation, LocalX, LocalY, LocalZ );

	if ( gbxP == Gunner )
	{
		Gunner.StartGoingToCrouch( true );
		Gunner.bNoTurnBody = true;
		Gunner.SetLocation( Self.Location + GunnerOffset.X*LocalX + GunnerOffset.Y*LocalY + GunnerOffset.Z*LocalZ );
		Gunner.SetRotation( Self.Rotation );
		Gunner.bHardAttach = true;
		Gunner.SetBase( self );	// new
		Gunner.FlushAnimQueueChannel( 0,, true );
		Gunner.LoadMountAnimations( Mount_MountPak36Gunner );
		Gunner.bDoTorsoTwist = false;
		Gunner.SetWeapon( Self );

		Instigator = Gunner;
		if ( Ammo[0] != None )
			Ammo[0].Instigator = Instigator;
		if ( FireMode[0] != None )
			FireMode[0].Instigator = Instigator;
		if ( ThirdPersonActor != None )
			ThirdPersonActor.Instigator = Instigator;

		bIsMounting = false;				

		// Send out a mount event to anyone who may be interested.
		TriggerEvent( MountEvent, Self, Gunner );
	}
	else if ( gbxP == Loader )
	{
		Loader.StartGoingToCrouch( true );
		Loader.bNoTurnBody = true;
		Loader.SetLocation( Self.Location + LoaderOffset.X*LocalX + LoaderOffset.Y*LocalY + LoaderOffset.Z*LocalZ );
		Loader.SetRotation( Self.Rotation );
		Loader.SetBase( self );	// new
		Loader.FlushAnimQueueChannel( 0,, true );
		Loader.LoadMountAnimations( Mount_MountPak36Loader );
		Loader.bDoTorsoTwist = false;
		Loader.SetWeapon( None );
	}	

	gbxP.PlayWaiting();
}

//---------------------------------------------------------
//	ReloadFinished
//---------------------------------------------------------
function ReloadFinished()
{
	ReloadClip();
	if ( Loader != None )
	{
		gbxAIController( Loader.Controller ).NotifyReloadFinished();
	}
	bReloadingShell = false;
}

//---------------------------------------------------------
//	NotifyForceDismount
//---------------------------------------------------------
function NotifyForceDismount( optional bool bDoNothingIfHumanControlled )
{
	ForceCrewDismount();
}

//---------------------------------------------------------
//	ForceCrewDismount
//---------------------------------------------------------
function ForceCrewDismount( optional Pawn InstigatedBy )
{
	bDontRecruit = true;

	if ( Gunner != None )
	{
		StopFiringWeapon();
		gbxAIController( Gunner.Controller ).BreakOutJustMe( InstigatedBy );
	}

	if ( Loader != None )
	{
		gbxAIController( Loader.Controller ).BreakOutJustMe( InstigatedBy );
	}	
}

//---------------------------------------------------------
//	GetMyMountPoint
//---------------------------------------------------------
function Vector GetMyMountPoint()
{
	local Vector X,Y,Z;
	GetAxes( Rotation, X,Y,Z );
	return Location + MountOffset.X*X + MountOffset.Y*Y + MountOffset.Z*Z;
}

//---------------------------------------------------------
//	PreparePlayerToMount
//---------------------------------------------------------
function PreparePlayerToMount()
{
	Gunner.SetTurretAnims( 'pak_gunner_mount', 'pak_gunner_dismount' );
	Gunner.SetLocation( GetMyMountPoint() );
	Gunner.SetRotation( Rotation );
	Gunner.Controller.SetRotation( Rotation );
	Gunner.PrepareToMount();
	Gunner.Job = Job_Pak36_Gunner;

	bIsMounting = true;
}

//---------------------------------------------------------
//	PreparePawnToDismount
//---------------------------------------------------------
function PreparePawnToDismount( gbxPawn gbxP )
{
	// Check that the pawn passed in is the gunner.
	if (gbxP != Gunner )
		return;

	StopFiringWeapon();
	//TBD: try this: SetBoneRotation( GunBone,,, 0 );

	bIsDismounting = true;
}

//---------------------------------------------------------
//	CompleteDismount
//---------------------------------------------------------
function CompleteDismount( gbxPawn gbxP )
{
	if ( gbxP == Gunner )
	{
		Instigator = None;
		Ammo[0].Instigator = None;
		FireMode[0].Instigator = None;

		Gunner = None;
		bIsDismounting = false;
		CameraOffset.X = 0.0;
		CameraOffset.Z = 0.0;

		// Send out a dismount event to anyone who may be interested.
		TriggerEvent( DismountEvent, Self, gbxP );

		// Reactivate my trigger, which was made inactive when used.
		MyTrigger.bDontDisplayMessage = false;
	}	
	else if ( gbxP == Loader )
	{
		Loader = None;
	}

	//gbxP.Weapon = None;  // hack so SetWeapon() will call ChangedWeapon() - jlb
	//gbxP.SetWeapon( gbxP.LastWeapon );
}

//---------------------------------------------------------
//	Trigger
//---------------------------------------------------------
event Trigger(Actor Other, Pawn EventInstigator)
{
	local gbxPawn gbxP;

	if ( !EventInstigator.IsA( 'gbxPlayerPawn' ) || bHasBlownUp )
		return;
	
	gbxP = gbxPawn( EventInstigator );

	// Don't allow the player to mount if an enemy is still using it!
	if ( ( Gunner != None && Gunner != gbxP ) || Loader != None )
		return;

	// Skip the action instigator is in the middle of a reload.
	if ( gbxP.IsTransitioning() || (gbxP.Weapon != Self && gbxP.Weapon.IsReloading() ) )
		return;

	if ( Gunner == gbxP )	// Gunner wishes to dismount.
	{
		Gunner.DismountTurretPlayer(true);
		MountStartStamp=Level.TimeSeconds;
	}
	else						// Gunner wishes to mount.
	if ( Gunner == None )
	{
		if ( Gunner != None )
		{
			// TBD: najnaj: Tell the current claimant that he's been booted!
		}
		Gunner = gbxP;
		PreparePlayerToMount();
		Gunner.MountTurretPlayer( Self, true );
		MountStartStamp=Level.TimeSeconds;
	}
}

//---------------------------------------------------------
//	SpecialCalcView
//---------------------------------------------------------
simulated function SpecialCalcView( gbxPawn thePawn, out actor ViewActor, out vector CameraLocation, out rotator CameraRotation )
{
	CalcThirdPersonView( thePawn, ViewActor, CameraLocation, CameraRotation );
}

//---------------------------------------------------------
//	CalcThirdPersonView
//---------------------------------------------------------
function CalcThirdPersonView( gbxPawn thePawn, out actor ViewActor, out vector CameraLocation, out rotator CameraRotation )
{
	local Vector X,Y,Z;
	local Rotator ControllerRot;
	local int Diff, RightOfCenter;
	local float T;
	ViewActor = thePawn;

	CameraLocation = thePawn.Location;

	if ( true )
	{
		if ( bIsMounting )
		{
			// Player is mounting turret.  Zoom out.
			T = FClamp( ( Level.TimeSeconds - MountStartStamp ) * CameraZoomSpeed, 0, 1 );
			CameraOffset = CameraStartOffset + T*( CameraEndOffset - CameraStartOffset );
		}
		else if ( bIsDismounting )
		{
			// Player is dismounting turret.  Zoom in.
			T = FClamp( ( Level.TimeSeconds - MountStartStamp ) * CameraZoomOutSpeed, 0, 1 );
			CameraOffset = CameraEndOffset + T*( CameraStartOffset - CameraEndOffset );
		}
		else
		{
			CameraOffset = CameraEndOffset;
		}
	}

	// Restrict the yaw and pitch to the cone of fire.
	if ( Gunner != None )
	{
		ControllerRot = Gunner.Controller.Rotation;

		// Yaw.
		GetRotDelta( ControllerRot.Yaw, Rotation.Yaw, Diff, RightOfCenter );
		if ( Diff > MaxYawDelta )
		{
			if ( RightOfCenter == 1 )
				ControllerRot.Yaw = (Rotation.Yaw + MaxYawDelta) & 65535;
			else
				ControllerRot.Yaw = (Rotation.Yaw - MaxYawDelta) & 65535;
		}

		// Pitch.
		GetRotDelta( ControllerRot.Pitch, Rotation.Pitch, Diff, RightOfCenter );
		if ( RightOfCenter == 1 && Diff > MaxPitchDeltaUp )
		{
			ControllerRot.Pitch = ( Rotation.Pitch + MaxPitchDeltaUp ) & 65535;
		}
		else if ( RightOfCenter == 0 && Diff > MaxPitchDeltaDown )
		{
			ControllerRot.Pitch = ( Rotation.Pitch - MaxPitchDeltaDown ) & 65535;
		}		

		Gunner.Controller.SetRotation( ControllerRot );
	}
	GetAxes( CameraRotation, X,Y,Z );
	CameraLocation = CameraLocation - CameraOffset.X*X + CameraOffset.Y*Y + CameraOffset.Z*Z;

	CameraRotation.Pitch -= CameraPitchUU;
}

//---------------------------------------------------------
//	SetBlownUp
//---------------------------------------------------------
function SetBlownUp()
{
	Super.SetBlownUp();

	SetDrawType( DT_StaticMesh );
	SetStaticMesh( StaticMesh(DynamicLoadObject("s_military_de.Weapon.pak36_dmg", class'StaticMesh')) );

	if ( Gunner != None )
	{
		Gunner.Destroy();
	}

	if ( Loader != None )
	{
		Loader.Destroy();
	}
}

//---------------------------------------------------------
//	BlowUp
//---------------------------------------------------------
function BlowUp( Pawn EventInstigator )
{
	Super.BlowUp( EventInstigator );

	SetDrawType( DT_StaticMesh );
	SetStaticMesh( StaticMesh(DynamicLoadObject("s_military_de.Weapon.pak36_dmg", class'StaticMesh')) );

	HurtRadius( 500.0,500.0, class'DamageTankCannon', 50000, Location - 100.0*Vector( Rotation ) );

	if ( DeathEvent != '' )
	{
		TriggerEvent( DeathEvent, Self, EventInstigator );
	}

	// If the crew is still alive, get them off this thing.
	if ( Gunner != None && !Gunner.IsDead() )
	{
		ForceCrewDismount( EventInstigator );
	}
}

//---------------------------------------------------------
//	CanBeDestroyedBy
//---------------------------------------------------------
function bool CanBeDestroyedBy( class<DamageType> DamageType ) 
{ 
	return (
		DamageType == class'DamageGammonBomb' || 
		DamageType == class'DamageTankCannon' ||
		DamageType == class'DamageDEPanzerfaust' ||
		DamageType == class'DamageUSBazooka' 
		);
}

//---------------------------------------------------------
//	NeverAllowShotProtectionFor
//---------------------------------------------------------
function bool NeverAllowShotProtectionFor( class<DamageType> DamageType )
{
	return (
		CanBeDestroyedBy( DamageType ) ||
		DamageType == class'DamageGrenade' 
		);
}

function bool TargetIsInKillZone( Pawn Target )
{
	return VSize( Target.Location - Self.Location ) <= KillZoneDist;
}

simulated event RenderOverlays( Canvas Canvas ) {}

//=========================================================
//	Default Properties
//=========================================================
defaultproperties
{
	DrawType=DT_Mesh
//	Mesh=Mesh'k_prop.de_pak36'
//	StaticMesh=StaticMesh's_military_de.Weapon.pak36_dmg'
	Mesh=None
	StaticMesh=None
	Texture=Texture'Engine.S_Weapon'

	GunnerOffset=(x=-104.022,y=-44.898,z=38.447)	// Landon's original
	LoaderOffset=(x=-102.999,y=55.599,z=42.412)
	YawRotSpeed=1800.0
	PitchRotSpeed=1800.0
	PitchOffset=0
	MaxPitchDeltaDown=1000
	MaxPitchDeltaUp=2000
	//MaxYawDelta=4096
	MaxYawDelta=1820
	MaxYawModelDelta=900
	GunBone=gun_base

	FireModeClass(0)=FireDEPak36_
	FireModeClass(1)=FireDEPak36_
	AttachmentClass=class'AttachmentDEPak36_'

	ClientState=WS_Hidden
	AmmoClipSize=1

	EnemyTooCloseDist=1500.0
	MountOffset=(X=-104.022,Y=-44.898,Z=0)
	CameraStartOffset=(X=100,Y=0,Z=100)
	CameraEndOffset=(X=150,Y=40,Z=80)

	MyMountAction=Mount_MountPak36Gunner
	CameraPitchUU=1000
	CameraZoomSpeed=2.0
	CameraZoomOutSpeed=0.8

	Physics=PHYS_Falling
	bStatic=false
	bBlockKarma=true
	bCollideActors=true
	bBlockActors=true
	bBlockPlayers=true
	bCollideWorld=true
	bCollideSkeletalMesh=true
	bBlockZeroExtentTraces=true
	bBlockNonZeroExtentTraces=true
	CollisionHeight=0.0

	bUseShotProtection=true
	NoShotProtectionDist=1000.0
	MaxFiringDist=10000

	BreatheProfileClass=class'gbxSupportedBreatheProfile'

	CrewCheckInterval=3.0
	bDamagesVehicles=true
	
	bOpaqueForSituationalAwarenessView=true
}