//=============================================================================
// WeapDEPanzerfaust
//=============================================================================
class WeapDEPanzerfaust extends WargameWeapon
    config(user);


var Cue				FireSoundCue;
var bool			bDropCasingOnSwitch;

var ShellDEPanzerfaustCasing casing;

simulated function PostBeginPlay()
{
	FireSoundCue = LoadSound( "W_PANZERFAUST.FIRE" );

	Super.PostBeginPlay();
}

simulated function LoadWeaponMesh()
{
	Super.LoadWeaponMesh();
	
	Mesh = Mesh( DynamicLoadObject( "k_view_model_de.view_panzerfaust", class'Mesh' ) );
	m_NormalViewMesh = Mesh( DynamicLoadObject( "k_view_model_de.view_panzerfaust", class'Mesh' ) );
	m_ZoomViewMesh = Mesh( DynamicLoadObject( "k_view_model_de.view_panzerfaust_crosshair", class'Mesh' ) );
	LinkMesh( Mesh );
}

function ConsumeAmmo(int Mode, float load)
{
	Super.ConsumeAmmo(Mode, load);
	
	if( !HasAmmo() )
	{
		bDropCasingOnSwitch = true;
	}
}

simulated function AnimEnd(int channel)
{
	Super.AnimEnd(channel);

	if(ClientState == WS_ReadyToFire && Instigator.IsHumanControlled() && Level.NetMode == NM_Standalone )
    {
		SpawnCasing();
	}
}

simulated function SpawnCasing()
{
	if( casing == None && Instigator.IsHumanControlled() )
	{
		casing = Spawn( class'ShellDePanzerfaustCasing', None,,,Rotation + rot(0,0,49152) );
		casing.bHidden = true;
	}

	casing.SetLocation( GetBoneCoords( 'bolt' ).Origin + ( Vector(Instigator.GetViewRotation()) * 32.0 >> Rot(0,16384,0) ) );
	casing.SetBase( self );
}

simulated function ThrowCasing()
{
	if( casing != None )
	{
		casing.bHidden = false;
		casing.ThrowCasing( Instigator );
		casing = None;
	}
}

simulated function Timer()
{
	Super.Timer();

	if ( Level.NetMode == NM_Standalone )
	{
		if( bDropCasingOnSwitch && ClientState == WS_Hidden  )
		{
			ThrowCasing();
			bDropCasingOnSwitch = false;
		}
		else if( ClientState == WS_ReadyToFire )
		{
			SpawnCasing();
		}
	}
}

function byte BestMode()
{
	return 0;
}

function bool IsRocketLauncher()
{
	return true;
}

function bool RequiresSpecialMind( out class<gbxAIController> RequiredFC )
{
	RequiredFC = class<gbxAIController>( DynamicLoadObject( "gbxAI.MindPanzerfaust", class'Class' ) );
	return true;
}

defaultproperties
{
    ItemName="German Panzerfaust"

    FireModeClass(0)=FireDEPanzerfaust
    FireModeClass(1)=FireSecondaryItem
    FireModeClass(2)=FireMelee
    InventoryGroup=2
//	Mesh=Mesh'K_view_model_de.view_panzerfaust'
//	m_NormalViewMesh=Mesh'K_view_model_de.view_panzerfaust'
//	m_ZoomViewMesh=Mesh'K_view_model_de.view_panzerfaust_crosshair'
	m_nCharacterBodyIdx=3
    BobDamping=0.5
    PickupClass=class'PickupWeapDEPanzerfaust'
    EffectOffset=(X=130.0,Y=23.0,Z=-20.0)
	DisplayFOV=+50.0
    AttachmentClass=class'AttachmentDEPanzerfaust'

    DrawScale=1.0
	PlayerViewOffset=(X=20.000,Y=0.000000,Z=-0.50000)
    SelectSound=None

	AIRating=+0.4
    CurrentRating=0.4

	m_nBulletIconTextureX=258
	m_nBulletIconTextureY=203
	m_nClipIconTextureX=258
	m_nClipIconTextureY=203

	m_eWeaponType=WeaponPrimary
	MyMeleeDamageType=class'DamageUSGunButtMelee'

	DefaultAmmoCount(0)=1

	ZoomInAnim=aim_up
	ZoomOutAnim=aim_down
	SelectAnim=deploy
	PutDownAnim=holster
	IdleAnim(0)=idle_01
	IdleAnim(1)=idle_02
	IdleAnim(2)=idle_03
	BreathingAnim=breathe

	AmmoClipSize=1

	m_flEncumberance=1.0
	m_fCanSprint=true

	BreatheProfileClass=class'gbxHeavyBreatheProfile'

	bDamagesVehicles=true
	m_fDestroyWhenEmpty=true

	nIconX=358
	nIconY=0
	nIconW=27
	nIconH=108

	SelectAnimReadyDelay = 2.3
	PutDownAnimReadyDelay = 1.5

	DEPawnPackGoreClass(0)=class'gbxInventory.ConscriptK98GoreSetup'
	DEPawnPackGoreClass(1)=class'gbxInventory.InfantryK98GoreSetup'
	DEPawnPackGoreClass(2)=class'gbxInventory.PanzerK98GoreSetup'
	DEPawnPackGoreClass(3)=class'gbxInventory.ParaSturmGoreSetup'

	bThrowGrenadeLeftHanded=true

	WeaponName = WN_DEPanzerfaust
	bMustSwitchWhenEmpty = true
}
