//=============================================================================
// US Stuart Tank turret-mounted machine gun.
//=============================================================================
class WeapUSStuart30cal extends WeapUSUsableMachineGun
	config(User);


simulated function LoadWeaponMesh()
{
	Super.LoadWeaponMesh();
	Mesh = Mesh( DynamicLoadObject( "k_view_model_us.view_30cal_crosshair", class'Mesh' ) );
	LinkMesh( Mesh );
}

//---------------------------------------------------------
//	Tick
//---------------------------------------------------------
function Tick( float DeltaTime )
{
	local gbxMuzzleFlash MuzFlash;

	Super.Tick( DeltaTime );

	if (MyGunner == None || bIsDismounting)
	{
		if ((FireMode[0] != None) && FireMode[0].bIsFiring)
			StopFiringWeapon();

		return;
	}

	// Hide the muzzle flash if in first-person mode because we only want 
	// to render it during PostRender().
	MuzFlash = gbxMuzzleFlash( FireUSStuart30cal( FireMode[0] ).GetMuzzleFlash() );

	if ( m_fZoomed )
		MuzFlash.bHidden = true;
	else
		MuzFlash.bHidden = false;

	if ( FireMode[0] != None )
	{
		if ( FireMode[0].bIsFiring && !bWeaponFiring && !bIsDismounting )
		{
			StartFiringWeapon();
		}
		else if ( !FireMode[0].bIsFiring && bWeaponFiring )
		{
			StopFiringWeapon();
		}
	}
}

function vector GetPlayerMountPoint(gbxPawn gbxP)
{
	local vector MountPoint;
	local vector RotX, RotY, RotZ;

	GetAxes(MyTankPawn.Rotation, RotX, RotY, RotZ);

	if (bIsRearTrigger)
	{
		gbxP.SetTurretAnims( 'Stuart_player_mount_gun', 'Stuart_player_dismount_gun' );
		MountPoint = MyTankPawn.Location + (MountAnimStartOffset.X * RotX + MountAnimStartOffset.Y * RotY + MountAnimStartOffset.Z * RotZ);
	}
	else
	{
		gbxP.SetTurretAnims( 'Stuart_player_remount_gun', 'Stuart_player_dismount_gun' );
		MountPoint = MyTankPawn.Location + (GunnerOffsetStanding.X * RotX + GunnerOffsetStanding.Y * RotY + GunnerOffsetStanding.Z * RotZ);
	}

	return MountPoint;
}

//---------------------------------------------------------
//	GetAimRot
//---------------------------------------------------------
function Rotator GetAimRot()
{
	return FireUSStuart30cal(FireMode[0]).AimRot;
}

//---------------------------------------------------------
//	StartFiringWeapon
//---------------------------------------------------------
function StartFiringWeapon()
{
	if (bAimingAtFriendly && Instigator.IsA('gbxPawn'))
	{
		bFiringAtFriendly = true;
	}
	
	if ( bWeaponFiring || bIsDismounting )
		return;

	if (bIsMounting || bIsDismounting)
		return;

	// RSC: dont fire if we are issuing a command
	if( !Instigator.Controller.AllowFire() )
		return;

	if ( m_fZoomed )
	{
		FireLoopSoundHandle = PlaySound( StereoFireLoopSound, SLOT_None, TransientSoundVolume,,,,false);
	}
	else
	{
		FireLoopSoundHandle = PlaySound( FireLoopSound, SLOT_WeaponFire1, TransientSoundVolume,,,,true);
	}

	if ( FireMode[0].IsA( 'FireUSStuart30cal' ) )
		FireUSStuart30cal( FireMode[0] ).StartFiringWeapon();

	if ( m_fZoomed )
		LoopAnim( 'Fire', 1.0, 0.0 );

	Super.StartFiringWeapon();

	bWeaponFiring = true;
}

//---------------------------------------------------------
//	StopFiringWeapon
//---------------------------------------------------------
function StopFiringWeapon()
{
	if ( FireMode[0].IsA( 'FireUSStuart30cal' ) )
		FireUSStuart30cal( FireMode[0] ).StopFiringWeapon();

	if ( !bWeaponFiring )
		return;

	if (FireLoopSoundHandle != 0)
	{
		StopSound( FireLoopSoundHandle );
		FireLoopSoundHandle = 0;
	}

	if ( m_fZoomed )
	{
		PlaySound( StereoFireOneBulletSound, SLOT_NONE, TransientSoundVolume,,,,false);
	}
	else
	{
		PlaySound( FireOneBulletSound, SLOT_WeaponFire1, TransientSoundVolume,,,,true);
	}

	if ( m_fZoomed )
		PlayAnim( 'Idle', 1.0 );

	Super.StopFiringWeapon();

	bWeaponFiring = false;
}

//---------------------------------------------------------
//	FlashMuzzleFlash
//---------------------------------------------------------
// For 3rd person only.  See PostRender() in this class for 1st person muzzle flashes.
function FlashMuzzleFlash()
{
	local gbxMuzzleFlash MuzFlash;

	if ( MyGunner == None )
		return;

	if (bIsMounting || bIsDismounting)
		return;

	if ( FireMode[0].IsA( 'FireUSStuart30cal' ) )
	{
		MuzFlash = gbxMuzzleFlash( FireUSStuart30cal( FireMode[0] ).GetMuzzleFlash() );

		if ( m_fZoomed )
		{
			MuzFlash.bHidden = true;		// PostRender() will unhide it and render it.
			MuzFlash.Flash();
		}
		else
		{
			MuzFlash.SetLocation( GetMuzzleLocationWorld() );
			MuzFlash.SetRotation( Rotation );
			MuzFlash.Flash();
		}
	}
}

//---------------------------------------------------------
//	GetAttachment
//---------------------------------------------------------
function AttachmentUSStuart30cal GetAttachment()
{
	return AttachmentUSStuart30cal( ThirdPersonActor );
}

//---------------------------------------------------------
//	PostRender
//---------------------------------------------------------
function PostRender( Canvas TheCanvas )
{
	local gbxMuzzleFlash MuzFlash;

	if ( m_fZoomed )
	{		
		MuzFlash = gbxMuzzleFlash( FireUSStuart30cal( FireMode[0] ).GetMuzzleFlash() );
		if ( MuzFlash != None )
		{
			MuzFlash.bHidden = false;
			MuzFlash.SetLocation( GetMuzzleLocationWorld() );			
			MuzFlash.SetRotation( Rotation );			
			TheCanvas.DrawActor( MuzFlash, 50 );			
		}
		TheCanvas.DrawActor(self, DisplayFOV);		
		
		CalcOffsetWeaponCollision();
	}	
}

function bool AlwaysAutoReload()
{
	return true;
}

//=========================================================
//	Default Properties
//=========================================================
defaultproperties
{
	DrawType=DT_Mesh
//	Mesh=Mesh'k_view_model_us.view_30cal_crosshair'
	ItemName="US Stuart 30cal"

	GunBoneName="30cal"

	MyMountAction=Mount_MountTankStuart30cal
	MyDismountAction=Mount_DismountTankStuart30cal

	FireModeClass(0)=FireUSStuart30cal
	FireModeClass(1)=FireUSStuart30cal
	AttachmentClass=class'AttachmentUSStuart30cal'

	DefaultAmmoCount(0)=400

	// Nine-point blending animations.
	PawnForwardDownAnim="stuart_forward_down"
	PawnForwardLevelAnim="stuart_forward_level"
	PawnForwardUpAnim="stuart_forward_up"
	PawnLeftDownAnim="stuart_left_down"
	PawnLeftLevelAnim="stuart_left_level"
	PawnLeftUpAnim="stuart_left_up"
	PawnRightDownAnim="stuart_right_down"
	PawnRightLevelAnim="stuart_right_level"
	PawnRightUpAnim="stuart_right_up"

	FireLoopSoundName="W_30CAL.Loop"
	FireOneBulletSoundName="W_30CAL.End"
	StereoFireLoopSoundName="W_30CAL.LOOP_ST"
	StereoFireOneBulletSoundName="W_30CAL.END_ST"

	MaxRotSpeed=800.0

	GunnerOffset=(X=-92,Y=68,Z=80)
	GunnerOffsetStanding=(X=-88,Y=68,Z=120)

	MountAnimStartOffset=(X=-220.0,Y=142.0,Z=-40.0)

	CameraRearZoomTime=3.0
	CameraTurretZoomTime=1.0
	CameraTurretZoomHeadTime=1.4
	CameraRearStartOffset=(X=-200.0,Y=0.0,Z=300.0)
	CameraTurretStartOffset=(X=-40.0,Y=0.0,Z=0.0)
	CameraEndOffset=(X=-400,Y=0.0,Z=200.0)

	CameraOffsetZoomed=(X=-38.5,Y=0.1,Z=0.92)

	HorizCosTheta=0.5  // 40 degrees  //Bueno
	VertCosTheta=0.4067  // 66 degrees

	HorizFieldUU=7281
	VertFieldUU=4369
	ThirdPersonHorizFieldUU=12000
	ThirdPersonVertFieldUpUU=5461
	ThirdPersonVertFieldDownUU=5461
	CameraPitchUU=2000

	ClientState=WS_Hidden

	AmmoClipSize=400

	MuzzleOffset=(X=50,Y=0,Z=0)
	MuzzleOffset3rdPerson=(X=100,Y=0,Z=0)
}
