//=============================================================================
// US turret-mounted machine gun usable by players
//=============================================================================
class WeapUSUsableMachineGun extends TurretWeapon;

//---------------------------------------------------------
//	Variables / Constants
//---------------------------------------------------------
var gbxContextUseTrigger	MyRearTrigger;		// The context-use trigger used to get on/off this weapon.
var gbxContextUseTrigger	MyTurretTrigger;	// The context-use trigger used to get on/off this weapon.
var bool		bIsRearTrigger;				// keep track of which trigger was used (rear or turret)
var vector		BaseVector;
var vector		GunnerOffset;				// Used to position the gunner mesh.
var vector		GunnerOffsetStanding;

var vector		CameraRearStartOffset;		// relative to the mount point location
var vector		CameraTurretStartOffset;	// relative to the mount point location
var	vector		CameraSweepStartOffset;		// relative to the tank's Location
var float		CameraStartPitch;

var vector		CameraEndOffset;			// relative to the gunner's Location
var vector		CameraSweepEndOffset;		// relative to the tank's Location
var float		CameraEndPitch;

var float		CameraZoomTime;				// how long the camera takes to move from CameraSweepStartOffset to CameraSweepEndOffset
var float		CameraRearZoomTime;
var float		CameraTurretZoomTime;
var float		CameraTurretZoomHeadTime;
var bool		bCameraNeedsToPause;
var bool		bCameraIsPaused;
var float		CameraPausedTime;

var vector		CameraOffset;
var vector		CameraOffsetZoomed;
var bool		bIsMounting;
var bool		bIsDismounting;
var bool		bWeaponFiring;
var float		MountStartStamp;
var float		MaxRotSpeed;
var float		LastTimeSecs;
var gbxPawn		MyGunner;
var gbxPawn		MyTankPawn;

var vector	MountAnimStartOffset;
var vector	MuzzleOffset;
var vector	MuzzleOffset3rdPerson;

var float	CrossHairNextDrawTime, CrosshairDrawFreq;

var rotator	MyPreviousFacingRot;
var float	HorizCosTheta, VertCosTheta;
var rotator PreviousTankRotation;
var float	RotRate;
var float	LastDeltaTime;

var const int HorizFieldUU;
var const int VertFieldUU;
var const int ThirdPersonHorizFieldUU;
var const int ThirdPersonVertFieldUpUU;
var const int ThirdPersonVertFieldDownUU;
var const int CameraPitchUU;


// The names of the animations to use nine-point blending with
// when using this weapon.
var Name	PawnForwardDownAnim, PawnForwardLevelAnim, PawnForwardUpAnim,
			PawnLeftDownAnim, PawnLeftLevelAnim, PawnLeftUpAnim,
			PawnRightDownAnim, PawnRightLevelAnim, PawnRightUpAnim;

var name	TankForwardLevelAnim;
var name	TankIdleClosedAnim;

var name	GunBoneName;

enum VerticalFacing		{VF_Up, VF_Center, VF_Down};
enum HorizontalFacing	{HF_Left, HF_Center, HF_Right};

var AttachedPart ExtendedAttachment;  // attached skeletalmesh for the machine gun (so we can hide it in "iron sights" view)
var mesh ExtendedAttachmentMesh;

var bool	PreviousGodMode;
var bool	bMatchStarted;

const NINEPOINT_CHANNEL0 = 0;
const NINEPOINT_CHANNEL1 = 17;
const NINEPOINT_CHANNEL2 = 18;


//---------------------------------------------------------
//	PostNetBeginPlay
//---------------------------------------------------------
function PostNetBeginPlay()
{
	Super.PostNetBeginPlay();

	GiveAmmo(0);

	if ( ThirdPersonActor == None )
	{
		ThirdPersonActor = Spawn(AttachmentClass,Owner);		
		InventoryAttachment(ThirdPersonActor).InitFor(self);
	}

	if ( FireLoopSoundName != "" )
		FireLoopSound = Sound( DynamicLoadObject( FireLoopSoundName, class'Sound' ) );
	if ( FireOneBulletSoundName != "" )
		FireOneBulletSound = Sound( DynamicLoadObject( FireOneBulletSoundName, class'Sound' ) );
	if ( StereoFireLoopSoundName != "" )
		StereoFireLoopSound = Sound( DynamicLoadObject( StereoFireLoopSoundName, class'Sound' ) );
	if ( StereoFireOneBulletSoundName != "" )
		StereoFireOneBulletSound = Sound( DynamicLoadObject( StereoFireOneBulletSoundName, class'Sound' ) );
}

//---------------------------------------------------------
//	SetMount - used to hard attach this weapon to some other actor
//---------------------------------------------------------
function SetMount(gbxPawn NewMount)
{
	MyTankPawn = NewMount;
	SetBase(MyTankPawn);

	BaseVector = Vector( MyTankPawn.Rotation );
}

function SetExtendedAttachment(AttachedPart ExtAttach)
{
	ExtendedAttachment = ExtAttach;
	ExtendedAttachmentMesh = ExtAttach.Mesh;
}

function SetTriggers(gbxContextUseTrigger CtxRearUseTrigger, gbxContextUseTrigger CtxTurretUseTrigger)
{
	MyRearTrigger = CtxRearUseTrigger;
	MyTurretTrigger = CtxTurretUseTrigger;
}

function SetAnims(name ForwardLevelAnim, name IdleClosedAnim)
{
	TankForwardLevelAnim = ForwardLevelAnim;
	TankIdleClosedAnim = IdleClosedAnim;
}

function MatchStarting()
{
	bMatchStarted = true;
}

//---------------------------------------------------------
//	IsInUse
//---------------------------------------------------------
function bool IsInUse()
{
	// The gun is in use if it has a gunner.
	return (MyGunner != None);
}

//---------------------------------------------------------
//	GetBestBurstLength
//---------------------------------------------------------
function GetBestBurstLength( ESuppressionLevel SuppressionLevel, out float MinLen, out float MaxLen )
{
	FireMode[0].GetBestBurstLength( SuppressionLevel, MinLen, MaxLen );
}

//---------------------------------------------------------
//	Tick
//---------------------------------------------------------
function Tick( float DeltaTime )
{
	if (bIsDismounting && (FireMode[0] != None) && FireMode[0].bIsFiring)
		StopFiringWeapon();

	Super.Tick( DeltaTime );

	LastDeltaTime = DeltaTime;
}

function ActivateOracle()
{
	if ( ExtendedAttachment.Mesh == None )  // is the 3rd person weapon currently hidden?
	{
		m_fZoomed = false;
		ExtendedAttachment.Mesh = ExtendedAttachmentMesh;  // un-hide the weapon
	}
}

function int GetRotDiff(int A, int B)
{
	local int comp;

	comp = (A - B) & 65535;
	if (comp > 32768)
		comp -= 65536;

	return comp;
}

function rotator LerpRotator( rotator R1, rotator R2, int Delta)
{
	local int PitchDiff, YawDiff, RollDiff;
	local rotator OutRot;

	PitchDiff = GetRotDiff(R1.Pitch, R2.Pitch);
	if (PitchDiff > Delta)
		PitchDiff = Delta;
	else if (PitchDiff < -Delta)
		PitchDiff = -Delta;

	YawDiff = GetRotDiff(R1.Yaw, R2.Yaw);
	if (YawDiff > Delta)
		YawDiff = Delta;
	else if (YawDiff < -Delta)
		YawDiff = -Delta;

	RollDiff = GetRotDiff(R1.Roll, R2.Roll);
	if (RollDiff > Delta)
		RollDiff = Delta;
	else if (RollDiff < -Delta)
		RollDiff = -Delta;

	OutRot = R1;
	OutRot.Pitch = (OutRot.Pitch - PitchDiff) & 65535;
	OutRot.Yaw = (OutRot.Yaw - YawDiff) & 65535;
	OutRot.Roll = (OutRot.Roll - RollDiff) & 65535;

	return OutRot;
}

//---------------------------------------------------------
//	FaceRotation
//---------------------------------------------------------
// Make the turret weapon rotate to face a particular direction.
function FaceRotation( Rotator Rot )
{
	// OK, now the aim is to end up with two alpha values for
	// animation channels 1 and 2.  These channels, along with
	// base channel 0, will be blended to produce a smooth transitioning
	// of the gunner's pose as the user moves the mouse.
	//
	// Channel 0 plays the 'level-forward' gunning animation at all times.
	// Channel 1 plays an 'edge animation' (i.e. 'level-left', 'level-right',
	// 'up-forward', or 'down-forward').  Channel 2 plays a 'diagonal animation'
	// (i.e. 'up-left', 'up-right', 'down-left', or 'down-right').
	//
	// The blending occurs progressively; that is, the edge animation is
	// blended over the base animation first, and then the diagonal animation
	// is blended over that result in turn.
	//
	local Vector Aim, AimXY, AimXZ, D, V;
	local Float DotH, DotV, Alpha1, Alpha2, m;//, TempPitch, TempLastPitch;
	local HorizontalFacing HorizFace;
	local VerticalFacing VertFace;
	local NinePointDir DirDiag, DirEdge;
	local int PitchDiff, YawDiff;
	local Vector RotX, RotY, RotZ;

	// Sanity check.
	if (MyGunner == None)
	{
		log("Error: WeapUSUsableMachineGun Gunner is null!", 'ScriptWarning');
		return;
	}

	if (bMatchStarted)  // was a saved game reloaded?
	{
		bMatchStarted = false;

		MyGunner.PlayAnim( PawnForwardLevelAnim,,, NINEPOINT_CHANNEL0 );
	}

	MyGunner.SetRotation( MyTankPawn.Rotation );

	GetAxes(MyTankPawn.Rotation, RotX, RotY, RotZ);

	if ( !m_fZoomed )
	{
		Rot = LerpRotator( MyPreviousFacingRot, Rot, MaxRotSpeed );
	}

	MyPreviousFacingRot = Rot;

	// Restrict Yaw
	YawDiff = GetRotDiff(Rot.Yaw, MyTankPawn.Rotation.Yaw);
	if (YawDiff > HorizFieldUU)
		Rot.Yaw = (MyTankPawn.Rotation.Yaw + HorizFieldUU) & 65535;
	else if (YawDiff < -HorizFieldUU)
		Rot.Yaw = (MyTankPawn.Rotation.Yaw - HorizFieldUU) & 65535;

	// Restrict Pitch
	PitchDiff = GetRotDiff(Rot.Pitch, MyTankPawn.Rotation.Pitch);
	if (PitchDiff > VertFieldUU)
		Rot.Pitch = (MyTankPawn.Rotation.Pitch + VertFieldUU) & 65535;
	else if (PitchDiff < -VertFieldUU)
		Rot.Pitch = (MyTankPawn.Rotation.Pitch - VertFieldUU) & 65535;

	SetRotation(Rot);


	// Next, we'll calculate DotH and DotV again, only using the gun's actual
	// current orientation (due to the motion-delay placed on the gun, the
	// actual orientation may differ from the requested orientation.
	Aim = Vector(Rotation);
	AimXZ = Aim - ((Aim dot RotY) * RotY);
	AimXY = Aim - ((Aim dot RotZ) * RotZ);
	AimXZ = Normal(AimXZ);
	AimXY = Normal(AimXY);

	DotH = AimXY dot RotY;
	DotV = AimXZ dot RotZ;

	// Remap the dot-prods into the space -1.0 to 1.0.
	DotH /= HorizCosTheta;
	DotV /= VertCosTheta;

	DotH = FClamp( DotH, -1.0, 1.0 );
	DotV = FClamp( DotV, -1.0, 1.0 );

	// Determine what quadrant the aim vector is in.
	if (DotV < 0)
		VertFace = VF_Down;
	else
		VertFace = VF_Up;

	//if (DotH < 0)
	if (DotH > 0)
		HorizFace = HF_Right;
	 else
		HorizFace = HF_Left;


	// Absolute-value the dot prods so that we can simplify calculations below.
	DotH = abs(DotH);
	DotV = abs(DotV);

	// From the quadrant information, determine which two
	// diagonal and edge animations to use.
	if (VertFace == VF_Down) {
		if (HorizFace == HF_Left) {
			DirDiag = NPD_LowLeft;
			if (DotH >= DotV)
				DirEdge = NPD_MidLeft;
			else
				DirEdge = NPD_LowCenter;
		} else {
			DirDiag = NPD_LowRight;
			if (DotH >= DotV)
				DirEdge = NPD_MidRight;
			else
				DirEdge = NPD_LowCenter;
		}
	} else {	// VF_Up
		if (HorizFace == HF_Left) {
			DirDiag = NPD_UpLeft;
			if (DotH >= DotV)
				DirEdge = NPD_MidLeft;
			else
				DirEdge = NPD_UpCenter;
		} else {
			DirDiag = NPD_UpRight;
			if (DotH >= DotV)
				DirEdge = NPD_MidRight;
			else
				DirEdge = NPD_UpCenter;
		}
	}

	// Step 4: Figure the alpha blending values to use for the
	// two channels.  Point D is the point in 2D space that we found
	// above.  Point V is a point of intersection between the line thru
	// D and the nearest corner with the x- or the y-axis.
	//
	// Alpha1 is found by finding the distance from the origin to V.
	// Alpha2 is the distance between V and D.
	//
	// By absolute-valuing the dot prods above, we reduce the problem
	// to quadrant 1 (at this point, we have the two animations we need.
	// Except for knowing whether our edge-animation is horizontal or vertical,
	// quadrant information is no longer relevant.
	//
	D.X = DotH;	// 0.0 to 1.0, no negative values.
	D.Y = DotV;	// 0.0 to 1.0, no negative values.
	D.Z = 0;

	if (D.X == 1.0)	{	// Avoid divide-by-zero errors in the slope formula.
		Alpha1 = 1.0;
		Alpha2 = D.Y;
	} else {
		// m is the slope of the line through D and the corner (1,1).
		m = (1.0 - D.Y) / (1.0 - D.X);

		if (DirEdge == NPD_MidRight || DirEdge == NPD_MidLeft) {

			// The edge-animation is horizontal, so find x when
			// the line intercepts the x-axis (when y=0).
			Alpha1 = D.X - (D.Y / m);	// -b/m in intercept-slope formula.
			V.x = Alpha1;
			V.y = 0;
			V.z = 0;

		} else {	// Vertical edge (NPD_UpCenter or NPD_LowCenter).
			// The edge-animation is vertical, so find y when
			// the line intercepts the y-axis (when x=0).
			Alpha1 = D.Y - (m * D.X);	// 'b' in intercept-slope formula.
			V.x = 0;
			V.y = Alpha1;
			V.z = 0;
		}

		// Alpha2 is the proportion of the length of vector D - V
		// to that of vector (1,1) - V.  The equation used below
		// is the optimized form of the one in comments.
		Alpha2 = VSize(D - V) / (1.41421 - 0.41421 * Alpha1);	// 0.41421 == sqrt(2)
		//Alpha2 = VSize(D - V) / VSize(Vect(1,1,0) - V);		// Keep this for reference.
	}

	// Play the gunning animations, supplying the edge-animation,
	// the diagonal-animation, and their blending factors.
	MyGunner.DoNinePointBlending( DirEdge, Alpha1, DirDiag, Alpha2 );
	MyTankPawn.DoNinePointBlending( DirEdge, Alpha1, DirDiag, Alpha2 );
}


function vector GetPlayerMountPoint(gbxPawn gbxP)
{
	Log(name $ ".GetMountPoint() - YOU FORGOT TO OVERRIDE THIS FUNCTION!!!");
	return vect(0,0,0);
}

//---------------------------------------------------------
//	PreparePawnToMount
//---------------------------------------------------------
function PreparePawnToMount(gbxPawn gbxP)
{
	local vector RotX, RotY, RotZ;
	local vector MountPoint;
	local vector v1, v2;
	local vector TankPawnLocation;

	gbxP.bDontFrustumCull = true;  // need to do this when tank is at steep angles otherwise player mesh gets culled out while climb anim is playing

	gbxP.bBlockZeroExtentTraces = false;
	if (gbxP.Controller != None)
	{
		PreviousGodMode = gbxP.Controller.bGodMode;  // save this so we can restore it later
		gbxP.Controller.bGodMode = true;
		gbxPlayerController( gbxP.Controller ).EnableHeavyTrauma( false );
	}

	gbxP.ResetRootRotation();  // don't twist the Pawn's lower body

	gbxP.SetCollision(false, false, false);  // jlb-hack for collision during pawn placement for mount animation
	gbxP.bCollideWorld = false;  // you HAVE to do this in case the tank is on a sleeply sloped terrain (terrain higher at right side of tank)

	MountPoint = GetPlayerMountPoint( gbxP );

	gbxP.SetLocation( MountPoint );
	gbxP.SetPhysics(PHYS_None);  // this MUST happen after the SetLocation call since it can change to PHYS_Falling

	gbxP.SetRotation( MyTankPawn.Rotation );
	gbxP.Controller.SetRotation( MyTankPawn.Rotation );

	// hard attach player pawn to tank so that animation plays properly if tank is moving...
	gbxP.bHardAttach = true;
	gbxP.SetBase(MyTankPawn);

	gbxP.PrepareToMount();

	GetAxes(MyTankPawn.Rotation, RotX, RotY, RotZ);

	TankPawnLocation = MyTankPawn.GetBoneCoords('chassis').origin;

	MyTankPawn.bFriendlyFireListen = false;

	if (bIsRearTrigger)
	{
		CameraZoomTime = CameraRearZoomTime;
		CameraSweepStartOffset = MountPoint +
							(CameraRearStartOffset.X * RotX + CameraRearStartOffset.Y * RotY + CameraRearStartOffset.Z * RotZ) -
							TankPawnLocation;
		// convert the offset vector from world space to local space (relative to the tank's location)
		CameraSweepStartOffset = CameraSweepStartOffset << MyTankPawn.Rotation;
	}
	else
	{
		CameraZoomTime = CameraTurretZoomTime;
		CameraSweepStartOffset = MountPoint +
							(CameraTurretStartOffset.X * RotX + CameraTurretStartOffset.Y * RotY + CameraTurretStartOffset.Z * RotZ) -
							TankPawnLocation;
		// convert the offset vector from world space to local space (relative to the tank's location)
		CameraSweepStartOffset = CameraSweepStartOffset << MyTankPawn.Rotation;
		CameraSweepStartOffset = CameraSweepStartOffset + vect(0,0,79);  // offset by eye height
	}

	v1 = TankPawnLocation + (GunnerOffset.X * RotX + GunnerOffset.Y * RotY + GunnerOffset.Z * RotZ);
	CameraSweepEndOffset = v1 + (CameraEndOffset.X * RotX + CameraEndOffset.Y * RotY + CameraEndOffset.Z * RotZ) - TankPawnLocation;
	CameraSweepEndOffset = CameraSweepEndOffset << MyTankPawn.Rotation;

	v1 = TankPawnLocation + (CameraSweepStartOffset.X * RotX + CameraSweepStartOffset.Y * RotY + CameraSweepStartOffset.Z * RotZ);
	v2 = MountPoint + vect(0,0,79);  // offset by eye height
	CameraStartPitch = rotator(v2 - v1).Pitch;

	CameraEndPitch = -CameraPitchUU;

	bIsMounting = true;

	bCameraIsPaused = false;
	bCameraNeedsToPause = false;

	MyGunner = gbxP;
}


//---------------------------------------------------------
//	CompleteMount
//---------------------------------------------------------
function CompleteMount( gbxPawn gbxP )
{
	local int Mode;
	local Vector RotX, RotY, RotZ;
	local int count;
	local vector NewLoc, OffsetVec;
	local bool result;
	local vector TankPawnLocation;

	Instigator = MyGunner;
	if ( Ammo[0] != None )
		Ammo[0].Instigator = MyGunner;
	if ( FireMode[0] != None )
		FireMode[0].Instigator = Instigator;
	if ( ThirdPersonActor != None )
		ThirdPersonActor.Instigator = Instigator;

	gbxP.bDontFrustumCull = false;  // shouldn't need this anymore now that player is completely on tank

	gbxP.SetNinePointAnimations(
		PawnForwardDownAnim, PawnForwardLevelAnim, PawnForwardUpAnim,
		PawnLeftDownAnim, PawnLeftLevelAnim, PawnLeftUpAnim,
		PawnRightDownAnim, PawnRightLevelAnim, PawnRightUpAnim);

	// skeletalmesh will be stuck in last frame of mounting animation, play 0 frames of "idle" animation before blending begins...
	gbxP.PlayAnim( PawnForwardLevelAnim,,, NINEPOINT_CHANNEL0 );

	gbxP.Acceleration = vect(0,0,0);
	gbxP.Velocity = vect(0,0,0);
	gbxP.bRotateToDesired = false;
	gbxP.DesiredRotation = MyTankPawn.Rotation;

	gbxP.bBlockZeroExtentTraces = true;
	if (gbxP.Controller != None)
	{
		gbxP.Controller.bGodMode = PreviousGodMode;
		gbxPlayerController( gbxP.Controller ).EnableHeavyTrauma( true );
	}

	TankPawnLocation = MyTankPawn.GetBoneCoords('chassis').origin;

	gbxP.bHardAttach = false;
	gbxP.SetBase(None);

	// have to do this because Pawn.Physics == PHYS_None and APawn::performPhysics() won't run
	gbxP.ForceCrouch();

	// player HAS to collide here or they can't use the trigger to get off of the gun...
	gbxP.SetCollision(true, true, true);
	gbxP.bCollideWorld = true;

	MyGunner.SetRotation( MyTankPawn.Rotation );
	MyGunner.Controller.SetRotation( MyTankPawn.Rotation );

	GetAxes(MyTankPawn.Rotation, RotX, RotY, RotZ);

	count = 0;
	NewLoc = TankPawnLocation + (GunnerOffset.X * RotX + GunnerOffset.Y * RotY + GunnerOffset.Z * RotZ);
	OffsetVec = vect(0,0,1) + RotX;  // up and back
	result = gbxP.SetLocation(NewLoc);
	while ((!result) && (count < 20))
	{
		// keep pushing the player up and out slightly until they don't collide!!!
		count++;
		NewLoc = NewLoc + OffsetVec;
		result = gbxP.SetLocation(NewLoc);
	}

	gbxP.SetPhysics(PHYS_None);

	gbxP.bHardAttach = true;
	gbxP.SetBase(MyTankPawn);

	gbxP.bInterpolating = false;  // allow the player to rotate the camera again

	SetRotation(MyTankPawn.Rotation);

	MyPreviousFacingRot = MyTankPawn.Rotation;

	// make sure the gun is facing the same direction as the player's controller...
	FaceRotation( gbxP.Controller.Rotation );

	PreviousTankRotation = MyTankPawn.Rotation;

	bIsMounting = false;

	for( Mode = 0; Mode < NUM_FIRE_MODES; Mode++ )
	{
		if (FireMode[Mode] != None)
		{
//				FireMode[Mode].InitEffects();
			FireMode[Mode].GetMuzzleFlash();
		}
	}

	MyTankPawn.PlayAnim(TankForwardLevelAnim);

	gbxP.SetWeapon( Self );

	// gbxWeapon AnimEnd transitions ClientState from WS_BringUp to WS_ReadyToFire, since this weapon has no deploy animation, we set WS_ReadyToFire ourselves.
	ClientState = WS_ReadyToFire;

	bAimingAtFriendly = false;
}

//---------------------------------------------------------
//	MountFailed
//---------------------------------------------------------
function MountFailed( gbxPawn gbxP )
{
	bIsMounting = false;
	gbxP.bHardAttach = false;
	gbxP.SetBase( None );
	gbxP.SetPhysics(PHYS_Walking);
	gbxP.SetCollision(true, true, true);
	gbxP.bCollideWorld = true;
	gbxP.bInterpolating = false;
	gbxP.bBlockZeroExtentTraces = true;	

	if (gbxP.Controller != None)
	{
		gbxP.Controller.bGodMode = PreviousGodMode;
		gbxPlayerController( gbxP.Controller ).EnableHeavyTrauma( true );
	}

	MyGunner = None;

	MyTankPawn.bFriendlyFireListen = true;

	// Reactivate the trigger.
	MyRearTrigger.bDontDisplayMessage = false;
	MyTurretTrigger.bDontDisplayMessage = false;
}

//---------------------------------------------------------
//	PreparePawnToDismount
//---------------------------------------------------------
function PreparePawnToDismount( gbxPawn gbxP )
{
	local vector RotX, RotY, RotZ;
	local vector MountPoint;
	local vector v1, v2;
	local vector TankPawnLocation;

	// Check that the pawn passed in is the gunner.
	if (gbxP != MyGunner)
		return;

	StopFiringWeapon();

	gbxP.bHardAttach = false;
	gbxP.SetBase(None);

	SetRotation(Rot(0,0,0));

	if (gbxP.bUseRootMotion)
		gbxP.SetPhysics(PHYS_RootMotion);
	else
		gbxP.SetPhysics(PHYS_Walking);

	GetAxes(MyTankPawn.Rotation, RotX, RotY, RotZ);

	TankPawnLocation = MyTankPawn.GetBoneCoords('chassis').origin;

	MountPoint = TankPawnLocation + (GunnerOffsetStanding.X * RotX + GunnerOffsetStanding.Y * RotY + GunnerOffsetStanding.Z * RotZ);

	CameraZoomTime = CameraTurretZoomHeadTime;

	CameraSweepEndOffset = MountPoint +
							(CameraTurretStartOffset.X * RotX + CameraTurretStartOffset.Y * RotY + CameraTurretStartOffset.Z * RotZ) -
							TankPawnLocation;
	// convert the offset vector from world space to local space (relative to the tank's location)
	CameraSweepEndOffset = CameraSweepEndOffset << MyTankPawn.Rotation;
	CameraSweepEndOffset = CameraSweepEndOffset + vect(0,0,79);  // offset by eye height

	v1 = TankPawnLocation + (GunnerOffset.X * RotX + GunnerOffset.Y * RotY + GunnerOffset.Z * RotZ);
	CameraSweepStartOffset = v1 + (CameraEndOffset.X * RotX + CameraEndOffset.Y * RotY + CameraEndOffset.Z * RotZ) - TankPawnLocation;
	CameraSweepStartOffset = CameraSweepStartOffset << MyTankPawn.Rotation;

	v1 = TankPawnLocation + (CameraSweepEndOffset.X * RotX + CameraSweepEndOffset.Y * RotY + CameraSweepEndOffset.Z * RotZ);
	v2 = MountPoint + vect(0,0,79);  // offset by eye height
	CameraEndPitch = rotator(v2 - v1).Pitch;

	CameraStartPitch = -CameraPitchUU;

	SetFirstPersonZoom( false );

	gbxP.AnimBlendParams( NINEPOINT_CHANNEL1, 0.0, 0, 0 );
	gbxP.AnimBlendParams( NINEPOINT_CHANNEL2, 0.0, 0, 0 );
	gbxP.PlayWaiting();
	gbxP.PlayMoving();

	bIsDismounting = true;
}

//---------------------------------------------------------
//	CompleteDismount
//---------------------------------------------------------
function CompleteDismount( gbxPawn gbxP )
{
	if ( gbxP == MyGunner )
	{
		Instigator = None;
		Ammo[0].Instigator = None;
		FireMode[0].Instigator = None;

		MyGunner = None;
		bIsDismounting = false;
		CameraOffset.X = 0.0;
		CameraOffset.Z = 0.0;

		gbxP.SetRotation( MyTankPawn.Rotation );
		gbxP.Controller.SetRotation( MyTankPawn.Rotation );

		// Reactivate my trigger, which was made inactive when used.
		MyRearTrigger.bDontDisplayMessage = false;
		MyTurretTrigger.bDontDisplayMessage = false;

		MyTankPawn.bFriendlyFireListen = true;

		MyTankPawn.AnimBlendParams( NINEPOINT_CHANNEL1, 0.0, 0, 0 );
		MyTankPawn.AnimBlendParams( NINEPOINT_CHANNEL2, 0.0, 0, 0 );
		MyTankPawn.PlayAnim(TankIdleClosedAnim);

		gbxP.Weapon = None;  // hack so SetWeapon() will call ChangedWeapon() - jlb
		gbxP.SetWeapon( gbxP.LastWeapon );
	}
}

//---------------------------------------------------------
//	GunnerDied
//---------------------------------------------------------
function GunnerDied( gbxPawn gbxP )
{
	if ( gbxP == MyGunner )
	{
		SetRotation(Rot(0,0,0));

		MyGunner.bCollideWorld = true;
//		MyGunner.SetLocation( GetMyMountPoint() );

		MyGunner = None;

		// Reactivate my trigger, which was made inactive when used.
		MyRearTrigger.bDontDisplayMessage = false;
		MyTurretTrigger.bDontDisplayMessage = false;
	}
}


//---------------------------------------------------------
//	IsFacingRightDirection
//---------------------------------------------------------
function bool IsFacingRightDirection( gbxPawn gbxP )
{
	local Rotator RotToWeapon;
	local int YawDiff;
	local bool bResult;

	RotToWeapon = Rotator(Location-gbxP.Location);

	RotToWeapon.Yaw = RotToWeapon.Yaw & 65535;

	YawDiff = abs(RotToWeapon.Yaw - (gbxP.Rotation.Yaw & 65535) );

	if (YawDiff >= 32767)
	{
		YawDiff = (65536 - YawDiff);
	}

	bResult = (YawDiff < 100);

	return bResult;
}


//---------------------------------------------------------
//	Trigger
//---------------------------------------------------------
event Trigger(Actor Other, Pawn EventInstigator)
{
	local gbxPawn gbxP;

	gbxP = gbxPawn( EventInstigator );

	// Skip the action instigator is in the middle of a reload.
	if ( gbxP.IsTransitioning() || (gbxP.Weapon != Self && gbxP.Weapon.IsReloading() ) )
		return;

	if (Other == MyRearTrigger)
		bIsRearTrigger = true;
	else
		bIsRearTrigger = false;

	if ( MyGunner == gbxP )	// Gunner wishes to dismount.
	{
		MyGunner.DismountTurretPlayer(false);
		MountStartStamp=Level.TimeSeconds;
	}
	else if ( MyGunner == None )  // Gunner wishes to mount.
	{
		MyRearTrigger.bDontDisplayMessage = true;
		MyTurretTrigger.bDontDisplayMessage = true;

		MyTankPawn.PrepareToMount();

		if ( gbxP.Controller.bIsHumanPlayer )
		{
			if (gbxP.Weapon != None)
			{
				if (gbxP.Weapon.ThirdPersonActor != None)
					gbxP.Weapon.ThirdPersonActor.bHidden = true;  // hide the weapon quickly
			}

			PreparePawnToMount( gbxP );

			gbxP.MountTurretPlayer( Self, false );

			MountStartStamp=Level.TimeSeconds;
		}
	}
}

function ConsumeAmmo(int Mode, float load)
{
}

simulated event RenderOverlays( Canvas Canvas )
{
}

simulated function DrawCrossHair( canvas Canvas )
{
	local float XLength, YLength;
	local PlayerController PlayerOwner;
	local vector HitLoc, HitNorm;
	local Actor HitActor;
	local HUD	Hud;

	if ( m_fZoomed )
		return;

	if (bIsMounting || bIsDismounting )
		return;

	if ( Instigator != None && Instigator.IsHumanControlled() )
	{
		PlayerOwner = PlayerController(Instigator.Controller);
			
		if ( PlayerOwner.DisableCrosshair() )
			return;	

		if ( PlayerOwner.iCrossHairIndex == 0)
			return;

		Hud = PlayerOwner.MyHud;

		if (Hud == None)
		{
			XLength = 32;
			YLength = 32;
		}
		else
		{
			XLength = 32 * Hud.flMinScaleX;
			YLength = 32 * Hud.flMinScaleY;
		}

		if ( Level.TimeSeconds >= CrosshairNextDrawTime )
		{
			HitActor = Trace( HitLoc, HitNorm, Location + 15000 * Vector( Rotation ), Location, false );
			
			if ( HitActor != None )
			{
				Canvas.Style = ERenderStyle.STY_Additive;
				Canvas.SetDrawColor(255,255,255,96);
				Canvas.DrawTile3D( m_TestCrosshairs[PlayerOwner.iCrossHairIndex - 1], HitLoc, XLength, YLength, 0, 0, m_TestCrosshairs[PlayerOwner.iCrossHairIndex - 1].USize, m_TestCrosshairs[PlayerOwner.iCrossHairIndex - 1].VSize);
			}

			CrossHairNextDrawTime = Level.TimeSeconds + CrosshairDrawFreq;
		}
	}		
}

simulated function DrawHud(canvas Canvas, gbxHud Hud, FontInfo Fonts, int nPass)
{
}

//---------------------------------------------------------
//	AnimEnd
//---------------------------------------------------------
simulated function AnimEnd(int channel)
{
	if ( m_fZoomed )
	{
		if ( bWeaponFiring )
		{
			//PlayAnim( 'Fire', 1.0 );
		}			
		else
		{
			PlayAnim( 'Idle', 1.0 );
		}
	}
	else
		Super.AnimEnd( channel );
}

//---------------------------------------------------------
//	Fire
//---------------------------------------------------------
simulated function Fire(float F)
{
	if( bIsDismounting )
		return;

	StartFiringWeapon();

	Super.Fire(F);	
}

//---------------------------------------------------------
//	PlayFiringAnim
//---------------------------------------------------------
simulated function PlayFiringAnim()
{
	// Deprecate. 
}

//---------------------------------------------------------
//	SetFirstPersonZoom
//---------------------------------------------------------
function SetFirstPersonZoom( bool bZoomed )
{
	local PlayerController PC;

	// don't change zoom settings if currently in OracleCam (SA View) mode...
	if( Level.Game.bOracleCamActive )
		return;

	m_fZoomed = bZoomed;

	if ( MyGunner == None || !MyGunner.IsA( 'gbxPlayerPawn' ) )
		return;

	if ( bZoomed )
	{
		ExtendedAttachment.Mesh = None;

		PC = PlayerController( MyGunner.Controller );
		PC.DesiredFOV = DisplayFOV;
	}
	else
	{
		ExtendedAttachment.Mesh = ExtendedAttachmentMesh;

		PC = PlayerController( MyGunner.Controller );
		PC.DesiredFOV = PC.DefaultFOV;
	}
}

//---------------------------------------------------------
//	CalcThirdPersonView
//---------------------------------------------------------
function CalcThirdPersonView( gbxPawn thePawn, out actor ViewActor, out vector CameraLocation, out rotator CameraRotation )
{
	local Vector RotX, RotY, RotZ;
	local Rotator ControllerRot;
	local int PitchDiff;
	local float T;
	local vector TankRotX, TankRotY, TankRotZ;
	local vector v1, v2;
	local vector TankPawnLocation;
	local rotator RotDelta;

	ViewActor = thePawn;

	GetAxes( MyTankPawn.Rotation, TankRotX, TankRotY, TankRotZ );

	TankPawnLocation = MyTankPawn.GetBoneCoords('chassis').origin;

	if ( bIsMounting || bIsDismounting )
	{
		// Player is mounting turret.  Zoom out.
		if (bCameraIsPaused)
			T = FClamp( ( CameraPausedTime - MountStartStamp ) / CameraZoomTime, 0.0, 1.0 );
		else
			T = FClamp( ( Level.TimeSeconds - MountStartStamp ) / CameraZoomTime, 0.0, 1.0 );

		v1 = TankPawnLocation + (CameraSweepStartOffset.X * TankRotX + CameraSweepStartOffset.Y * TankRotY + CameraSweepStartOffset.Z * TankRotZ);
		v2 = TankPawnLocation + (CameraSweepEndOffset.X * TankRotX + CameraSweepEndOffset.Y * TankRotY + CameraSweepEndOffset.Z * TankRotZ);
		CameraLocation = v1 + T*(v2 - v1);

		CameraRotation.Pitch = (CameraStartPitch + T*(CameraEndPitch - CameraStartPitch)) & 65535;
		CameraRotation.Yaw = MyTankPawn.Rotation.Yaw;

		if (bCameraNeedsToPause)
		{
			bCameraIsPaused = true;
			bCameraNeedsToPause = false;

			CameraPausedTime = Level.TimeSeconds;
		}
	}
	else
	{
		CameraOffset = CameraEndOffset;
	}

	// Restrict the yaw and pitch of the player's controller...
	if ( MyGunner != None )
	{
		RotDelta = PreviousTankRotation - MyTankPawn.Rotation;  // delta is negative change since previous frame

		ControllerRot = MyGunner.Controller.Rotation;
		ControllerRot.Pitch = (ControllerRot.Pitch + RotDelta.Pitch) & 65535;
		ControllerRot.Yaw = (ControllerRot.Yaw + RotDelta.Yaw) & 65535;
		ControllerRot.Roll = 0;  // always reset the roll to 0 to make the iron sights view "level"

		PreviousTankRotation = MyTankPawn.Rotation;

/*
		// Restrict Yaw
		YawDiff = GetRotDiff(ControllerRot.Yaw, MyTankPawn.Rotation.Yaw);
		if (YawDiff > ThirdPersonHorizFieldUU)
			ControllerRot.Yaw = (MyTankPawn.Rotation.Yaw + ThirdPersonHorizFieldUU) & 65535;
		else if (YawDiff < -ThirdPersonHorizFieldUU)
			ControllerRot.Yaw = (MyTankPawn.Rotation.Yaw - ThirdPersonHorizFieldUU) & 65535;
*/

		// Restrict Pitch
		PitchDiff = GetRotDiff(ControllerRot.Pitch, MyTankPawn.Rotation.Pitch);
		if (PitchDiff > ThirdPersonVertFieldUpUU)
			ControllerRot.Pitch = (MyTankPawn.Rotation.Pitch + ThirdPersonVertFieldUpUU) & 65535;
		else if (PitchDiff < -ThirdPersonVertFieldDownUU)
			ControllerRot.Pitch = (MyTankPawn.Rotation.Pitch - ThirdPersonVertFieldDownUU) & 65535;

		MyGunner.Controller.SetRotation( ControllerRot );

		if ( !bIsMounting && !bIsDismounting )
		{
			CameraRotation = ControllerRot;

			GetAxes( CameraRotation, RotX, RotY, RotZ );
			CameraLocation = thePawn.Location + (CameraOffset.X * RotX + CameraOffset.Y * RotY + CameraOffset.Z * RotZ);

			CameraRotation.Pitch -= CameraPitchUU;
		}
	}
}

//---------------------------------------------------------
//	CalcFirstPersonView
//---------------------------------------------------------
function CalcFirstPersonView( gbxPawn thePawn, out actor ViewActor, out vector CameraLocation, out rotator CameraRotation )
{
	local Vector RotX, RotY, RotZ;
	local Rotator ControllerRot;
	local int PitchDiff, YawDiff;
	local rotator RotDelta;

	ViewActor = Self;

	CameraOffset.X = CameraOffsetZoomed.X;
	CameraOffset.Z = CameraOffsetZoomed.Z;

	GetAxes( Self.Rotation, RotX, RotY, RotZ );
	CameraLocation = Self.Location + (CameraOffset.X*RotX + CameraOffset.Z*RotZ);

	// Restrict the yaw to the cone of fire.
	if ( MyGunner != None )
	{
		RotDelta = PreviousTankRotation - MyTankPawn.Rotation;  // delta is negative change since previous frame

		ControllerRot = MyGunner.Controller.Rotation;
		ControllerRot.Pitch = (ControllerRot.Pitch + RotDelta.Pitch) & 65535;
		ControllerRot.Yaw = (ControllerRot.Yaw + RotDelta.Yaw) & 65535;
		ControllerRot.Roll = 0;  // always reset the roll to 0 to make the iron sights view "level"

		PreviousTankRotation = MyTankPawn.Rotation;

		// Yaw.
		YawDiff = GetRotDiff(ControllerRot.Yaw, MyTankPawn.Rotation.Yaw);
		if (YawDiff > HorizFieldUU)
			ControllerRot.Yaw = (MyTankPawn.Rotation.Yaw + HorizFieldUU) & 65535;
		else if (YawDiff < -HorizFieldUU)
			ControllerRot.Yaw = (MyTankPawn.Rotation.Yaw - HorizFieldUU) & 65535;

		// Pitch.
		PitchDiff = GetRotDiff(ControllerRot.Pitch, MyTankPawn.Rotation.Pitch);
		if (PitchDiff > VertFieldUU)
			ControllerRot.Pitch = (MyTankPawn.Rotation.Pitch + VertFieldUU) & 65535;
		else if (PitchDiff < -VertFieldUU)
			ControllerRot.Pitch = (MyTankPawn.Rotation.Pitch - VertFieldUU) & 65535;

		CameraRotation = ControllerRot;

		MyGunner.Controller.SetRotation( ControllerRot );

		MyPreviousFacingRot = ControllerRot;
	}
}

//---------------------------------------------------------
//	SpecialCalcView
//---------------------------------------------------------
simulated function SpecialCalcView( gbxPawn thePawn, out actor ViewActor, out vector CameraLocation, out rotator CameraRotation )
{
	if ( m_fZoomed )
		CalcFirstPersonView( thePawn, ViewActor, CameraLocation, CameraRotation );
	else
		CalcThirdPersonView( thePawn, ViewActor, CameraLocation, CameraRotation );
}

//---------------------------------------------------------
//	GetFireRotation
//---------------------------------------------------------
function Rotator GetFireRotation()
{
	return Rotation;
}

simulated function Vector GetMuzzleLocationWorld()
{
	local rotator r_temp;
	local vector RotX, RotY, RotZ;
	local vector BoneLoc;

	if ( !m_fZoomed )
	{
		r_temp = GetFireRotation();
		GetAxes(r_temp, RotX, RotY, RotZ);
		BoneLoc = MyTankPawn.GetBoneCoords(GunBoneName).origin;
		return BoneLoc + (MuzzleOffset3rdPerson.X * RotX) + (MuzzleOffset3rdPerson.Y * RotY) + (MuzzleOffset3rdPerson.Z * RotZ);
	}
	else
	{
		GetAxes(Rotation, RotX, RotY, RotZ);
		return Location + (MuzzleOffset.X * RotX) + (MuzzleOffset.Y * RotY) + (MuzzleOffset.Z * RotZ);
	}
}

//---------------------------------------------------------
//	ClientStartFire
//---------------------------------------------------------
// Needed to override this because Owner is different for this weapon than most other weapons.
simulated event ClientStartFire(int Mode)
{
    if (!MyGunner.Controller.CanFire() || bIsDismounting )
        return;

	if (bAimingAtFriendly && Instigator.IsA('gbxPawn'))
	{
		bFiringAtFriendly = true;
	}

    if (Role < ROLE_Authority)
    {
        if (StartFire(Mode))
        {
            ServerStartFire(Mode);
        }
    }
    else
    {
        StartFire(Mode);
    }
}

//---------------------------------------------------------
//	AdjustAimNew
//---------------------------------------------------------
// Used only with player pawns; AI pawns use the AdjustAimNew() in FiringController.
function Rotator AdjustAimNew( Ammunition Ammo, Vector ProjStart, Rotator AimRotator, int WeaponAimError )
{
	local float FinalErrorMax;

	FinalErrorMax = WeaponAimError;

	// Yaw.
	if ( Rand(2) == 0 )
		AimRotator.Yaw += FRand() * FinalErrorMax;
	else
		AimRotator.Yaw -= FRand() * FinalErrorMax;

	// Pitch
	if ( Rand(2) == 0 )
		AimRotator.Pitch += FRand() * FinalErrorMax;
	else
		AimRotator.Pitch -= FRand() * FinalErrorMax;

	return AimRotator;
}

//---------------------------------------------------------
//	CanSpawnShellCasing
//---------------------------------------------------------
function bool CanSpawnShellCasing()
{
	return ( MyGunner.IsHumanControlled() && !m_fZoomed );
}


//---------------------------------------------------------
//	PlayZoomIn
//---------------------------------------------------------
simulated function PlayZoomIn()
{
	SetFirstPersonZoom( true );
}

//---------------------------------------------------------
//	PlayZoomOut
//---------------------------------------------------------
simulated function PlayZoomOut()
{
	SetFirstPersonZoom( false );
}

//---------------------------------------------------------
//	IsZoomed
//---------------------------------------------------------
event bool IsZoomed()
{
	return m_fZoomed;
}

//---------------------------------------------------------
//	CanZoom
//---------------------------------------------------------
simulated function bool CanZoom()
{
	return true;
}

//---------------------------------------------------------
//	GetBaseVector
//---------------------------------------------------------
function Vector GetBaseVector()
{
	return BaseVector;
}

//---------------------------------------------------------
//	NotifyForceDismount
//---------------------------------------------------------
function NotifyForceDismount( optional bool bDoNothingIfHumanControlled )
{
	if (MyGunner == None)
		return;

	if ( MyGunner.Controller.IsA( 'gbxAIController' ) )
	{
		gbxAIController( MyGunner.Controller ).BreakOutJustMe();
	}
	else if ( MyGunner.Controller.bIsHumanPlayer && !bDoNothingIfHumanControlled )
	{
		MyGunner.DismountTurretPlayer(false);
	}
}


//=========================================================
//	Default Properties
//=========================================================
defaultproperties
{
	bHidden=true
	DisplayFOV=+35.0
	bAcceptsProjectors=false
	DrawScale=1.0

	BreatheProfileClass=class'gbxSupportedBreatheProfile'

	bMatchStarted=false;

	RotRate=16384  // in Unreal rotators per second

	CrosshairDrawFreq=0.01
}