class gbxMortarShell extends gbxExplosive;


const BAZOOKA_ROCKET_BURN_DIST=11520;

var Vector	FireVector;
var bool	bHitWater;
var bool	bActive;
var bool	bSoundStarted;
var vector	Dir, TargetLoc;
var float	m_flLaunchTime;
var float	m_flTravelDist;
var float	TimeBeforeDropToPlaySound;
var Cue		MortarFlightSound;

var float	ShellDelay;

simulated function PostNetBeginPlay()
{
	Super.PostNetBeginPlay();
	
	SetStaticMesh(StaticMesh(DynamicLoadObject("s_military_de.Weapon.mortar_round", class'StaticMesh')));
}

simulated function BeginPlay()
{
	if ( (Level.DetailMode != DM_Low) && !Level.bDropDetail )
		SmokeRate = 0.03;
	else
		SmokeRate = 0.15;
}

simulated function PhysicsVolumeChange( PhysicsVolume NewVolume )
{
	if ( !NewVolume.bWaterVolume || bHitWater )
		return;

	bHitWater = True;
	Spawn(class'WaterSplashLarge',,,,rot(16384,0,0));			//-BB
	Velocity=0.6*Velocity;
}

simulated function ProcessTouch( actor Other, vector HitLocation )
{
	if (!bActive)
		return;

	if ( Other!=instigator )
	{
		Explosion(HitLocation);
		bActive = false;
	}
}

simulated function HitWall( vector HitLocation, vector HitNormal, actor Wall )
{
	local Projector Decal;

	if (!bActive)
		return;

	Velocity = Vect( 0, 0, 0 );
	
	if ( Role == ROLE_Authority )
	{
		if ( Mover(Wall) != None )
			Wall.TakeDamage( Damage, instigator, Location, MomentumTransfer * Normal(Velocity), MyDamageType);

		MakeNoise(1.0);
	}

	Explosion(Location + ExploWallOut * HitNormal);

	if ( (ExplosionDecal != None) && (Level.NetMode != NM_DedicatedServer) )
	{
		Decal = Spawn(ExplosionDecal,self,,Location, rotator(-HitNormal));
	}
}

function ServerExplosion(vector HitLocation)
{
	Explosion( HitLocation );
}

simulated function Destroyed()
{
	Super.Destroyed();
}

simulated function Tick(float DeltaTime)
{
	Super.Tick( DeltaTime );
	
	m_flTravelDist = m_flTravelDist + VSize( Velocity * DeltaTime );
	if (m_flTravelDist > BAZOOKA_ROCKET_BURN_DIST)
	{
		Velocity.X = Velocity.X * (1.0f - (0.1 * DeltaTime));
		Velocity.Y = Velocity.Y * (1.0f - (0.1 * DeltaTime));
	}
	Velocity.Z = Velocity.Z - (150 * DeltaTime);
}

simulated function Timer()
{
	Super.Timer();
	// Make the shell disappear and go dormant until notified to teleport.
	Velocity = Vect(0,0,0);
	SetPhysics( PHYS_None );
	//bHidden = true;
}

function Launch( Vector NewTargetLoc, float NewShellDelay )
{
	Dir = vector(Rotation);
	Velocity = speed * Dir;
	if (PhysicsVolume.bWaterVolume)
	{
		bHitWater = True;
		Velocity=0.6*Velocity;
	}
	bActive = true;

	m_flLaunchTime = Level.TimeSeconds;

	TargetLoc = NewTargetLoc;
	ShellDelay = NewShellDelay;
	GotoState( 'Ascending' );
}

//=========================================================
//	State Ascending
//=========================================================
state Ascending 
{
	function BeginState()
	{
		SetTimer( 0.25, false );
	}

	function Timer()
	{
		Super.Timer();
		GotoState( 'WaitingForDrop' );
	}
}

//=========================================================
//	State Waiting
//=========================================================
state WaitingForDrop
{
	function BeginState()
	{
		bHidden = true;
		SetPhysics( PHYS_None );
		Velocity = Vect(0,0,0);
		
		if ( TimeBeforeDropToPlaySound < ShellDelay )
		{
			SetTimer( ShellDelay - TimeBeforeDropToPlaySound, false );			
		}
		else
		{
			bSoundStarted = true;
			PlayOwnedSoundCue( MortarFlightSound, SLOT_None );
			SetTimer( ShellDelay, false );			
		}
	}

	function Timer()
	{
		local Rotator NewRot;
		Super.Timer();
		//DHW+ -- Removed line as it was causing mortar shells to re-appear mid-air.
		//bHidden = false;
		//DHW-

		if ( bSoundStarted )
		{
			NewRot = Rotation;
			NewRot.Pitch = ( NewRot.Pitch - 2*NewRot.Pitch ) & 65535;
			SetRotation( NewRot );
			SetLocation( TargetLoc - 2000*Vector( NewRot ) );		
			SetPhysics( PHYS_Projectile );
			Velocity = speed * Vector( NewRot );
			bHidden = false;					
			GotoState( '' );
		}
		else
		{
			bSoundStarted = true;
			PlaySoundCue( MortarFlightSound, SLOT_None );
			SetTimer( TimeBeforeDropToPlaySound, false );
		}
	}
}

defaultproperties
{
	ExplosionDecal=class'BlastMark'
	MyDamageType=class'DamageUSBazooka'
    speed=6000.0
	MaxSpeed=6000.000000
	Damage=500.000000
    RemoteRole=ROLE_SimulatedProxy
    LifeSpan=30.0
	DamageRadius=375.000000	// GBX:naj - modified 4/5/04
	MomentumTransfer=50000
	Physics=PHYS_Projectile
//	StaticMesh=StaticMesh's_military_de.Weapon.mortar_round'
	DrawScale=1.00000
	AmbientGlow=64
	bBounce=false
    bCollideWorld=true
	bFixedRotationDir=True
	DrawType=DT_STATICMESH

    ShakeRadius=2000.000000
	ViewRollTime=0.200000
	RollMag=20.000000
	RollRate=100.000000
	OffsetMagVertical=20.000000
	OffsetRateVertical=800.000000
	OffsetMagHorizontal=20.000000
	OffsetRateHorizontal=800.000000
	OffsetIterations=10.000000

	bBlowingUp=false

	FuseTimeMin=5.0
	FuseTimeMax=6.0

	HeavyTraumaRadius=384.0
	TimeBeforeDropToPlaySound=2.75
	WeaponCategory=WC_Mortar

	bActive=false
	bDestroyWhenDone=true
}