//=============================================================================
// Base class for all grenades
//=============================================================================
class gbxTankShell extends gbxExplosive;


var Vector	FireVector;
var bool	bHitWater;
var bool	bActive;
var vector Dir;

//var	TankShellTrail trail;

var class<Projector> ExplosionDecalVehicle;


simulated function PostNetBeginPlay()
{
	Dir = vector(Rotation);
	Velocity = speed * Dir;
	if (PhysicsVolume.bWaterVolume)
	{
		bHitWater = True;
		Velocity=0.6*Velocity;
	}
	bActive = true;

//	if (ROLE != ROLE_Authority)
//	{
//		Trail = Spawn(class'TankShellTrail ',self);
//		Trail.SetBase(Self);
//	}
	
	Super.PostNetBeginPlay();

	SetStaticMesh(StaticMesh(DynamicLoadObject("s_military_us.tank_round_stuart", class'StaticMesh')));
}

simulated function BeginPlay()
{
	if ( (Level.DetailMode != DM_Low) && !Level.bDropDetail )
		SmokeRate = 0.03;
	else
		SmokeRate = 0.15;
}

simulated function PhysicsVolumeChange( PhysicsVolume NewVolume )
{
	if ( !NewVolume.bWaterVolume || bHitWater )
		return;

	bHitWater = True;
	Spawn(class'WaterSplashLarge',,,,rot(16384,0,0));			//-BB
	Velocity=0.6*Velocity;
}

simulated function ProcessTouch( actor Other, vector HitLocation )
{
	if (!bActive)
		return;

	if (Other != instigator)
	{
		Explosion(HitLocation);

		bActive = false;
	}
}

simulated function HitWall( vector HitLocation, vector HitNormal, actor Wall )
{
	local Projector Decal;
	local ProjectorRelative DecalRelative;
	local vector vHit, TraceEnd;
	local float Length;

	if (!bActive)
		return;

	Velocity = Vect( 0, 0, 0 );

	vHit = HitLocation - Location;
	Length = VSize(vHit);
	vHit = Normal(vHit);

	TraceEnd = Location + (vHit * (Length + 1.0f));

	if (TraceHitBackDrop(TraceEnd, Location))
	{
		Destroy();
		return;
	}

	if ( Role == ROLE_Authority )
	{
		if ( Mover(Wall) != None )
			Wall.TakeDamage( Damage, instigator, Location, MomentumTransfer * Normal(Velocity), MyDamageType);

		MakeNoise(1.0);
	}

	Explosion(Location + ExploWallOut * HitNormal);

	if (Wall.IsA('gbxSVehicle'))
	{
		if ( (ExplosionDecalVehicle != None) && (Level.NetMode != NM_DedicatedServer) )
		{
			Decal = Spawn(ExplosionDecalVehicle,self,,Location + HitNormal, rotator(-HitNormal));
			DecalRelative = ProjectorRelative(Decal);
			if (DecalRelative != None)
			{
			    DecalRelative.SetRelativeProjector(Wall, Location + HitNormal, rotator(-HitNormal));
			}
		}
	}
	else
	{
		if ( (ExplosionDecal != None) && (Level.NetMode != NM_DedicatedServer) )
		{
			Decal = Spawn(ExplosionDecal,self,,Location + ExploWallOut * HitNormal, rotator(-HitNormal));
		}
	}
}

function ServerExplosion(vector HitLocation)
{
	Explosion( HitLocation );
}

simulated function Destroyed()
{
//	if ( Trail != None )
//	{
//		Trail.Emitters[0].AutomaticInitialSpawning=False;
//		Trail.Emitters[0].RespawnDeadParticles=false;
//		Trail.Emitters[1].AutomaticInitialSpawning=False;
//		Trail.Emitters[1].RespawnDeadParticles=false;
//		Trail.Emitters[1].ParticlesPerSecond=0;
//		Trail.AutoReset=False;
//		Trail.AutoDestroy=True;
//	}
	
	Super.Destroyed();
}

simulated function HurtRadius( float DamageAmount, float DamageRadius, class<DamageType> DamageType, float Momentum, vector HitLocation )
{
	local actor Victim;
	local float damageScale, dist;
	local vector dir;
	local Pawn SomePawn;

	if( bHurtEntry )
		return;

	bHurtEntry = true;

	// For tank shells use 'RadiusActors()' instead of 'VisibleCollidingActors()' to slightly damage
	// Pawns that are behind nearby cover...
	foreach RadiusActors( class 'Actor', Victim, DamageRadius, HitLocation )
	{
		// don't let blast damage affect fluid - CollisingActors doesn't really work for them - jag
		if( (Victim != self) && (Victim.Role == ROLE_Authority) )
		{
			dir = Victim.Location - HitLocation;
			dist = FMax(1,VSize(dir));
			dir = dir/dist;
			damageScale = 1 - FMax(0,(dist - Victim.CollisionRadius)/DamageRadius);

			if (FastTrace( Victim.Location, HitLocation ))
			{
				Victim.TakeDamage
				(
					damageScale * DamageAmount,
					Instigator,
					Victim.Location - 0.5 * (Victim.CollisionHeight + Victim.CollisionRadius) * dir,
					(damageScale * Momentum * dir),
					DamageType
				);
			}
			else  // victim was only partially exposed (or completely hidden), gets 1/2 of blast...
			{
				SomePawn = Pawn(Victim);

				if (SomePawn != None)
				{
					if (SomePawn.PawnTeam == Team_US)
						damageScale = damageScale * 0.2f;  // US only gets damaged 20% of full amount
					else
						damageScale = damageScale * 0.5f;  // Germans (and TEAM_None) get damaged 50% of full amount
				}
				else
				{
					damageScale = damageScale * 0.5f;  // damage to something other than a Pawn
				}

				Victim.TakeDamage
				(
					damageScale * DamageAmount,
					Instigator,
					Victim.Location - 0.5 * (Victim.CollisionHeight + Victim.CollisionRadius) * dir,
					(damageScale * Momentum * dir),
					DamageType
				);
			}
		}
	}
	bHurtEntry = false;
}


defaultproperties
{
	ExplosionDecal=class'TankProjectileScorch'
	ExplosionDecalVehicle=class'TankProjectileScorchSmall'
	MyDamageType=class'DamageTankCannon'
    Speed=10000.0
	MaxSpeed=15000.000000
	Damage=500.000000
    RemoteRole=ROLE_SimulatedProxy
    LifeSpan=8.0
	DamageRadius=500.000000
	MomentumTransfer=50000
	Physics=PHYS_Projectile
//	StaticMesh=StaticMesh's_npc_weapon_us.us_rocket_projectile'
	StaticMesh=StaticMesh's_military_us.tank_round_stuart'
	DrawScale=1.00000
	AmbientGlow=64
	bBounce=false
    bCollideWorld=true
	bFixedRotationDir=True
	DrawType=DT_STATICMESH

	ExplosionMessageName="gbxAI.PerceptionTankShellExplosion"

    ShakeRadius=2000.000000
	ViewRollTime=0.200000
	RollMag=20.000000
	RollRate=100.000000
	OffsetMagVertical=20.000000
	OffsetRateVertical=800.000000
	OffsetMagHorizontal=20.000000
	OffsetRateHorizontal=800.000000
	OffsetIterations=10.000000

	bBlowingUp=false

	FuseTimeMin=5.0
	FuseTimeMax=6.0

	HeavyTraumaRadius=384.0
	
	bActive=false
	bDestroyWhenDone=true

	WeaponCategory=WC_TankShell

	ExploWallOut=31.0
}
